@@ -20,6 +20,7 @@
 from crispy_forms.helper import FormHelper
 from crispy_forms.layout import HTML, Div, Field, Fieldset, Layout, Submit
 from django import forms
+from django.conf import settings
 from django.contrib.auth import authenticate, password_validation
 from django.contrib.auth.forms import SetPasswordForm as DjangoSetPasswordForm
 from django.middleware.csrf import rotate_token
@@ -185,7 +186,13 @@ class ProfileForm(ProfileBaseForm):
 
     public_email = forms.ChoiceField(
         label=_("Public e-mail"),
-        choices=(("", ""),),
+        choices=[("", _("Do not publicly display e-mail"))],
+        required=False,
+    )
+
+    commit_email = forms.ChoiceField(
+        label=_("Commit e-mail"),
+        choices=[("", _("Use account e-mail"))],
         required=False,
     )
 
@@ -194,6 +201,7 @@ class ProfileForm(ProfileBaseForm):
         fields = (
             "website",
             "public_email",
+            "commit_email",
             "liberapay",
             "codesite",
             "github",
@@ -207,9 +215,18 @@ class ProfileForm(ProfileBaseForm):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         emails = get_all_user_mails(self.instance.user)
-        emails.add("")
 
-        self.fields["public_email"].choices = [(x, x) for x in sorted(emails)]
+        commit_emails = get_all_user_mails(self.instance.user, filter_deliverable=False)
+        site_commit_email = self.instance.get_site_commit_email()
+        if site_commit_email:
+            if not settings.PRIVATE_COMMIT_EMAIL_OPT_IN:
+                self.fields["commit_email"].choices = [("", site_commit_email)]
+            else:
+                commit_emails.add(site_commit_email)
+
+        self.fields["public_email"].choices += [(x, x) for x in sorted(emails)]
+        self.fields["commit_email"].choices += [(x, x) for x in sorted(commit_emails)]
+
         self.helper = FormHelper(self)
         self.helper.disable_csrf = True
         self.helper.form_tag = False
@@ -468,11 +485,10 @@ class CaptchaForm(forms.Form):
             self.generate_captcha()
             self.fresh = True
         else:
-            self.captcha = MathCaptcha.unserialize(request.session.pop("captcha"))
+            self.captcha = MathCaptcha.unserialize(request.session["captcha"])
+            self.set_label()
 
-    def generate_captcha(self):
-        self.captcha = MathCaptcha()
-        self.request.session["captcha"] = self.captcha.serialize()
+    def set_label(self):
         # Set correct label
         self.fields["captcha"].label = (
             pgettext(
@@ -482,6 +498,13 @@ class CaptchaForm(forms.Form):
             )
             % self.captcha.display
         )
+        if self.is_bound:
+            self["captcha"].label = self.fields["captcha"].label
+
+    def generate_captcha(self):
+        self.captcha = MathCaptcha()
+        self.request.session["captcha"] = self.captcha.serialize()
+        self.set_label()
 
     def clean_captcha(self):
         """Validation for CAPTCHA."""
@@ -505,10 +528,16 @@ class CaptchaForm(forms.Form):
             self.cleaned_data["captcha"],
         )
 
+    def cleanup_session(self, request):
+        del request.session["captcha"]
+
 
 class EmptyConfirmForm(forms.Form):
     def __init__(self, request, *args, **kwargs):
         self.request = request
+        self.user = request.user
+        if "user" in kwargs:
+            self.user = kwargs.pop("user")
         super().__init__(*args, **kwargs)
 
 
@@ -521,8 +550,8 @@ class PasswordConfirmForm(EmptyConfirmForm):
 
     def clean_password(self):
         cur_password = self.cleaned_data["password"]
-        if self.request.user.has_usable_password():
-            valid = self.request.user.check_password(cur_password)
+        if self.user.has_usable_password():
+            valid = self.user.check_password(cur_password)
         else:
             valid = cur_password == ""
         if not valid:
@@ -847,7 +876,9 @@ class GroupChoiceField(forms.ModelChoiceField):
 
 class GroupAddForm(forms.Form):
     add_group = GroupChoiceField(
-        label=_("Add user to a group"), queryset=Group.objects.order(), required=True
+        label=_("Add user to a group"),
+        queryset=Group.objects.prefetch_related("defining_project").order(),
+        required=True,
     )
 
     def __init__(self, *args, **kwargs):
