@@ -87,6 +87,9 @@ class WeblateAccountsConf(AppConf):
 
     CONTACT_FORM = "reply-to"
 
+    PRIVATE_COMMIT_EMAIL_TEMPLATE = "{username}@users.noreply.{site_domain}"
+    PRIVATE_COMMIT_EMAIL_OPT_IN = True
+
     # Auth0 provider default image & title on login page
     SOCIAL_AUTH_AUTH0_IMAGE = "auth0.svg"
     SOCIAL_AUTH_AUTH0_TITLE = "Auth0"
@@ -260,7 +263,7 @@ class AuditLogQuerySet(models.QuerySet):
 class AuditLog(models.Model):
     """User audit log storage."""
 
-    user = models.ForeignKey(User, on_delete=models.deletion.CASCADE)
+    user = models.ForeignKey(User, on_delete=models.deletion.CASCADE, null=True)
     activity = models.CharField(
         max_length=20,
         choices=[(a, a) for a in sorted(ACCOUNT_ACTIVITY.keys())],
@@ -316,8 +319,10 @@ class AuditLog(models.Model):
 
     def should_notify(self):
         return (
-            not self.user.is_bot
+            self.user is not None
+            and not self.user.is_bot
             and self.user.is_active
+            and self.user.email
             and self.activity in NOTIFY_ACTIVITY
         )
 
@@ -346,6 +351,7 @@ class AuditLog(models.Model):
 class VerifiedEmail(models.Model):
     """Storage for verified e-mails from auth backends."""
 
+    is_deliverable = models.BooleanField(default=True)
     social = models.ForeignKey(UserSocialAuth, on_delete=models.deletion.CASCADE)
     email = EmailField()
 
@@ -564,6 +570,12 @@ class Profile(models.Model):
         max_length=EMAIL_LENGTH,
     )
 
+    commit_email = EmailField(
+        verbose_name=_("Commit e-mail"),
+        blank=True,
+        max_length=EMAIL_LENGTH,
+    )
+
     class Meta:
         verbose_name = "User profile"
         verbose_name_plural = "User profiles"
@@ -698,14 +710,6 @@ class Profile(models.Model):
             return 2
         return 3
 
-    @cached_property
-    def watched_project_ids(self):
-        # We do not use values_list, because we prefetch this
-        return {watched.id for watched in self.watched.all()}
-
-    def watches_project(self, project):
-        return project.id in self.watched_project_ids
-
     def fixup_profile(self, request):
         fields = set()
         if not self.language:
@@ -753,6 +757,22 @@ class Profile(models.Model):
         if fields:
             self.save(update_fields=fields)
 
+    def get_commit_email(self) -> str:
+        email = self.commit_email
+        if not email and not settings.PRIVATE_COMMIT_EMAIL_OPT_IN:
+            email = self.get_site_commit_email()
+        if not email:
+            email = self.user.email
+        return email
+
+    def get_site_commit_email(self) -> str:
+        if not settings.PRIVATE_COMMIT_EMAIL_TEMPLATE:
+            return ""
+        return settings.PRIVATE_COMMIT_EMAIL_TEMPLATE.format(
+            username=self.user.username,
+            site_domain=settings.SITE_DOMAIN,
+        )
+
 
 def set_lang_cookie(response, profile):
     """Set session language based on user preferences."""
