@@ -92,6 +92,7 @@ class Notification:
     digest_template: str = "digest"
     filter_languages: bool = False
     ignore_watched: bool = False
+    any_watched: bool = False
     required_attr: Optional[str] = None
 
     def __init__(self, outgoing, perm_cache=None):
@@ -123,17 +124,26 @@ class Notification:
         result = Subscription.objects.filter(notification=self.get_name())
         if users is not None:
             result = result.filter(user_id__in=users)
-        query = Q(scope__in=(SCOPE_WATCHED, SCOPE_ADMIN, SCOPE_ALL))
+        query = Q(scope__in=(SCOPE_ADMIN, SCOPE_ALL))
+        # Special case for site-wide announcements
+        if self.any_watched and not project and not component:
+            query |= Q(scope=SCOPE_WATCHED)
         if component:
+            if not self.ignore_watched:
+                query |= Q(scope=SCOPE_WATCHED) & Q(
+                    user__profile__watched=component.project
+                )
             query |= Q(component=component)
         if project:
+            if not self.ignore_watched:
+                query |= Q(scope=SCOPE_WATCHED) & Q(user__profile__watched=project)
             query |= Q(project=project)
         if lang_filter:
             result = result.filter(user__profile__languages=translation.language)
         return (
             result.filter(query)
             .order_by("user", "-scope")
-            .prefetch_related("user__profile__watched")
+            .prefetch_related("user", "user__profile", "user__profile__watched")
         )
 
     def get_subscriptions(self, change, project, component, translation, users):
@@ -163,8 +173,8 @@ class Notification:
             return False
 
         if project.pk not in self.perm_cache:
-            self.perm_cache[project.pk] = User.objects.all_admins(project).values_list(
-                "pk", flat=True
+            self.perm_cache[project.pk] = set(
+                User.objects.all_admins(project).values_list("pk", flat=True)
             )
 
         return user.pk in self.perm_cache[project.pk]
@@ -204,13 +214,6 @@ class Notification:
                     subscription.scope == SCOPE_ADMIN
                     and not self.is_admin(user, project)
                 )
-                # Watched scope for not watched
-                or (
-                    subscription.scope == SCOPE_WATCHED
-                    and not self.ignore_watched
-                    and project is not None
-                    and not user.profile.watches_project(project)
-                )
             ):
                 continue
 
@@ -424,7 +427,7 @@ class MergeFailureNotification(Notification):
     def should_skip(self, user, change):
         fake = copy(change)
         fake.action = Change.ACTION_ALERT
-        fake.alert = Alert()
+        fake.alert = Alert(name="MergeFailure", details={"error": ""})
         if self.fake_notify is None:
             self.fake_notify = NewAlertNotificaton(None, self.perm_cache)
         return bool(
@@ -472,6 +475,16 @@ class ParseErrorNotification(Notification):
     verbose = _("Parse error")
     template_name = "parse_error"
 
+    def get_context(
+        self, change=None, subscription=None, extracontext=None, changes=None
+    ):
+        context = super().get_context(change, subscription, extracontext, changes)
+        if change:
+            context["details"]["filelink"] = change.component.get_repoweb_link(
+                change.details.get("filename"), "1", user=context["user"]
+            )
+        return context
+
 
 @register_notification
 class NewStringNotificaton(Notification):
@@ -663,6 +676,7 @@ class NewAnnouncementNotificaton(Notification):
     verbose = _("New announcement")
     template_name = "new_announcement"
     required_attr = "announcement"
+    any_watched: bool = True
 
     def should_skip(self, user, change):
         return not change.announcement.notify
@@ -677,12 +691,30 @@ class NewAlertNotificaton(Notification):
     required_attr = "alert"
 
     def should_skip(self, user, change):
-        if not change.component.linked_component or not change.alert.obj.link_wide:
-            return False
-        fake = copy(change)
-        fake.component = change.component.linked_component
-        fake.project = fake.component.project
-        return bool(list(self.get_users(FREQ_INSTANT, fake, users=[user.pk])))
+        if change.alert.obj.link_wide:
+            # Notify for main component
+            if not change.component.linked_component:
+                return False
+            # Notify only for others only when user will not get main.
+            # This handles component level subscriptions.
+            fake = copy(change)
+            fake.component = change.component.linked_component
+            fake.project = fake.component.project
+            return bool(list(self.get_users(FREQ_INSTANT, fake, users=[user.pk])))
+        if change.alert.obj.project_wide:
+            first_component = change.component.project.component_set.order_by(
+                "id"
+            ).first()
+            # Notify for the first component
+            if change.component.id == first_component.id:
+                return True
+            # Notify only for others only when user will not get first.
+            # This handles component level subscriptions.
+            fake = copy(change)
+            fake.component = first_component
+            fake.project = fake.component.project
+            return bool(list(self.get_users(FREQ_INSTANT, fake, users=[user.pk])))
+        return False
 
 
 class SummaryNotification(Notification):
