@@ -38,11 +38,12 @@ from weblate.accounts.utils import (
     cycle_session_keys,
     invalidate_reset_codes,
 )
-from weblate.auth.models import User, get_anonymous
+from weblate.auth.models import User
 from weblate.trans.defines import FULLNAME_LENGTH
 from weblate.utils import messages
+from weblate.utils.ratelimit import reset_rate_limit
 from weblate.utils.requests import request
-from weblate.utils.validators import USERNAME_MATCHER, clean_fullname
+from weblate.utils.validators import USERNAME_MATCHER, EmailValidator, clean_fullname
 
 STRIP_MATCHER = re.compile(r"[^\w\s.@+-]")
 CLEANUP_MATCHER = re.compile(r"[-\s]+")
@@ -56,7 +57,7 @@ class EmailAlreadyAssociated(AuthAlreadyAssociated):
     pass
 
 
-def get_github_email(access_token):
+def get_github_emails(access_token):
     """Get real e-mail from GitHub."""
     response = request(
         "get",
@@ -66,14 +67,29 @@ def get_github_email(access_token):
     )
     data = response.json()
     email = None
+    primary = None
+    public = None
+    emails = []
     for entry in data:
+        # Skip noreply e-mail only if we need deliverable e-mails
+        if entry["email"].endswith("@users.noreply.github.com"):
+            # Add E-Mail and set is_deliverable to false
+            emails.append((entry["email"], False))
+            continue
         # Skip not verified ones
         if not entry["verified"]:
             continue
+
+        # Add E-Mail and set is_deliverable to true
+        emails.append((entry["email"], True))
+        if entry.get("visibility") == "public":
+            # There is just one public mail, prefer it
+            public = entry["email"]
+            continue
         email = entry["email"]
         if entry["primary"]:
-            break
-    return email
+            primary = entry["email"]
+    return public or primary or email, emails
 
 
 @partial
@@ -99,12 +115,10 @@ def reauthenticate(strategy, backend, user, social, uid, weblate_action, **kwarg
 def require_email(backend, details, weblate_action, user=None, is_new=False, **kwargs):
     """Force entering e-mail for backends which don't provide it."""
     if backend.name == "github":
-        email = get_github_email(kwargs["response"]["access_token"])
+        email, emails = get_github_emails(kwargs["response"]["access_token"])
+        details["verified_emails"] = emails
         if email is not None:
             details["email"] = email
-        details_email = details.get("email") or ""
-        if details_email.endswith("@users.noreply.github.com"):
-            del details["email"]
 
     # Remove any pending e-mail validation codes
     if details.get("email") and backend.name == "email":
@@ -140,6 +154,7 @@ def send_validation(strategy, backend, code, partial_token):
     context = {"url": url, "validity": settings.AUTH_TOKEN_VALID // 3600}
 
     template = "activation"
+    user = None
     if session.get("password_reset"):
         template = "reset"
     elif session.get("account_remove"):
@@ -147,10 +162,11 @@ def send_validation(strategy, backend, code, partial_token):
     elif session.get("user_invite"):
         template = "invite"
         context.update(session["invitation_context"])
+        user = User.objects.get(pk=session["social_auth_user"])
 
     # Create audit log, it might be for anonymous at this point for new registrations
     AuditLog.objects.create(
-        strategy.request.user,
+        user,
         strategy.request,
         "sent-email",
         email=code.email,
@@ -200,7 +216,9 @@ def remove_account(
         session = strategy.request.session
         session.set_expiry(90)
         session["remove_confirm"] = True
-        # Redirect to form to change password
+        # Reset rate limit to allow form submission
+        reset_rate_limit("remove", strategy.request)
+        # Redirect to the confirmation form
         return redirect("remove")
     return None
 
@@ -355,16 +373,35 @@ def ensure_valid(
             AuditLog.objects.create(same[0].social.user, strategy.request, "connect")
             raise EmailAlreadyAssociated(backend, "E-mail exists")
 
+        validator = EmailValidator()
+        # This raises ValidationError
+        validator(details["email"])
+
 
 def store_email(strategy, backend, user, social, details, **kwargs):
     """Store verified e-mail."""
     # The email can be empty for some services
-    if details.get("email"):
+    if details.get("verified_emails"):
+        # For some reasons tuples get converted to lists inside python social auth
+        current = {tuple(verified) for verified in details["verified_emails"]}
+        existing = set(social.verifiedemail_set.values_list("email", "is_deliverable"))
+        for remove in existing - current:
+            social.verifiedemail_set.filter(
+                email=remove[0], is_deliverable=remove[1]
+            ).delete()
+        for add in current - existing:
+            social.verifiedemail_set.create(email=add[0], is_deliverable=add[1])
+    elif details.get("email"):
         verified, created = VerifiedEmail.objects.get_or_create(
             social=social, defaults={"email": details["email"]}
         )
-        if not created and verified.email != details["email"]:
+        if (
+            not created
+            and verified.email != details["email"]
+            or not verified.is_deliverable
+        ):
             verified.email = details["email"]
+            verified.is_deliverable = True
             verified.save()
 
 
@@ -381,7 +418,7 @@ def notify_connect(
     """Notify about adding new link."""
     # Adjust possibly pending email confirmation audit logs
     AuditLog.objects.filter(
-        user=get_anonymous(),
+        user=None,
         activity="sent-email",
         params={"email": details["email"]},
     ).update(user=user)
