@@ -21,7 +21,7 @@
 from django import template
 from django.conf import settings
 from django.contrib.staticfiles.storage import staticfiles_storage
-from django.utils.safestring import mark_safe
+from django.utils.html import format_html
 from django.utils.translation import gettext_lazy
 
 register = template.Library()
@@ -84,16 +84,19 @@ def get_auth_params(auth: str):
     return params
 
 
+auth_name_default_separator = format_html("<br />")
+
+
 @register.simple_tag
-def auth_name(auth: str, separator: str = "<br />"):
+def auth_name(auth: str, separator: str = auth_name_default_separator):
     """Create HTML markup for social authentication method."""
     params = get_auth_params(auth)
 
     if not params["image"].startswith("http"):
         params["image"] = staticfiles_storage.url("auth/" + params["image"])
-    params["icon"] = IMAGE_SOCIAL_TEMPLATE.format(separator=separator, **params)
+    params["icon"] = format_html(IMAGE_SOCIAL_TEMPLATE, separator=separator, **params)
 
-    return mark_safe(SOCIAL_TEMPLATE.format(separator=separator, **params))
+    return format_html(SOCIAL_TEMPLATE, separator=separator, **params)
 
 
 def get_auth_name(auth: str):
