@@ -590,15 +590,29 @@ class RegistrationTest(BaseRegistrationTest):
             responses.GET,
             "https://api.github.com/user/emails",
             json=[
+                {
+                    "email": "noreply@users.noreply.github.com",
+                    "verified": True,
+                    "primary": False,
+                    "visibility": "public",
+                },
                 {
                     "email": "noreply2@example.org",
                     "verified": False,
                     "primary": False,
+                    "visibility": "public",
+                },
+                {
+                    "email": "noreply-other@example.org",
+                    "verified": True,
+                    "primary": True,
+                    "visibility": "private",
                 },
                 {
                     "email": "noreply-weblate@example.org",
                     "verified": True,
-                    "primary": True,
+                    "primary": False,
+                    "visibility": "public",
                 },
             ],
         )
@@ -626,6 +640,18 @@ class RegistrationTest(BaseRegistrationTest):
         user = User.objects.get(username="weblate")
         self.assertEqual(user.full_name, "Test Weblate Name")
         self.assertEqual(user.email, "noreply-weblate@example.org")
+        self.assertEqual(
+            set(
+                VerifiedEmail.objects.filter(social__user=user).values_list(
+                    "email", "is_deliverable"
+                )
+            ),
+            {
+                ("noreply-other@example.org", True),
+                ("noreply-weblate@example.org", True),
+                ("noreply@users.noreply.github.com", False),
+            },
+        )
 
     def test_github_existing(self):
         """Adding GitHub association to existing account."""
