@@ -112,8 +112,8 @@ class GettextCustomizeForm(BaseAddonForm):
     width = forms.ChoiceField(
         label=_("Long lines wrapping"),
         choices=[
-            (77, _("Wrap lines at 77 characters and at newlines")),
-            (65535, _("Only wrap lines at newlines")),
+            (77, _("Wrap lines at 77 characters and at newlines (xgettext default)")),
+            (65535, _("Only wrap lines at newlines (like 'xgettext --no-wrap')")),
             (-1, _("No line wrapping")),
         ],
         required=True,
@@ -201,6 +201,16 @@ class JSONCustomizeForm(BaseAddonForm):
     )
 
 
+class XMLCustomizeForm(BaseAddonForm):
+    """Class defining user Form to configure XML Formatting AddOn."""
+
+    closing_tags = forms.BooleanField(
+        label=_("Include closing tag for blank XML tags"),
+        required=False,
+        initial=True,
+    )
+
+
 class YAMLCustomizeForm(BaseAddonForm):
     indent = forms.IntegerField(
         label=_("YAML indentation"), min_value=1, max_value=10, initial=2, required=True
@@ -276,6 +286,17 @@ class DiscoveryForm(BaseAddonForm):
             "For gettext choose .pot file."
         ),
     )
+    intermediate_template = forms.CharField(
+        label=_("Intermediate language file"),
+        initial="",
+        required=False,
+        help_text=_(
+            "Filename of intermediate translation file. In most cases "
+            "this is a translation file provided by developers and is "
+            "used when creating actual source strings."
+        ),
+    )
+
     language_regex = forms.CharField(
         label=_("Language filter"),
         max_length=200,
@@ -309,6 +330,7 @@ class DiscoveryForm(BaseAddonForm):
             Field("name_template"),
             Field("base_file_template"),
             Field("new_base_template"),
+            Field("intermediate_template"),
             Field("language_regex"),
             Field("copy_addons"),
             Field("remove"),
@@ -397,6 +419,9 @@ class DiscoveryForm(BaseAddonForm):
     def clean_new_base_template(self):
         return self.template_clean("new_base_template")
 
+    def clean_intermediate_template(self):
+        return self.template_clean("intermediate_template")
+
 
 class AutoAddonForm(AutoForm, AddonFormMixin):
     def __init__(self, user, addon, instance=None, *args, **kwargs):
@@ -516,6 +541,10 @@ class PseudolocaleAddonForm(BaseAddonForm):
             "the length of the source string."
         ),
     )
+    include_readonly = forms.BooleanField(
+        label=_("Include read-only strings"),
+        required=False,
+    )
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -528,6 +557,7 @@ class PseudolocaleAddonForm(BaseAddonForm):
         self.helper = FormHelper(self)
         self.helper.layout = Layout(
             Field("source"),
+            Field("include_readonly"),
             Field("target"),
             Field("prefix"),
             Field("var_prefix"),
