@@ -21,7 +21,7 @@ from appconf import AppConf
 from django.db import Error as DjangoDatabaseError
 from django.db import models
 from django.db.models import Q
-from django.db.models.signals import post_delete, post_save
+from django.db.models.signals import post_save
 from django.dispatch import receiver
 from django.urls import reverse
 from django.utils.functional import cached_property
@@ -40,7 +40,7 @@ from weblate.addons.events import (
     EVENT_UNIT_POST_SAVE,
     EVENT_UNIT_PRE_CREATE,
 )
-from weblate.trans.models import Component, Unit
+from weblate.trans.models import Change, Component, Unit
 from weblate.trans.signals import (
     component_post_update,
     store_post_load,
@@ -98,12 +98,22 @@ class Addon(models.Model):
         cls = self.addon_class
         self.project_scope = cls.project_scope
         self.repo_scope = cls.repo_scope
-        if self.component:
-            # Reallocate to repository
-            if self.repo_scope and self.component.linked_component:
-                self.component = self.component.linked_component
-            # Clear add-on cache
-            self.component.drop_addons_cache()
+
+        # Reallocate to repository
+        if self.repo_scope and self.component.linked_component:
+            self.component = self.component.linked_component
+
+        # Clear add-on cache
+        self.component.drop_addons_cache()
+
+        # Store history (if not updating state only)
+        if update_fields != ["state"]:
+            self.store_change(
+                Change.ACTION_ADDON_CREATE
+                if self.pk or force_insert
+                else Change.ACTION_ADDON_CHANGE
+            )
+
         return super().save(
             force_insert=force_insert,
             force_update=force_update,
@@ -121,6 +131,15 @@ class Addon(models.Model):
             },
         )
 
+    def store_change(self, action):
+        Change.objects.create(
+            action=action,
+            user=self.component.acting_user,
+            component=self.component,
+            target=self.name,
+            details=self.configuration,
+        )
+
     def configure_events(self, events):
         for event in events:
             Event.objects.get_or_create(addon=self, event=event)
@@ -134,11 +153,16 @@ class Addon(models.Model):
     def addon(self):
         return self.addon_class(self)
 
-    def delete(self, *args, **kwargs):
+    def delete(self, using=None, keep_parents=False):
+        # Store history
+        self.store_change(Change.ACTION_ADDON_REMOVE)
         # Delete any addon alerts
         if self.addon.alert:
             self.component.delete_alert(self.addon.alert)
-        super().delete(*args, **kwargs)
+        result = super().delete(using=using, keep_parents=keep_parents)
+        # Trigger post uninstall action
+        self.addon.post_uninstall()
+        return result
 
     def disable(self):
         self.component.log_warning(
@@ -152,7 +176,7 @@ class Event(models.Model):
     event = models.IntegerField(choices=EVENT_CHOICES)
 
     class Meta:
-        unique_together = ("addon", "event")
+        unique_together = [("addon", "event")]
         verbose_name = "add-on event"
         verbose_name_plural = "add-on events"
 
@@ -181,6 +205,7 @@ class AddonsConf(AppConf):
         "weblate.addons.generate.PseudolocaleAddon",
         "weblate.addons.generate.PrefillAddon",
         "weblate.addons.json.JSONCustomizeAddon",
+        "weblate.addons.xml.XMLCustomizeAddon",
         "weblate.addons.properties.PropertiesSortAddon",
         "weblate.addons.git.GitSquashAddon",
         "weblate.addons.removal.RemoveComments",
@@ -354,8 +379,3 @@ def store_post_load_handler(sender, translation, store, **kwargs):
             raise
         except Exception:
             handle_addon_error(addon, translation.component)
-
-
-@receiver(post_delete, sender=Addon)
-def addon_post_delete(sender, instance, **kwargs):
-    instance.addon.post_uninstall()
