@@ -27,6 +27,7 @@ from weblate.addons.models import ADDONS, Addon
 from weblate.auth.models import Group, Permission, Role, User
 from weblate.checks.models import CHECKS
 from weblate.lang.models import Language, Plural
+from weblate.memory.models import Memory
 from weblate.screenshots.models import Screenshot
 from weblate.trans.defines import LANGUAGE_NAME_LENGTH, REPO_LENGTH
 from weblate.trans.models import (
@@ -34,6 +35,7 @@ from weblate.trans.models import (
     Change,
     Component,
     ComponentList,
+    Label,
     Project,
     Translation,
     Unit,
@@ -127,6 +129,7 @@ class LanguageSerializer(serializers.ModelSerializer):
             "plural",
             "aliases",
             "direction",
+            "population",
             "web_url",
             "url",
             "statistics_url",
@@ -812,11 +815,17 @@ class UploadRequestSerializer(ReadOnlySerializer):
         choices=("", "process", "approve"), required=False, default=""
     )
     conflicts = serializers.ChoiceField(
-        choices=("", "replace-translated", "replace-approved"),
+        choices=("", "ignore", "replace-translated", "replace-approved"),
         required=False,
         default="",
     )
 
+    def validate_conflicts(self, value):
+        # These are handled same
+        if value == "ignore":
+            return ""
+        return value
+
     def check_perms(self, user, obj):
         data = self.validated_data
         if data["conflicts"] and not user.has_perm("upload.overwrite", obj):
@@ -836,8 +845,11 @@ class UploadRequestSerializer(ReadOnlySerializer):
             )
 
         if not check_upload_method_permissions(user, obj, data["method"]):
+            hint = "Check your permissions or use different translation object."
+            if data["method"] == "add" and not obj.is_source:
+                hint = "Try adding to the source instead of the translation."
             raise serializers.ValidationError(
-                {"method": "This method is not available here."}
+                {"method": f"This method is not available here. {hint}"}
             )
 
 
@@ -905,6 +917,51 @@ class PluralField(serializers.ListField):
         return getattr(instance, f"get_{self.field_name}_plurals")()
 
 
+class MemorySerializer(serializers.ModelSerializer):
+    class Meta:
+        model = Memory
+        fields = (
+            "id",
+            "source",
+            "target",
+            "source_language",
+            "target_language",
+            "origin",
+            "project",
+            "from_file",
+            "shared",
+        )
+
+
+class LabelsSerializer(serializers.RelatedField):
+    class Meta:
+        model = Label
+
+    def get_queryset(self):
+        """
+        List of available labels for an unit.
+
+        The queryset argument is only ever required for writable relationship field,
+        in which case it is used for performing the model instance lookup, that maps
+        from the primitive user input, into a model instance.
+        """
+        unit = self.parent.parent.instance
+        project = unit.translation.component.project
+        return project.label_set.all()
+
+    def to_representation(self, value):
+        return value.name
+
+    def to_internal_value(self, data):
+        try:
+            label = self.get_queryset().get(name=data)
+        except Label.DoesNotExist as err:
+            raise serializers.ValidationError(
+                "Label with this name was not found."
+            ) from err
+        return label
+
+
 class UnitSerializer(serializers.ModelSerializer):
     web_url = AbsoluteURLField(source="get_absolute_url", read_only=True)
     translation = MultiFieldHyperlinkedIdentityField(
@@ -923,6 +980,7 @@ class UnitSerializer(serializers.ModelSerializer):
     target = PluralField()
     timestamp = serializers.DateTimeField(read_only=True)
     pending = serializers.BooleanField(read_only=True)
+    labels = LabelsSerializer(many=True, read_only=True)
 
     class Meta:
         model = Unit
@@ -937,6 +995,7 @@ class UnitSerializer(serializers.ModelSerializer):
             "context",
             "note",
             "flags",
+            "labels",
             "state",
             "fuzzy",
             "translated",
@@ -963,6 +1022,7 @@ class UnitWriteSerializer(serializers.ModelSerializer):
     """Serializer for updating source unit."""
 
     target = PluralField()
+    labels = LabelsSerializer(many=True)
 
     class Meta:
         model = Unit
@@ -971,8 +1031,15 @@ class UnitWriteSerializer(serializers.ModelSerializer):
             "state",
             "explanation",
             "extra_flags",
+            "labels",
         )
 
+    def to_internal_value(self, data):
+        # Allow blank target for untranslated strings
+        if data.get("state") in (0, "0"):
+            self.fields["target"].child.allow_blank = True
+        return super().to_internal_value(data)
+
 
 class NewUnitSerializer(serializers.Serializer):
     def as_kwargs(self, data=None):
@@ -1157,8 +1224,8 @@ class AddonSerializer(serializers.ModelSerializer):
         try:
             name = attrs["name"]
         except KeyError:
-            if self.partial and self.instance:
-                name = self.instance.name
+            if self.partial and instance:
+                name = instance.name
             else:
                 raise serializers.ValidationError(
                     {"name": "Can not change add-on name"}
