@@ -29,7 +29,7 @@ from django.db import transaction
 from django.db.models import Q
 from django.http import Http404, HttpResponse
 from django.shortcuts import get_object_or_404
-from django.utils.safestring import mark_safe
+from django.utils.html import format_html
 from django_filters import rest_framework as filters
 from rest_framework import parsers, viewsets
 from rest_framework.decorators import action
@@ -64,6 +64,7 @@ from weblate.api.serializers import (
     LanguageSerializer,
     LockRequestSerializer,
     LockSerializer,
+    MemorySerializer,
     MonolingualUnitSerializer,
     NotificationSerializer,
     ProjectSerializer,
@@ -83,6 +84,7 @@ from weblate.auth.models import Group, Role, User
 from weblate.checks.models import Check
 from weblate.formats.models import EXPORTERS
 from weblate.lang.models import Language
+from weblate.memory.models import Memory
 from weblate.screenshots.models import Screenshot
 from weblate.trans.exceptions import FileParseError
 from weblate.trans.forms import AutoForm
@@ -146,8 +148,8 @@ def get_view_description(view, html=False):
         doc_url = get_doc_url("api", user=view.request.user)
 
     if html:
-        return formatting.markup_description(description) + mark_safe(
-            DOC_TEXT.format(doc_url)
+        return formatting.markup_description(description) + format_html(
+            DOC_TEXT, doc_url
         )
     return description
 
@@ -337,7 +339,7 @@ class UserViewSet(viewsets.ModelViewSet):
         remove_user(instance, request)
         return Response(status=HTTP_204_NO_CONTENT)
 
-    @action(detail=True, methods=["post"])
+    @action(detail=True, methods=["post", "delete"])
     def groups(self, request, **kwargs):
         obj = self.get_object()
         self.perm_check(request)
@@ -350,7 +352,10 @@ class UserViewSet(viewsets.ModelViewSet):
         except (Group.DoesNotExist, ValueError) as error:
             raise ValidationError(str(error))
 
-        obj.groups.add(group)
+        if request.method == "POST":
+            obj.groups.add(group)
+        if request.method == "DELETE":
+            obj.groups.remove(group)
         serializer = self.get_serializer_class()(obj, context={"request": request})
 
         return Response(serializer.data, status=HTTP_200_OK)
@@ -833,9 +838,7 @@ class ComponentViewSet(
                 )
 
             if not obj.can_add_new_language(request.user):
-                self.permission_denied(
-                    request, message="Could not add new translation file."
-                )
+                self.permission_denied(request, message=obj.new_lang_error_message)
 
             translation = obj.add_new_language(language, request)
             serializer = TranslationSerializer(
@@ -856,6 +859,7 @@ class ComponentViewSet(
     @action(detail=True, methods=["post"])
     def addons(self, request, **kwargs):
         obj = self.get_object()
+        obj.acting_user = request.user
 
         if not request.user.has_perm("component.edit", obj):
             self.permission_denied(request, "Can not create addon")
@@ -978,6 +982,27 @@ class ComponentViewSet(
         )
 
 
+class MemoryViewSet(viewsets.ModelViewSet, DestroyModelMixin):
+    """Memory API."""
+
+    queryset = Memory.objects.none()
+    serializer_class = MemorySerializer
+
+    def get_queryset(self):
+        if not self.request.user.is_superuser:
+            self.permission_denied(self.request, "Access not allowed")
+        return Memory.objects.order_by("id")
+
+    def perm_check(self, request, instance):
+        if not request.user.has_perm("memory.delete", instance):
+            self.permission_denied(request, "Can not delete memory entry")
+
+    def destroy(self, request, *args, **kwargs):
+        instance = self.get_object()
+        self.perm_check(request, instance)
+        return super().destroy(request, *args, **kwargs)
+
+
 class TranslationViewSet(MultipleFieldMixin, WeblateViewSet, DestroyModelMixin):
     """Translation components API."""
 
@@ -1097,8 +1122,8 @@ class TranslationViewSet(MultipleFieldMixin, WeblateViewSet, DestroyModelMixin):
             )
             serializer.is_valid(raise_exception=True)
 
-            obj.add_unit(request, **serializer.as_kwargs())
-            serializer = self.serializer_class(obj, context={"request": request})
+            unit = obj.add_unit(request, **serializer.as_kwargs())
+            serializer = UnitSerializer(unit, context={"request": request})
             return Response(serializer.data, status=HTTP_200_OK)
 
         query_string = request.GET.get("q", "")
@@ -1212,10 +1237,22 @@ class UnitViewSet(viewsets.ReadOnlyModelViewSet, UpdateModelMixin, DestroyModelM
     def get_queryset(self):
         return Unit.objects.filter_access(self.request.user).order_by("id")
 
+    def filter_queryset(self, queryset):
+        result = super().filter_queryset(queryset)
+        query_string = self.request.GET.get("q", "")
+        try:
+            parse_query(query_string)
+        except Exception as error:
+            report_error()
+            raise ValidationError(f"Failed to parse query string: {error}")
+        if query_string:
+            result = result.search(query_string)
+        return result
+
     def perform_update(self, serializer):
         data = serializer.validated_data
         do_translate = "target" in data or "state" in data
-        do_source = "extra_flags" in data or "explanation" in data
+        do_source = "extra_flags" in data or "explanation" in data or "labels" in data
         unit = serializer.instance
         translation = unit.translation
         request = self.request
@@ -1276,6 +1313,8 @@ class UnitViewSet(viewsets.ReadOnlyModelViewSet, UpdateModelMixin, DestroyModelM
                     setattr(unit, name, data[name])
                 except KeyError:
                     continue
+            if "labels" in data:
+                unit.labels.set(data["labels"])
             unit.save(update_fields=fields)
 
         # Handle translate
@@ -1602,10 +1641,12 @@ class AddonViewSet(viewsets.ReadOnlyModelViewSet, UpdateModelMixin, DestroyModel
 
     def update(self, request, *args, **kwargs):
         instance = self.get_object()
+        instance.component.acting_user = request.user
         self.perm_check(request, instance)
         return super().update(request, *args, **kwargs)
 
     def destroy(self, request, *args, **kwargs):
         instance = self.get_object()
+        instance.component.acting_user = request.user
         self.perm_check(request, instance)
         return super().destroy(request, *args, **kwargs)
