@@ -22,7 +22,7 @@ from django.contrib import admin
 from django.contrib.auth.admin import UserAdmin
 from django.contrib.auth.forms import UserChangeForm, UserCreationForm
 from django.core.exceptions import ValidationError
-from django.utils.safestring import mark_safe
+from django.utils.html import format_html
 from django.utils.translation import gettext_lazy as _
 
 from weblate.accounts.forms import FullNameField, UniqueEmailMixin, UniqueUsernameField
@@ -43,8 +43,23 @@ def block_role_edit(obj):
     return obj and obj.name in BUILT_IN_ROLES
 
 
+class AutoGroupChangeForm(forms.ModelForm):
+    class Meta:
+        model = AutoGroup
+        fields = "__all__"
+
+    def has_changed(self):
+        """
+        Should returns True if data differs from initial.
+
+        By always returning true even unchanged inlines will get validated and saved.
+        """
+        return True
+
+
 class InlineAutoGroupAdmin(admin.TabularInline):
     model = AutoGroup
+    form = AutoGroupChangeForm
     extra = 0
 
     def has_add_permission(self, request, obj=None):
@@ -120,7 +135,9 @@ class WeblateAuthAdmin(WeblateModelAdmin):
         perms_needed.discard("Audit log entry")
         perms_needed.discard("verified_email")
         perms_needed.discard("verified email")
+        perms_needed.discard("verified e-mail")
         perms_needed.discard("Verified email")
+        perms_needed.discard("Verified e-mail")
         return deleted_objects, model_count, perms_needed, protected
 
 
@@ -168,7 +185,7 @@ class WeblateUserAdmin(WeblateAuthAdmin, UserAdmin):
             return ""
         return super().action_checkbox(obj)
 
-    action_checkbox.short_description = mark_safe(
+    action_checkbox.short_description = format_html(
         '<input type="checkbox" id="action-toggle" />'
     )
 
@@ -192,6 +209,12 @@ class GroupChangeForm(forms.ModelForm):
         model = Group
         fields = "__all__"
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        if "components" in self.fields:
+            components = self.fields["components"]
+            components.queryset = components.queryset.select_related("project")
+
     def clean(self):
         super().clean()
         has_componentlist = bool(self.cleaned_data["componentlists"])
@@ -240,7 +263,7 @@ class WeblateGroupAdmin(WeblateAuthAdmin):
             return ""
         return super().action_checkbox(obj)
 
-    action_checkbox.short_description = mark_safe(
+    action_checkbox.short_description = format_html(
         '<input type="checkbox" id="action-toggle" />'
     )
 
