@@ -22,6 +22,7 @@ from collections import defaultdict
 from itertools import chain
 from typing import Optional, Set
 
+import sentry_sdk
 from appconf import AppConf
 from django.conf import settings
 from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
@@ -158,6 +159,14 @@ class Group(models.Model):
         verbose_name=_("Internal Weblate group"), default=False
     )
 
+    admins = models.ManyToManyField(
+        "User",
+        verbose_name=_("Team administrators"),
+        blank=True,
+        help_text=_("The administrator can add or remove users from a team."),
+        related_name="administered_group_set",
+    )
+
     objects = GroupQuerySet.as_manager()
 
     class Meta:
@@ -194,6 +203,9 @@ class Group(models.Model):
                 clear=True,
             )
 
+    def get_absolute_url(self):
+        return reverse("team", kwargs={"pk": self.pk})
+
     def long_name(self):
         if self.defining_project:
             return f"{self.defining_project} / {self}"
@@ -231,10 +243,9 @@ class UserQuerySet(models.QuerySet):
 
         Note: This intentionally does not list superusers.
         """
-        groups = Group.objects.filter(
-            roles__permissions__codename=perm, projects=project
-        )
-        return self.filter(groups__in=groups).distinct()
+        return self.filter(
+            groups__roles__permissions__codename=perm, groups__projects=project
+        ).distinct()
 
     def all_admins(self, project):
         """All admins in a project."""
@@ -551,51 +562,58 @@ class User(AbstractBaseUser):
     def managed_projects(self):
         return self.projects_with_perm("project.edit")
 
+    @cached_property
+    def administered_group_ids(self):
+        return set(self.administered_group_set.values_list("id", flat=True))
+
     def _fetch_permissions(self):
         """Fetch all user permissions into a dictionary."""
         projects = defaultdict(list)
         components = defaultdict(list)
-        for group in self.groups.prefetch_related(
-            "roles__permissions",
-            "componentlists__components",
-            "components",
-            "projects",
-            "languages",
-        ):
-            languages = {language.id for language in group.languages.all()}
-            permissions = {
-                permission.codename
-                for permission in chain.from_iterable(
-                    role.permissions.all() for role in group.roles.all()
-                )
-            }
-            # Component list specific permissions
-            componentlist_values = {
-                (component.id, component.project_id)
-                for component in chain.from_iterable(
-                    clist.components.all() for clist in group.componentlists.all()
-                )
-            }
-            if group.componentlists.exists():
-                for component, project in componentlist_values:
-                    components[component].append((permissions, languages))
-                    # Grant access to the project
-                    projects[project].append(((), languages))
-                continue
-            # Component specific permissions
-            component_values = {
-                (component.id, component.project_id)
-                for component in group.components.all()
-            }
-            if component_values:
-                for component, project in component_values:
-                    components[component].append((permissions, languages))
-                    # Grant access to the project
-                    projects[project].append(((), languages))
-                continue
-            # Project specific permissions
-            for project in group.projects.all():
-                projects[project.id].append((permissions, languages))
+        with sentry_sdk.start_span(op="permissions", description=self.username):
+            for group in self.groups.prefetch_related(
+                "roles__permissions",
+                "componentlists__components",
+                "components",
+                "projects",
+            ):
+                if group.language_selection == SELECTION_ALL:
+                    languages = None
+                else:
+                    languages = {language.id for language in group.languages.all()}
+                permissions = {
+                    permission.codename
+                    for permission in chain.from_iterable(
+                        role.permissions.all() for role in group.roles.all()
+                    )
+                }
+                # Component list specific permissions
+                componentlist_values = {
+                    (component.id, component.project_id)
+                    for component in chain.from_iterable(
+                        clist.components.all() for clist in group.componentlists.all()
+                    )
+                }
+                if group.componentlists.exists():
+                    for component, project in componentlist_values:
+                        components[component].append((permissions, languages))
+                        # Grant access to the project
+                        projects[project].append(((), languages))
+                    continue
+                # Component specific permissions
+                component_values = {
+                    (component.id, component.project_id)
+                    for component in group.components.all()
+                }
+                if component_values:
+                    for component, project in component_values:
+                        components[component].append((permissions, languages))
+                        # Grant access to the project
+                        projects[project].append(((), languages))
+                    continue
+                # Project specific permissions
+                for project in group.projects.all():
+                    projects[project.id].append((permissions, languages))
         # Apply blocking
         now = timezone.now()
         for block in self.userblock_set.all():
@@ -630,22 +648,16 @@ class User(AbstractBaseUser):
         groups = Group.objects.filter(user=self, roles__permissions__codename=perm)
         return Project.objects.filter(group__in=groups).distinct().order()
 
-    def get_visible_name(self):
-        # Get full name from database or username
+    def get_visible_name(self) -> str:
+        """Get full name from database or username."""
         result = self.full_name or self.username
-        return result.replace("<", "").replace(">", "").replace('"', "")
-
-    def get_author_name(self, email=True):
-        """Return formatted author name with e-mail."""
         # The < > are replace to avoid tricking Git to use
         # name as e-mail
+        return result.replace("<", "").replace(">", "").replace('"', "")
 
-        full_name = self.get_visible_name()
-
-        # Add e-mail if we are asked for it
-        if not email:
-            return full_name
-        return f"{full_name} <{self.email}>"
+    def get_author_name(self) -> str:
+        """Return formatted author name with e-mail."""
+        return f"{self.get_visible_name()} <{self.profile.get_commit_email()}>"
 
 
 class AutoGroup(models.Model):
@@ -681,7 +693,7 @@ class UserBlock(models.Model):
     class Meta:
         verbose_name = "Blocked user"
         verbose_name_plural = "Blocked users"
-        unique_together = ("user", "project")
+        unique_together = [("user", "project")]
 
     def __str__(self):
         return f"{self.user} blocked for {self.project}"
@@ -698,10 +710,20 @@ def create_groups(update):
     create_anonymous(User, Group, update)
 
     # Automatic assignment to the users group
-    group = Group.objects.get(name="Users")
+    group = Group.objects.get(
+        name="Users",
+        internal=True,
+        project_selection=SELECTION_ALL_PUBLIC,
+        language_selection=SELECTION_ALL,
+    )
     if not AutoGroup.objects.filter(group=group).exists():
         AutoGroup.objects.create(group=group, match="^.*$")
-    group = Group.objects.get(name="Viewers")
+    group = Group.objects.get(
+        name="Viewers",
+        internal=True,
+        project_selection=SELECTION_ALL_PROTECTED,
+        language_selection=SELECTION_ALL,
+    )
     if not AutoGroup.objects.filter(group=group).exists():
         AutoGroup.objects.create(group=group, match="^.*$")
 
@@ -740,6 +762,20 @@ def change_componentlist(sender, instance, action, **kwargs):
         )
 
 
+@receiver(m2m_changed, sender=User.groups.through)
+def remove_group_admin(sender, instance, action, pk_set, **kwargs):
+    if not action == "post_remove":
+        return
+    pk = pk_set.pop()
+    if reverse:
+        group = Group.objects.get(pk=pk)
+        user = instance
+    else:
+        group = instance
+        user = User.objects.get(pk=pk)
+    group.admins.remove(user)
+
+
 @receiver(post_save, sender=User)
 @disable_for_loaddata
 def auto_group_upon_save(sender, instance, created=False, **kwargs):
