@@ -19,7 +19,6 @@
 
 from django.conf import settings
 
-from weblate.machinery import MACHINE_TRANSLATION_SERVICES
 from weblate.trans.models import (
     Component,
     ComponentList,
@@ -81,19 +80,19 @@ def check_permission(user, permission, obj):
         return (
             not obj.component.restricted
             and any(
-                permission in permissions and lang in langs
+                permission in permissions and (langs is None or lang in langs)
                 for permissions, langs in user.project_permissions[
                     obj.component.project_id
                 ]
             )
         ) or any(
-            permission in permissions and lang in langs
+            permission in permissions and (langs is None or lang in langs)
             for permissions, langs in user.component_permissions[obj.component_id]
         )
     raise ValueError(f"Permission {permission} does not support: {obj.__class__}")
 
 
-@register_perm("comment.delete", "suggestion.delete")
+@register_perm("comment.resolve", "comment.delete", "suggestion.delete")
 def check_delete_own(user, permission, obj):
     if user.is_authenticated and obj.user == user:
         return True
@@ -234,6 +233,11 @@ def check_unit_delete(user, permission, obj):
     # Check if removing is generally allowed
     if not check_manage_units(obj, component):
         return False
+
+    # Does file format support removing?
+    if not component.file_format_cls.can_delete_unit:
+        return False
+
     if component.is_glossary:
         permission = "glossary.delete"
     return check_can_edit(user, permission, obj)
@@ -321,10 +325,6 @@ def check_contribute(user, permission, translation):
 
 @register_perm("machinery.view")
 def check_machinery(user, permission, obj):
-    # No permission in case there are no machinery services enabled
-    if not MACHINE_TRANSLATION_SERVICES.exists():
-        return False
-
     # No machinery for source without intermediate language
     if (
         isinstance(obj, Translation)
@@ -367,6 +367,21 @@ def check_repository_status(user, permission, obj):
     )
 
 
+@register_perm("meta:team.edit")
+def check_team_edit(user, permission, obj):
+    return check_global_permission(user, "group.edit", obj) or (
+        obj.defining_project
+        and check_permission(user, "project.permissions", obj.defining_project)
+    )
+
+
+@register_perm("meta:team.users")
+def check_team_edit_users(user, permission, obj):
+    return (
+        check_team_edit(user, permission, obj) or obj.pk in user.administered_group_ids
+    )
+
+
 @register_perm("billing.view")
 def check_billing_view(user, permission, obj):
     if hasattr(obj, "all_projects"):
@@ -406,3 +421,18 @@ def check_unit_flag(user, permission, obj):
         return user.has_perm("source.edit", obj)
 
     return user.has_perm("glossary.edit", obj)
+
+
+@register_perm("memory.edit", "memory.delete")
+def check_memory_perms(user, permission, memory):
+    from weblate.memory.models import Memory
+
+    if isinstance(memory, Memory):
+        if memory.user_id == user.id:
+            return True
+        if memory.project is None:
+            return user.is_superuser
+        project = memory.project
+    else:
+        project = memory
+    return check_permission(user, permission, project)
