@@ -30,8 +30,7 @@ from django.dispatch import receiver
 from django.urls import reverse
 from django.utils import timezone
 from django.utils.functional import cached_property
-from django.utils.html import escape
-from django.utils.safestring import mark_safe
+from django.utils.html import format_html
 from django.utils.translation import gettext_lazy as _
 from django.utils.translation import ngettext
 
@@ -193,7 +192,7 @@ class Billing(models.Model):
     )
     # Payment detailed information, used for integration
     # with payment processor
-    payment = JSONField(editable=False, default={})
+    payment = JSONField(editable=False, default=dict)
 
     objects = BillingManager.from_queryset(BillingQuerySet)()
 
@@ -207,7 +206,7 @@ class Billing(models.Model):
         if projects:
             base = projects
         elif owners:
-            base = ", ".join(x.get_author_name(False) for x in owners)
+            base = ", ".join(x.get_visible_name() for x in owners)
         else:
             base = "Unassigned"
         trial = ", trial" if self.is_trial else ""
@@ -313,9 +312,7 @@ class Billing(models.Model):
 
     @cached_property
     def count_languages(self):
-        if not self.all_projects:
-            return 0
-        return max(p.stats.languages for p in self.all_projects)
+        return max((p.stats.languages for p in self.all_projects), default=0)
 
     def display_languages(self):
         return f"{self.count_languages} / {self.plan.display_limit_languages}"
@@ -445,12 +442,11 @@ class Billing(models.Model):
         for project in self.all_projects:
             yield LibreCheck(
                 bool(project.web),
-                mark_safe(
-                    '<a href="{0}">{1}</a>, <a href="{2}">{2}</a>'.format(
-                        escape(project.get_absolute_url()),
-                        escape(project),
-                        escape(project.web),
-                    )
+                format_html(
+                    '<a href="{0}">{1}</a>, <a href="{2}">{2}</a>',
+                    project.get_absolute_url(),
+                    project,
+                    project.web,
                 ),
             )
         components = Component.objects.filter(
@@ -464,19 +460,18 @@ class Billing(models.Model):
         for component in components:
             yield LibreCheck(
                 component.libre_license,
-                mark_safe(
+                format_html(
                     """
                     <a href="{0}">{1}</a>,
                     <a href="{2}">{3}</a>,
                     <a href="{4}">{4}</a>,
-                    {5}""".format(
-                        escape(component.get_absolute_url()),
-                        escape(component.name),
-                        escape(component.license_url or "#"),
-                        escape(component.get_license_display() or _("Missing license")),
-                        escape(component.repo),
-                        escape(component.get_file_format_display()),
-                    )
+                    {5}""",
+                    component.get_absolute_url(),
+                    component.name,
+                    component.license_url or "#",
+                    component.get_license_display() or _("Missing license"),
+                    component.repo,
+                    component.get_file_format_display(),
                 ),
                 component=component,
             )
@@ -518,7 +513,7 @@ class Invoice(models.Model):
     note = models.TextField(blank=True)
     # Payment detailed information, used for integration
     # with payment processor
-    payment = JSONField(editable=False, default={})
+    payment = JSONField(editable=False, default=dict)
 
     objects = InvoiceQuerySet.as_manager()
 
