@@ -19,8 +19,11 @@
 
 import re
 from io import StringIO
+from typing import Iterable
 
 from django.http import Http404
+from django.utils.html import conditional_escape, format_html, format_html_join
+from django.utils.translation import gettext
 from lxml import etree
 from lxml.etree import XMLSyntaxError
 from siphashc import siphash
@@ -269,6 +272,32 @@ class TargetCheck(Check):
         """Check for single phrase, not dealing with plurals."""
         raise NotImplementedError()
 
+    def format_value(self, value: str):
+        from weblate.trans.templatetags.translations import Formatter
+
+        fmt = Formatter(0, value, None, None, None, None, None)
+        fmt.parse()
+        return format_html(
+            """<span class="hlcheck" data-value="{}">{}</span>""", value, fmt.format()
+        )
+
+    def get_values_text(self, message: str, values: Iterable[str]):
+        return format_html_join(
+            ", ",
+            conditional_escape(message),
+            ((self.format_value(value),) for value in sorted(values)),
+        )
+
+    def get_missing_text(self, values: Iterable[str]):
+        return self.get_values_text(
+            gettext("Following format strings are missing: {}"), values
+        )
+
+    def get_extra_text(self, values: Iterable[str]):
+        return self.get_values_text(
+            gettext("Following format strings are extra: {}"), values
+        )
+
 
 class SourceCheck(Check):
     """Basic class for source checks."""
@@ -284,11 +313,10 @@ class SourceCheck(Check):
         raise NotImplementedError()
 
 
-class TargetCheckParametrized(Check):
+class TargetCheckParametrized(TargetCheck):
     """Basic class for target checks with flag value."""
 
     default_disabled = True
-    target = True
 
     def get_value(self, unit):
         return unit.all_flags.get_value(self.enable_string)
@@ -311,10 +339,6 @@ class TargetCheckParametrized(Check):
         """We don't check single phrase here."""
         return False
 
-    def check_source_unit(self, source, unit):
-        """We don't check source strings here."""
-        return False
-
 
 class CountingCheck(TargetCheck):
     """Check whether there is same count of given string."""
