@@ -66,9 +66,9 @@ TYPED_FLAGS["font-weight"] = gettext_lazy("Font weight")
 TYPED_FLAGS_ARGS["font-weight"] = single_value_flag(get_font_weight)
 TYPED_FLAGS["font-spacing"] = gettext_lazy("Font spacing")
 TYPED_FLAGS_ARGS["font-spacing"] = single_value_flag(int)
-TYPED_FLAGS["icu-flags"] = gettext_lazy("ICU MessageFormat Flags")
+TYPED_FLAGS["icu-flags"] = gettext_lazy("ICU MessageFormat flags")
 TYPED_FLAGS_ARGS["icu-flags"] = multi_value_flag(str)
-TYPED_FLAGS["icu-tag-prefix"] = gettext_lazy("ICU MessageFormat Tag Prefix")
+TYPED_FLAGS["icu-tag-prefix"] = gettext_lazy("ICU MessageFormat tag prefix")
 TYPED_FLAGS_ARGS["icu-tag-prefix"] = single_value_flag(str)
 TYPED_FLAGS["priority"] = gettext_lazy("Priority")
 TYPED_FLAGS_ARGS["priority"] = single_value_flag(int)
@@ -235,11 +235,14 @@ class Flags:
     @staticmethod
     def format_value(value):
         # Regexp objects
+        prefix = ""
         if hasattr(value, "pattern"):
             value = value.pattern
-        if " " in value or any(c in value for c in SYNTAXCHARS):
-            return '"{}"'.format(
-                value.replace('"', r"\"").replace("\n", "\\n").replace("\r", "\\r")
+            prefix = "r"
+        if prefix or " " in value or any(c in value for c in SYNTAXCHARS):
+            return '{}"{}"'.format(
+                prefix,
+                value.replace('"', r"\"").replace("\n", "\\n").replace("\r", "\\r"),
             )
         return value
 
@@ -266,7 +269,7 @@ class Flags:
             if name in self._values:
                 if is_plain:
                     raise ValidationError(
-                        _('Translation flag has no parameters: "%s"') % name
+                        _('The translation flag has no parameters: "%s"') % name
                     )
                 try:
                     self.get_value(name)
