@@ -22,8 +22,7 @@ from collections import defaultdict
 from typing import Optional, Pattern
 
 from django.utils.functional import SimpleLazyObject
-from django.utils.html import escape
-from django.utils.safestring import mark_safe
+from django.utils.html import format_html, format_html_join
 from django.utils.translation import gettext
 from django.utils.translation import gettext_lazy as _
 
@@ -148,7 +147,7 @@ C_SHARP_MATCH = re.compile(
         {                               # initial {
         (?P<arg>\d+)                    # variable order
         (?P<width>
-            [,-?\s]+                    # flags
+            [-,?\s]+                    # flags
             (?:\d+)?                    # width
             (?:\.\d+)?                  # precision
         )?
@@ -258,7 +257,7 @@ FLAG_RULES = {
     "python-brace-format": (PYTHON_BRACE_MATCH, name_format_is_position_based),
     "scheme-format": (SCHEME_PRINTF_MATCH, scheme_format_is_position_based),
     "c-sharp-format": (C_SHARP_MATCH, name_format_is_position_based),
-    "java-format": (JAVA_MATCH, c_format_is_position_based),
+    "java-printf-format": (JAVA_MATCH, c_format_is_position_based),
 }
 
 
@@ -391,12 +390,12 @@ class BaseFormatCheck(TargetCheck):
             ) % ", ".join(self.format_string(x) for x in sorted(set(result["missing"])))
         else:
             if result["missing"]:
-                yield gettext("Following format strings are missing: %s") % ", ".join(
-                    self.format_string(x) for x in sorted(set(result["missing"]))
+                yield self.get_missing_text(
+                    self.format_string(x) for x in set(result["missing"])
                 )
             if result["extra"]:
-                yield gettext("Following format strings are extra: %s") % ", ".join(
-                    self.format_string(x) for x in sorted(set(result["extra"]))
+                yield self.get_extra_text(
+                    self.format_string(x) for x in set(result["extra"])
                 )
 
     def get_description(self, check_obj):
@@ -415,7 +414,9 @@ class BaseFormatCheck(TargetCheck):
         if results:
             errors.extend(self.format_result(results))
         if errors:
-            return mark_safe("<br />".join(escape(error) for error in errors))
+            return format_html_join(
+                format_html("<br />"), "{}", ((error,) for error in errors)
+            )
         return super().get_description(check_obj)
 
 
@@ -546,7 +547,7 @@ class CSharpFormatCheck(BaseFormatCheck):
 class JavaFormatCheck(BasePrintfCheck):
     """Check for Java format string."""
 
-    check_id = "java_format"
+    check_id = "java_printf_format"
     name = _("Java format")
     description = _("Java format string does not match source")
 
@@ -554,7 +555,7 @@ class JavaFormatCheck(BasePrintfCheck):
 class JavaMessageFormatCheck(BaseFormatCheck):
     """Check for Java MessageFormat string."""
 
-    check_id = "java_messageformat"
+    check_id = "java_format"
     name = _("Java MessageFormat")
     description = _("Java MessageFormat string does not match source")
     regexp = JAVA_MESSAGE_MATCH
