@@ -56,6 +56,7 @@ class MockComponent:
         self.project = MockProject()
         self.name = "MockComponent"
         self.file_format = "auto"
+        self.is_multivalue = False
 
 
 class MockTranslation:
@@ -66,6 +67,7 @@ class MockTranslation:
         self.component = MockComponent()
         self.is_template = False
         self.is_source = False
+        self.plural = self.language.plural
 
     @staticmethod
     def log_debug(text, *args):
