@@ -171,10 +171,15 @@ class PythonFormatCheckTest(CheckTestCase):
             extra_flags="python-format",
         )
         check = Check(unit=unit)
-        self.assertEqual(
+        self.assertHTMLEqual(
             self.check.get_description(check),
-            "Following format strings are missing: %(count)d<br />"
-            "Following format strings are extra: %(languages)d",
+            """
+            Following format strings are missing:
+            <span class="hlcheck" data-value="%(count)d">%(count)d</span>
+            <br />
+            Following format strings are extra:
+            <span class="hlcheck" data-value="%(languages)d">%(languages)d</span>
+            """,
         )
 
     def test_description_nolocation(self):
@@ -586,7 +591,7 @@ class JavaFormatCheckTest(CheckTestCase):
     def setUp(self):
         super().setUp()
         self.test_highlight = (
-            "java-format",
+            "java-printf-format",
             "%1s string %2s",
             [(0, 3, "%1s"), (11, 14, "%2s")],
         )
@@ -644,7 +649,7 @@ class JavaMessageFormatCheckTest(CheckTestCase):
     def setUp(self):
         super().setUp()
         self.test_highlight = (
-            "java-messageformat",
+            "java-format",
             "{0}string{1}",
             [(0, 3, "{0}"), (9, 12, "{1}")],
         )
@@ -730,7 +735,7 @@ class JavaMessageFormatCheckTest(CheckTestCase):
     def test_skip(self):
         unit = MockUnit(source="source")
         self.assertTrue(self.check.should_skip(unit))
-        unit = MockUnit(source="source", flags="java-messageformat")
+        unit = MockUnit(source="source", flags="java-format")
         self.assertFalse(self.check.should_skip(unit))
         unit = MockUnit(source="source", flags="auto-java-messageformat")
         self.assertTrue(self.check.should_skip(unit))
@@ -758,7 +763,7 @@ class JavaMessageFormatCheckTest(CheckTestCase):
         unit = Unit(
             source="{0}''s brush is {1} centimeters tall",
             target="{0}'s brush is {1} centimeters tall",
-            extra_flags="java-messageformat",
+            extra_flags="java-format",
             translation=Translation(
                 component=Component(
                     file_format="auto",
@@ -1210,10 +1215,15 @@ class ESTemplateLiteralsCheckTest(CheckTestCase):
             extra_flags="es-format",
         )
         check = Check(unit=unit)
-        self.assertEqual(
+        self.assertHTMLEqual(
             self.check.get_description(check),
-            "Following format strings are missing: ${foo}<br />"
-            "Following format strings are extra: ${bar}",
+            """
+            Following format strings are missing:
+            <span class="hlcheck" data-value="${foo}">${foo}</span>
+            <br />
+            Following format strings are extra:
+            <span class="hlcheck" data-value="${bar}">${bar}</span>
+            """,
         )
 
 
