@@ -28,9 +28,13 @@ class SettingQuerySet(models.QuerySet):
 
 class Setting(models.Model):
     CATEGORY_UI = 1
+    CATEGORY_MT = 2
 
     category = models.IntegerField(
-        choices=((CATEGORY_UI, "UI"),),
+        choices=(
+            (CATEGORY_UI, "UI"),
+            (CATEGORY_MT, "MT"),
+        ),
         db_index=True,
     )
     name = models.CharField(max_length=100)
@@ -39,9 +43,9 @@ class Setting(models.Model):
     objects = SettingQuerySet.as_manager()
 
     class Meta:
-        unique_together = ("category", "name")
+        unique_together = [("category", "name")]
         verbose_name = "Setting"
         verbose_name_plural = "Settings"
 
     def __str__(self):
-        return f"{self.name}:{self.value}"
+        return f"{self.get_category_display()}:{self.name}:{self.value}"
