@@ -20,13 +20,14 @@
 
 import os
 import tempfile
-from copy import deepcopy
+from copy import copy
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
 from django.utils.functional import cached_property
 from django.utils.translation import gettext as _
 from weblate_language_data.countries import DEFAULT_LANGS
 
+from weblate.trans.util import get_string
 from weblate.utils.errors import add_breadcrumb
 from weblate.utils.hash import calculate_hash
 from weblate.utils.state import STATE_TRANSLATED
@@ -44,6 +45,61 @@ LEGACY_CODES = {
     "zh_Hans_SG": "zh_SG",
     "zh_Hant_HK": "zh_HK",
 }
+APPSTORE_CODES = {
+    "ar": "ar-SA",
+    "de": "de-DE",
+    "fr": "fr-FR",
+    "nl": "nl-NL",
+    "pt": "pt-PT",
+}
+
+# Based on https://support.google.com/googleplay/android-developer/answer/9844778
+GOOGLEPLAY_CODES = {
+    "hy": "hy-AM",
+    "az": "az-AZ",
+    "eu": "eu-ES",
+    "my": "my-MM",
+    "zh_Hant_HK": "zh-HK",
+    "zh_Hans": "zh-CN",
+    "zh_Hant": "zh-TW",
+    "cs": "cs-CZ",
+    "da": "da-DK",
+    "nl": "nl-NL",
+    "en": "en-SG",
+    "fi": "fi-FI",
+    "fr": "fr-FR",
+    "gl": "gl-ES",
+    "ka": "ka-GE",
+    "de": "de-DE",
+    "el": "el-GR",
+    "he": "iw-IL",
+    "hi": "hi-IN",
+    "hu": "hu-HU",
+    "is": "is-IS",
+    "it": "it-IT",
+    "ja": "ja-JP",
+    "kn": "kn-IN",
+    "km": "km-KH",
+    "ko": "ko-KR",
+    "ky": "ky-KG",
+    "lo": "lo-LA",
+    "mk": "mk-MK",
+    "ms": "ms-MY",
+    "ml": "ml-IN",
+    "mr": "mr-IN",
+    "mn": "mn-MN",
+    "ne": "ne-NP",
+    "nb_NO": "no-NO",
+    "fa": "fa-IR",
+    "pl": "pl-PL",
+    "ru": "ru-RU",
+    "si": "si-LK",
+    "es": "es-ES",
+    "sv": "sv-SE",
+    "ta": "ta-IN",
+    "te": "te-IN",
+    "tr": "tr-TR",
+}
 
 
 class UnitNotFound(Exception):
@@ -67,6 +123,8 @@ class TranslationUnit:
     It handles ID/template based translations and other API differences.
     """
 
+    id_hash_with_source: bool = False
+
     def __init__(self, parent, unit, template=None):
         """Create wrapper object."""
         self.unit = unit
@@ -120,15 +178,23 @@ class TranslationUnit:
         """Return previous message source if there was any."""
         return ""
 
-    @cached_property
-    def id_hash(self):
+    @classmethod
+    def calculate_id_hash(cls, has_template: bool, source: str, context: str):
         """Return hash of source string, used for quick lookup.
 
         We use siphash as it is fast and works well for our purpose.
         """
-        if self.template is None:
-            return calculate_hash(self.source, self.context)
-        return calculate_hash(self.context)
+        if not has_template or cls.id_hash_with_source:
+            return calculate_hash(source, context)
+        return calculate_hash(context)
+
+    @cached_property
+    def id_hash(self):
+        return self.calculate_id_hash(
+            self.template is not None,
+            self.source,
+            self.context,
+        )
 
     def is_translated(self):
         """Check whether unit is translated."""
@@ -150,7 +216,7 @@ class TranslationUnit:
         """Check whether unit is read only."""
         return False
 
-    def set_target(self, target):
+    def set_target(self, target: Union[str, List[str]]):
         """Set translation unit target."""
         raise NotImplementedError()
 
@@ -158,6 +224,13 @@ class TranslationUnit:
         """Set fuzzy /approved flag on translated unit."""
         raise NotImplementedError()
 
+    def has_unit(self) -> bool:
+        return self.unit is not None
+
+    def clone_template(self):
+        self.mainunit = self.unit = copy(self.template)
+        self._invalidate_target()
+
 
 class TranslationFormat:
     """Generic object defining file format loader."""
@@ -169,6 +242,7 @@ class TranslationFormat:
     unit_class: Type[TranslationUnit] = TranslationUnit
     autoload: Tuple[str, ...] = ()
     can_add_unit: bool = True
+    can_delete_unit: bool = True
     language_format: str = "posix"
     simple_filename: bool = True
     new_translation: Optional[Union[str, bytes]] = None
@@ -176,6 +250,8 @@ class TranslationFormat:
     create_empty_bilingual: bool = False
     bilingual_class = None
     create_style = "create"
+    has_multiple_strings: bool = False
+    plural_preference: Optional[Tuple[int, ...]] = None
 
     @classmethod
     def get_identifier(cls):
@@ -218,13 +294,13 @@ class TranslationFormat:
         self.storefile = storefile
         self.language_code = language_code
         self.source_language = source_language
+        # Remember template
+        self.template_store = template_store
+        self.is_template = is_template
 
         # Load store
         self.store = self.load(storefile, template_store)
 
-        # Remember template
-        self.template_store = template_store
-        self.is_template = is_template
         self.add_breadcrumb(
             "Loaded translation file {}".format(
                 getattr(storefile, "filename", storefile)
@@ -239,6 +315,7 @@ class TranslationFormat:
             raise ValueError(
                 _("Failed to load strings from the file, try choosing other format.")
             )
+        self.ensure_index()
 
     def get_filenames(self):
         if isinstance(self.storefile, str):
@@ -251,6 +328,17 @@ class TranslationFormat:
     @classmethod
     def get_plural(cls, language, store=None):
         """Return matching plural object."""
+        if cls.plural_preference is not None:
+            # Fetch all matching plurals
+            plurals = language.plural_set.filter(source__in=cls.plural_preference)
+
+            # Use first matching in the order of preference
+            for source in cls.plural_preference:
+                for plural in plurals:
+                    if plural.source == source:
+                        return plural
+
+        # Fall back to default one
         return language.plural
 
     @cached_property
@@ -261,43 +349,51 @@ class TranslationFormat:
         ) and self.template_store is not None
 
     @cached_property
-    def _context_index(self):
+    def _template_index(self):
         """ID based index for units."""
-        return {unit.context: unit for unit in self.mono_units}
+        return {unit.id_hash: unit for unit in self.template_units}
 
-    def find_unit_mono(self, context: str) -> Optional[Any]:
+    def find_unit_template(
+        self, context: str, source: str, id_hash: Optional[int] = None
+    ) -> Optional[Any]:
+        if id_hash is None:
+            id_hash = self._calculate_string_hash(context, source)
         try:
             # The mono units always have only template set
-            return self._context_index[context].template
+            return self._template_index[id_hash].template
         except KeyError:
             return None
 
-    def _find_unit_template(self, context: str) -> Tuple[Any, bool]:
-        # Need to create new unit based on template
-        template_ttkit_unit = self.template_store.find_unit_mono(context)
-        if template_ttkit_unit is None:
-            raise UnitNotFound(context)
-
+    def _find_unit_monolingual(self, context: str, source: str) -> Tuple[Any, bool]:
         # We search by ID when using template
-        ttkit_unit = self.find_unit_mono(context)
+        id_hash = self._calculate_string_hash(context, source)
+        try:
+            result = self._unit_index[id_hash]
+        except KeyError:
+            raise UnitNotFound(context, source)
 
-        # We always need new unit to translate
-        if ttkit_unit is None:
-            ttkit_unit = deepcopy(template_ttkit_unit)
+        add = False
+        if not result.has_unit():
+            # We always need copy of template unit to translate
+            result.clone_template()
             add = True
-        else:
-            add = False
-
-        return (self.unit_class(self, ttkit_unit, template_ttkit_unit), add)
+        return result, add
 
     @cached_property
-    def _source_index(self):
+    def _unit_index(self):
         """Context and source based index for units."""
-        return {(unit.context, unit.source): unit for unit in self.all_units}
+        return {unit.id_hash: unit for unit in self.content_units}
+
+    def _calculate_string_hash(self, context: str, source: str) -> int:
+        """Calculates id hash for a string."""
+        return self.unit_class.calculate_id_hash(
+            self.has_template or self.is_template, get_string(source), context
+        )
 
     def _find_unit_bilingual(self, context: str, source: str) -> Tuple[Any, bool]:
+        id_hash = self._calculate_string_hash(context, source)
         try:
-            return (self._source_index[context, source], False)
+            return (self._unit_index[id_hash], False)
         except KeyError:
             raise UnitNotFound(context, source)
 
@@ -307,9 +403,12 @@ class TranslationFormat:
         Returns tuple (ttkit_unit, created) indicating whether returned unit is new one.
         """
         if self.has_template:
-            return self._find_unit_template(context)
+            return self._find_unit_monolingual(context, source)
         return self._find_unit_bilingual(context, source)
 
+    def ensure_index(self):
+        return self._unit_index
+
     def add_unit(self, ttkit_unit):
         """Add new unit to underlying store."""
         raise NotImplementedError()
@@ -341,18 +440,31 @@ class TranslationFormat:
         return self.store.units
 
     @cached_property
-    def mono_units(self):
+    def template_units(self):
         return [self.unit_class(self, None, unit) for unit in self.all_store_units]
 
+    def _get_all_bilingual_units(self):
+        return [self.unit_class(self, unit) for unit in self.all_store_units]
+
+    def _build_monolingual_unit(self, unit):
+        return self.unit_class(
+            self,
+            self.find_unit_template(unit.context, unit.source, unit.id_hash),
+            unit.template,
+        )
+
+    def _get_all_monolingual_units(self):
+        return [
+            self._build_monolingual_unit(unit)
+            for unit in self.template_store.template_units
+        ]
+
     @cached_property
     def all_units(self):
         """List of all units."""
         if not self.has_template:
-            return [self.unit_class(self, unit) for unit in self.all_store_units]
-        return [
-            self.unit_class(self, self.find_unit_mono(unit.context), unit.template)
-            for unit in self.template_store.mono_units
-        ]
+            return self._get_all_bilingual_units()
+        return self._get_all_monolingual_units()
 
     @property
     def content_units(self):
@@ -370,6 +482,9 @@ class TranslationFormat:
 
     def is_valid(self):
         """Check whether store seems to be valid."""
+        for unit in self.content_units:
+            # Just make sure that id_hash can be calculated
+            unit.id_hash
         return True
 
     @classmethod
@@ -398,6 +513,10 @@ class TranslationFormat:
     def get_language_bcp(code: str) -> str:
         return code.replace("_", "-")
 
+    @classmethod
+    def get_language_bcp_lower(cls, code: str) -> str:
+        return cls.get_language_bcp(code).lower()
+
     @staticmethod
     def get_language_posix_long(code: str) -> str:
         return EXPAND_LANGS.get(code, code)
@@ -429,10 +548,20 @@ class TranslationFormat:
         return sanitized.replace("_", "-r")
 
     @classmethod
-    def get_language_java(cls, code: str) -> str:
-        """Java doesn't use Hans/Hant, but rather TW/CN variants."""
+    def get_language_bcp_legacy(cls, code: str) -> str:
+        """BCP, but doesn't use Hans/Hant, but rather TW/CN variants."""
         return cls.get_language_bcp(cls.get_language_linux(code))
 
+    @classmethod
+    def get_language_appstore(cls, code: str) -> str:
+        """Apple App Store language codes."""
+        return cls.get_language_bcp(APPSTORE_CODES.get(code, code))
+
+    @classmethod
+    def get_language_googleplay(cls, code: str) -> str:
+        """Google Play language codes."""
+        return cls.get_language_bcp(GOOGLEPLAY_CODES.get(code, code))
+
     @classmethod
     def get_language_filename(cls, mask: str, code: str) -> str:
         """
@@ -458,6 +587,10 @@ class TranslationFormat:
 
         cls.create_new_file(filename, language, base, callback)
 
+    @classmethod
+    def get_new_file_content(cls):
+        return None
+
     @classmethod
     def create_new_file(
         cls,
@@ -521,7 +654,7 @@ class TranslationFormat:
             if self.is_template:
                 template_unit = unit
             else:
-                template_unit = self._find_unit_template(key)
+                template_unit = self._find_unit_monolingual(key, source)
         else:
             template_unit = None
         result = self.unit_class(self, unit, template_unit)
@@ -530,12 +663,12 @@ class TranslationFormat:
         # Update cached lookups
         if "all_units" in self.__dict__:
             self.all_units.append(result)
-        if "mono_units" in self.__dict__:
-            self.mono_units.append(mono_unit)
-        if "_source_index" in self.__dict__:
-            self._source_index[(result.context, result.source)] = result
-        if "_context_index" in self.__dict__:
-            self._context_index[mono_unit.context] = mono_unit
+        if "template_units" in self.__dict__:
+            self.template_units.append(mono_unit)
+        if "_unit_index" in self.__dict__:
+            self._unit_index[result.id_hash] = result
+        if "_template_index" in self.__dict__:
+            self._template_index[mono_unit.id_hash] = mono_unit
 
         return result
 
@@ -554,7 +687,7 @@ class TranslationFormat:
         """Removes unused strings, returning list of additional changed files."""
         if not self.template_store:
             return []
-        existing = {unit.context for unit in self.template_store.mono_units}
+        existing = {unit.context for unit in self.template_store.template_units}
         changed = False
 
         result = []
