@@ -52,14 +52,19 @@ from translate.storage.xml_extract.extract import (
 from weblate.checks.flags import Flags
 from weblate.formats.base import TranslationFormat
 from weblate.formats.helpers import BytesIOMode
-from weblate.formats.ttkit import TTKitUnit, XliffUnit
+from weblate.formats.ttkit import PoUnit, XliffUnit
+from weblate.trans.util import get_string
 from weblate.utils.errors import report_error
 from weblate.utils.state import STATE_APPROVED
 
 
-class ConvertUnit(TTKitUnit):
+class ConvertPoUnit(PoUnit):
+    id_hash_with_source: bool = True
+
     def is_translated(self):
         """Check whether unit is translated."""
+        if self.parent.is_template:
+            return bool(self.target)
         return self.unit is not None
 
     def is_fuzzy(self, fallback=False):
@@ -71,13 +76,11 @@ class ConvertUnit(TTKitUnit):
         return fallback
 
     @cached_property
-    def locations(self):
-        return ""
-
-    @cached_property
-    def context(self):
-        """Return context of message."""
-        return "".join(self.mainunit.getlocations())
+    def source(self):
+        """Return source string from a Translate Toolkit unit."""
+        if self.template:
+            return get_string(self.template.source)
+        return get_string(self.unit.source)
 
 
 class ConvertXliffUnit(XliffUnit):
@@ -91,6 +94,8 @@ class ConvertXliffUnit(XliffUnit):
 
     def is_translated(self):
         """Check whether unit is translated."""
+        if self.parent.is_template:
+            return bool(self.target)
         return self.unit is not None
 
     @cached_property
@@ -109,7 +114,8 @@ class ConvertFormat(TranslationFormat):
 
     monolingual = True
     can_add_unit = False
-    unit_class = ConvertUnit
+    can_delete_unit = False
+    unit_class = ConvertPoUnit
     autoaddon = {"weblate.flags.same_edit": {}}
     create_style = "copy"
 
@@ -127,7 +133,7 @@ class ConvertFormat(TranslationFormat):
 
     @staticmethod
     def needs_target_sync(template_store):
-        return True
+        return False
 
     def load(self, storefile, template_store):
         # Did we get file or filename?
@@ -202,6 +208,33 @@ class ConvertFormat(TranslationFormat):
         self.save()
         return []
 
+    @staticmethod
+    def convert_to_po(parser, template_store):
+        store = pofile()
+        if template_store:
+            parser.makeindex()
+            for unit in template_store.content_units:
+                thepo = store.addsourceunit(unit.source)
+                locations = unit.unit.getlocations()
+                thepo.addlocations(locations)
+                thepo.addnote(unit.unit.getnotes(), "developer")
+                for location in locations:
+                    try:
+                        translation = parser.locationindex[location]
+                        thepo.target = translation.source
+                        break
+                    except KeyError:
+                        continue
+        else:
+            for htmlunit in parser.units:
+                # Source file
+                thepo = store.addsourceunit(htmlunit.source)
+                thepo.target = htmlunit.source
+                thepo.addlocations(htmlunit.getlocations())
+                thepo.addnote(htmlunit.getnotes(), "developer")
+        store.removeduplicates("msgctxt")
+        return store
+
 
 class HTMLFormat(ConvertFormat):
     name = _("HTML file")
@@ -209,34 +242,11 @@ class HTMLFormat(ConvertFormat):
     format_id = "html"
     check_flags = ("safe-html", "strict-same")
 
-    @staticmethod
-    def needs_target_sync(template_store):
-        return False
-
-    @staticmethod
-    def convertfile(storefile, template_store):
-        store = pofile()
+    @classmethod
+    def convertfile(cls, storefile, template_store):
         # Fake input file with a blank filename
         htmlparser = htmlfile(inputfile=BytesIOMode("", storefile.read()))
-        for htmlunit in htmlparser.units:
-            locations = htmlunit.getlocations()
-            if template_store:
-                # Translation
-                template = template_store.find_unit_mono("".join(locations))
-                if template is None:
-                    # Skip locations not present in the source HTML file
-                    continue
-                # Create unit with matching source
-                thepo = store.addsourceunit(template.source)
-                thepo.target = htmlunit.source
-            else:
-                # Source file
-                thepo = store.addsourceunit(htmlunit.source)
-                thepo.target = htmlunit.source
-            thepo.addlocations(htmlunit.getlocations())
-            thepo.addnote(htmlunit.getnotes(), "developer")
-        store.removeduplicates("msgctxt")
-        return store
+        return cls.convert_to_po(htmlparser, template_store)
 
     def save_content(self, handle):
         """Store content to file."""
@@ -320,6 +330,10 @@ class OpenDocumentFormat(ConvertFormat):
         """Return most common file extension for format."""
         return "odt"
 
+    @staticmethod
+    def needs_target_sync(template_store):
+        return True
+
 
 class IDMLFormat(ConvertFormat):
     name = _("IDML file")
@@ -374,6 +388,10 @@ class IDMLFormat(ConvertFormat):
         """Return most common file extension for format."""
         return "idml"
 
+    @staticmethod
+    def needs_target_sync(template_store):
+        return True
+
 
 class WindowsRCFormat(ConvertFormat):
     name = _("RC file")
@@ -462,14 +480,9 @@ class PlainTextFormat(ConvertFormat):
     @classmethod
     def convertfile(cls, storefile, template_store):
         input_store = TxtFile(encoding="utf-8", flavour=cls.flavour)
-        input_store.filename = os.path.basename(storefile.name)
         input_store.parse(storefile.readlines())
-        store = pofile()
-        # This is translate.convert.txt2po.txt2po.convert_store
-        for source_unit in input_store.units:
-            target_unit = store.addsourceunit(source_unit.source)
-            target_unit.addlocations(source_unit.getlocations())
-        return store
+        input_store.filename = os.path.basename(storefile.name)
+        return cls.convert_to_po(input_store, template_store)
 
     def save_content(self, handle):
         """Store content to file."""
