@@ -88,7 +88,7 @@ class PoExporterTest(BaseTestCase):
         translation = Translation(language=lang, component=component, plural=plural)
         # Fake file format to avoid need for actual files
         translation.store = EmptyFormat(BytesIOMode("", b""))
-        unit = Unit(translation=translation, id_hash=-1, **kwargs)
+        unit = Unit(translation=translation, id_hash=-1, pk=-1, **kwargs)
         if source_info:
             for key, value in source_info.items():
                 setattr(unit, key, value)
@@ -118,6 +118,9 @@ class PoExporterTest(BaseTestCase):
     def test_unit_special(self):
         self.check_unit(source="bar\x1e\x1efoo", target="br\x1eff")
 
+    def test_unit_bom(self):
+        self.check_unit(source="For example ￾￾", target="For example ￾￾")
+
     def _encode(self, string):
         return string.encode("utf-8")
 
