@@ -1,5 +1,6 @@
 #
 # Copyright © 2012–2022 Michal Čihař <michal@cihar.com>
+# Copyright © 2022 WofWca <wofwca@protonmail.com>
 #
 # This file is part of Weblate <https://weblate.org/>
 #
@@ -47,7 +48,9 @@ from weblate.formats.ttkit import (
     PoFormat,
     PoXliffFormat,
     PropertiesFormat,
+    ResourceDictionaryFormat,
     RESXFormat,
+    RichXliffFormat,
     RubyYAMLFormat,
     StringsdictFormat,
     TBXFormat,
@@ -69,6 +72,7 @@ TEST_PO = get_test_file("cs.po")
 TEST_CSV = get_test_file("cs-mono.csv")
 TEST_CSV_NOHEAD = get_test_file("cs.csv")
 TEST_FLATXML = get_test_file("cs-flat.xml")
+TEST_RESOURCEDICTIONARY = get_test_file("cs.xaml")
 TEST_JSON = get_test_file("cs.json")
 TEST_NESTED_JSON = get_test_file("cs-nested.json")
 TEST_WEBEXT_JSON = get_test_file("cs-webext.json")
@@ -138,7 +142,7 @@ class AutoLoadTest(TestCase):
         self.single_test(TEST_ANDROID, AndroidFormat)
 
     def test_xliff(self):
-        self.single_test(TEST_XLIFF, XliffFormat)
+        self.single_test(TEST_XLIFF, RichXliffFormat)
 
     def test_resx(self):
         if "resx" not in FILE_FORMATS:
@@ -170,6 +174,7 @@ class AutoFormatTest(FixtureTestCase, TempDirMixin):
     FORMAT = AutodetectFormat
     FILE = TEST_PO
     BASE = TEST_POT
+    TEMPLATE = None
     MIME = "text/x-gettext-catalog"
     EXT = "po"
     COUNT = 4
@@ -187,11 +192,15 @@ class AutoFormatTest(FixtureTestCase, TempDirMixin):
     EDIT_TARGET = "Nazdar, svete!\n"
     MONOLINGUAL = False
 
+    @classmethod
+    def setUpClass(cls):
+        if cls.FORMAT.format_id not in FILE_FORMATS:
+            raise SkipTest(f"File format {cls.FORMAT!r} is not supported!")
+        super().setUpClass()
+
     def setUp(self):
         super().setUp()
         self.create_temp()
-        if self.FORMAT.format_id not in FILE_FORMATS:
-            raise SkipTest(f"File format {self.FORMAT!r} is not supported!")
 
     def tearDown(self):
         super().tearDown()
@@ -199,7 +208,10 @@ class AutoFormatTest(FixtureTestCase, TempDirMixin):
 
     def parse_file(self, filename):
         if self.MONOLINGUAL:
-            return self.FORMAT(filename, template_store=self.FORMAT(filename))
+            return self.FORMAT(
+                filename,
+                template_store=self.FORMAT(self.TEMPLATE or filename, is_template=True),
+            )
         return self.FORMAT(filename)
 
     def test_parse(self):
@@ -208,7 +220,7 @@ class AutoFormatTest(FixtureTestCase, TempDirMixin):
         self.assertEqual(storage.mimetype(), self.MIME)
         self.assertEqual(storage.extension(), self.EXT)
 
-    def test_save(self, edit=False):
+    def test_save(self, edit=None):
         # Read test content
         with open(self.FILE, "rb") as handle:
             testdata = handle.read()
@@ -225,7 +237,7 @@ class AutoFormatTest(FixtureTestCase, TempDirMixin):
 
         if edit:
             units = storage.all_units
-            units[self.EDIT_OFFSET].set_target(self.EDIT_TARGET)
+            units[self.EDIT_OFFSET].set_target(edit)
 
         # Save test file
         storage.save()
@@ -240,9 +252,10 @@ class AutoFormatTest(FixtureTestCase, TempDirMixin):
                 self.assert_same(newdata, testdata)
         else:
             self.assert_same(newdata, testdata)
+        return newdata
 
     def test_edit(self):
-        self.test_save(True)
+        return self.test_save(self.EDIT_TARGET)
 
     def assert_same(self, newdata, testdata):
         """Content aware comparison.
@@ -257,7 +270,7 @@ class AutoFormatTest(FixtureTestCase, TempDirMixin):
         unit, add = storage.find_unit(self.FIND_CONTEXT, self.FIND)
         self.assertFalse(add)
         if self.COUNT == 0:
-            self.assertIn(unit, None)
+            self.assertIsNone(unit)
         else:
             self.assertIsNotNone(unit)
             self.assertEqual(unit.target, self.FIND_MATCH)
@@ -318,11 +331,19 @@ class AutoFormatTest(FixtureTestCase, TempDirMixin):
             self.assertIn(self.NEW_UNIT_MATCH, newdata)
 
     def test_flags(self):
-        """Check flags on first translatable unit."""
-        storage = self.parse_file(self.FILE)
-        for unit in storage.content_units:
-            self.assertEqual(unit.flags, self.EXPECTED_FLAGS)
-            break
+        """
+        Check flags on corresponding translatable units.
+
+        If `EXPECTED_FLAGS` is a string instead of a list, check the first units.
+        """
+        units = self.parse_file(self.FILE).content_units
+        if type(self.EXPECTED_FLAGS) is list:
+            expected_list = self.EXPECTED_FLAGS
+        else:
+            expected_list = [self.EXPECTED_FLAGS]
+        for i, expected_flag in enumerate(expected_list):
+            unit = units[i]
+            self.assertEqual(unit.flags, expected_flag)
 
 
 class XMLMixin:
@@ -414,7 +435,7 @@ class PropertiesFormatTest(AutoFormatTest):
     COUNT = 12
     EXT = "properties"
     MASK = "java/swing_messages_*.properties"
-    EXPECTED_PATH = "java/swing_messages_cs-CZ.properties"
+    EXPECTED_PATH = "java/swing_messages_cs_CZ.properties"
     FIND = "IGNORE"
     FIND_CONTEXT = "IGNORE"
     FIND_MATCH = "Ignore"
@@ -437,7 +458,7 @@ class GWTFormatTest(AutoFormatTest):
     COUNT = 1
     EXT = "properties"
     MASK = "gwt/gwt_*.properties"
-    EXPECTED_PATH = "gwt/gwt_cs-CZ.properties"
+    EXPECTED_PATH = "gwt/gwt_cs_CZ.properties"
     FIND = "cartItems"
     FIND_CONTEXT = "cartItems"
     FIND_MATCH = (
@@ -515,7 +536,10 @@ class WebExtesionJSONFormatTest(JSONFormatTest):
     EXPECTED_PATH = "webextension/_locales/cs_CZ/messages.json"
     FIND_CONTEXT = "hello"
     NEW_UNIT_MATCH = b'\n    "key": {\n        "message": "Source string"\n    }\n'
-    EXPECTED_FLAGS = "placeholders:$URL$"
+    EXPECTED_FLAGS = [
+        "placeholders:$URL$,case-insensitive",
+        "placeholders:$COUNT$,case-insensitive",
+    ]
     MONOLINGUAL = True
 
 
@@ -586,7 +610,7 @@ class XliffFormatTest(XMLMixin, AutoFormatTest):
         b'<trans-unit xml:space="preserve" id="key" approved="no">',
         b"<source>Source string</source>",
     )
-    EXPECTED_FLAGS = "c-format, max-length:100, xml-text"
+    EXPECTED_FLAGS = "c-format, max-length:100"
 
     def test_set_state(self):
         # Read test content
@@ -609,7 +633,7 @@ class XliffFormatTest(XMLMixin, AutoFormatTest):
 
         # Verify the state is set
         with open(testfile) as handle:
-            self.assertIn("<target>test</target>", handle.read())
+            self.assertIn('<target state="translated">test</target>', handle.read())
 
         # Update first unit as fuzzy
         storage = self.parse_file(testfile)
@@ -625,7 +649,12 @@ class XliffFormatTest(XMLMixin, AutoFormatTest):
             )
 
 
-class XliffIdFormatTest(XliffFormatTest):
+class RichXliffFormatTest(XliffFormatTest):
+    FORMAT = RichXliffFormat
+    EXPECTED_FLAGS = "c-format, max-length:100, xml-text"
+
+
+class XliffIdFormatTest(RichXliffFormatTest):
     FILE = TEST_XLIFF_ID
     BASE = TEST_XLIFF_ID
     FIND_CONTEXT = "hello"
@@ -701,7 +730,7 @@ class PoXliffFormatTest(XMLMixin, AutoFormatTest):
         b'<trans-unit xml:space="preserve" id="key" approved="no">',
         b"<source>Source string</source>",
     )
-    EXPECTED_FLAGS = "c-format, max-length:100, xml-text"
+    EXPECTED_FLAGS = "c-format, max-length:100"
 
 
 class PoXliffFormatTest2(PoXliffFormatTest):
@@ -714,7 +743,6 @@ class PoXliffFormatTest2(PoXliffFormatTest):
             "font-size:22",
             "font-weight:bold",
             "max-size:100",
-            "xml-text",
         )
     )
     FIND_CONTEXT = "cs.po///2"
@@ -733,7 +761,7 @@ class RESXFormatTest(XMLMixin, AutoFormatTest):
     EXPECTED_PATH = "resx/cs-CZ.resx"
     FIND = "Hello"
     FIND_CONTEXT = "Hello"
-    FIND_MATCH = ""
+    FIND_MATCH = "Hello, world!"
     MATCH = "text/microsoft-resx"
     BASE = ""
     NEW_UNIT_MATCH = (
@@ -865,6 +893,24 @@ class FlatXMLFormatTest(AutoFormatTest):
     MONOLINGUAL = True
 
 
+class ResourceDictionaryFormatTest(AutoFormatTest):
+    FORMAT = ResourceDictionaryFormat
+    FILE = TEST_RESOURCEDICTIONARY
+    MIME = "application/xaml+xml"
+    COUNT = 2
+    EXT = "xaml"
+    MASK = "Languages/*.xaml"
+    BASE = TEST_RESOURCEDICTIONARY
+    EXPECTED_PATH = "Languages/cs-CZ.xaml"
+    MATCH = "hello"
+    FIND = "Hello World!"
+    FIND_CONTEXT = "hello_world"
+    FIND_MATCH = "Hello World!"
+    NEW_UNIT_MATCH = b'<system:String x:Key="key">Source string</system:String>\n'
+    EXPECTED_FLAGS = ""
+    MONOLINGUAL = True
+
+
 class INIFormatTest(AutoFormatTest):
     FORMAT = INIFormat
     FILE = TEST_INI
@@ -1008,7 +1054,9 @@ class XWikiPagePropertiesFormatTest(PropertiesFormatTest):
             translation_file, Language.objects.get(code="fr"), self.BASE
         )
         translation_data = self.FORMAT(
-            storefile=translation_file, template_store=storage, language_code="fr"
+            storefile=translation_file,
+            template_store=storage.template_store,
+            language_code="fr",
         )
         translation_units = translation_data.all_units
         self.assertEqual(self.COUNT, len(translation_units))
@@ -1056,6 +1104,7 @@ class XWikiFullPageFormatTest(AutoFormatTest):
     MATCH = "\n"
     NEW_UNIT_MATCH = b"\nkey=Source string\n"
     EXPECTED_FLAGS = ""
+    MONOLINGUAL = True
     EDIT_TARGET = """= Titre=\n"
                 "\n"
                 "* [[Bac à sable>>Sandbox.TestPage1]]\n"
@@ -1129,7 +1178,9 @@ class XWikiFullPageFormatTest(AutoFormatTest):
             translation_file, Language.objects.get(code="it"), self.BASE
         )
         translation_data = self.FORMAT(
-            storefile=translation_file, template_store=storage, language_code="it"
+            storefile=translation_file,
+            template_store=storage.template_store,
+            language_code="it",
         )
         translation_units = translation_data.all_units
         self.assertEqual(self.COUNT, len(translation_units))
