@@ -1,5 +1,6 @@
 #
 # Copyright © 2012–2022 Michal Čihař <michal@cihar.com>
+# Copyright © 2022 WofWca <wofwca@protonmail.com>
 #
 # This file is part of Weblate <https://weblate.org/>
 #
@@ -54,6 +55,7 @@ from weblate.formats.base import (
     UpdateError,
 )
 from weblate.lang.data import FORMULA_WITH_ZERO, ZERO_PLURAL_TYPES
+from weblate.lang.models import Plural
 from weblate.trans.util import (
     get_clean_env,
     get_string,
@@ -86,6 +88,8 @@ class TTKitUnit(TranslationUnit):
     def target(self):
         """Return target string from a Translate Toolkit unit."""
         if self.unit is None:
+            if self.parent.is_template:
+                return get_string(self.template.target)
             return ""
         return get_string(self.unit.target)
 
@@ -140,7 +144,7 @@ class TTKitUnit(TranslationUnit):
     def is_readonly(self):
         return not self.mainunit.istranslatable()
 
-    def set_target(self, target):
+    def set_target(self, target: Union[str, List[str]]):
         """Set translation unit target."""
         self._invalidate_target()
         if isinstance(target, list):
@@ -205,7 +209,7 @@ class KeyValueUnit(TTKitUnit):
             return not self.unit.isfuzzy() and self.unit.value != ""
         return self.unit.istranslated()
 
-    def set_target(self, target):
+    def set_target(self, target: Union[str, List[str]]):
         """Set translation unit target."""
         super().set_target(target)
         # Propagate to value so that searializing of empty values works correctly
@@ -219,6 +223,7 @@ class TTKitFormat(TranslationFormat):
     set_context_bilingual = True
     # Use settarget/setsource to set language as well
     use_settarget = False
+    force_encoding = None
 
     def __init__(
         self,
@@ -250,6 +255,8 @@ class TTKitFormat(TranslationFormat):
 
     def fixup(self, store):
         """Perform optional fixups on store."""
+        if self.force_encoding is not None:
+            store.encoding = self.force_encoding
         return
 
     def load(self, storefile, template_store):
@@ -279,13 +286,19 @@ class TTKitFormat(TranslationFormat):
     def get_class_kwargs():
         return {}
 
-    def parse_store(self, storefile):
-        """Parse the store."""
-        store = self.get_class()(**self.get_class_kwargs())
+    def get_store_instance(self, **kwargs):
+        kwargs.update(self.get_class_kwargs())
+        store = self.get_class()(**kwargs)
 
         # Apply possible fixups
         self.fixup(store)
 
+        return store
+
+    def parse_store(self, storefile):
+        """Parse the store."""
+        store = self.get_store_instance()
+
         # Read the content
         if isinstance(storefile, str):
             with open(storefile, "rb") as handle:
@@ -330,6 +343,8 @@ class TTKitFormat(TranslationFormat):
         In some cases Translate Toolkit happily "parses" the file, even though it really
         did not do so (e.g. gettext parser on a random textfile).
         """
+        if not super().is_valid():
+            return False
         if self.store is None:
             return False
 
@@ -452,11 +467,11 @@ class TTKitFormat(TranslationFormat):
                 callback(store)
             store.untranslate_store(language)
             store.store.savefile(filename)
-        elif cls.new_translation is None:
-            raise ValueError("Not supported")
-        else:
+        elif cls.new_translation is not None:
             with open(filename, "wb") as output:
                 output.write(cls.get_new_file_content())
+        else:
+            raise ValueError("Not supported")
 
     @classmethod
     def is_valid_base_for_new(
@@ -585,7 +600,7 @@ class PoMonoUnit(PoUnit):
                 result.append(context)
         return "\n".join(result)
 
-    def set_target(self, target):
+    def set_target(self, target: Union[str, List[str]]):
         """Set translation unit target."""
         # Add blank msgid_plural to store plural
         if isinstance(target, (list, multistring)) and not self.unit.hasplural():
@@ -601,71 +616,18 @@ class XliffUnit(TTKitUnit):
     is context in other formats.
     """
 
-    @cached_property
-    def source(self):
-        """Return source string from a Translate Toolkit unit."""
-        if self.template is not None:
-            # Use target if set, otherwise fall back to source
-            if self.template.target:
-                return rich_to_xliff_string(self.template.rich_target)
-            return rich_to_xliff_string(self.template.rich_source)
-        return rich_to_xliff_string(self.unit.rich_source)
-
-    @cached_property
-    def target(self):
-        """Return target string from a Translate Toolkit unit."""
-        if self.unit is None:
-            return ""
-
-        # Use source for monolingual base if target is not set
-        if self.unit.target is None:
-            if self.parent.is_template:
-                return rich_to_xliff_string(self.unit.rich_source)
-            return ""
-
-        return rich_to_xliff_string(self.unit.rich_target)
-
-    @cached_property
-    def flags(self):
-        flags = Flags(super().flags)
-        flags.merge("xml-text")
-        return flags.format()
-
     def _invalidate_target(self):
         """Invalidate target cache."""
         super()._invalidate_target()
         if "xliff_node" in self.__dict__:
             del self.__dict__["xliff_node"]
 
-    def set_target(self, target):
-        """Set translation unit target."""
-        self._invalidate_target()
-        # Delete the empty target element
-        if not target:
-            xmlnode = self.get_xliff_node()
-            if xmlnode is not None:
-                xmlnode.getparent().remove(xmlnode)
-            return
-        try:
-            converted = xliff_string_to_rich(target)
-        except (XMLSyntaxError, TypeError, KeyError):
-            # KeyError happens on missing attribute
-            converted = [target]
-        if self.template is not None:
-            if self.parent.is_template:
-                # Use source for monolingual files if editing template
-                self.unit.rich_source = converted
-            elif self.unit.source:
-                # Update source to match current source
-                self.unit.rich_source = self.template.rich_source
-        # Always set target, even in monolingual template
-        self.unit.rich_target = converted
+    @staticmethod
+    def get_unit_node(unit, element: str = "target"):
+        return unit.xmlelement.find(unit.namespaced(element))
 
     def get_xliff_node(self):
-        try:
-            return self.unit.getlanguageNode(lang=None, index=1)
-        except AttributeError:
-            return None
+        return self.get_unit_node(self.unit)
 
     @cached_property
     def xliff_node(self):
@@ -719,14 +681,14 @@ class XliffUnit(TTKitUnit):
         if state == STATE_FUZZY:
             # Always set state for fuzzy
             self.xliff_node.set("state", "needs-translation")
+        elif state == STATE_TRANSLATED:
+            # Always set state for translated
+            self.xliff_node.set("state", "translated")
+        elif state == STATE_APPROVED:
+            self.xliff_node.set("state", "final")
         elif self.xliff_state:
             # Only update state if it exists
-            if state == STATE_APPROVED:
-                self.xliff_node.set("state", "final")
-            elif state == STATE_TRANSLATED:
-                self.xliff_node.set("state", "translated")
-            else:
-                self.xliff_node.set("state", "new")
+            self.xliff_node.set("state", "new")
 
     def is_approved(self, fallback=False):
         """Check whether unit is approved."""
@@ -744,10 +706,114 @@ class XliffUnit(TTKitUnit):
         """
         return (
             not self.mainunit.isheader()
-            and bool(rich_to_xliff_string(self.mainunit.rich_source))
+            and bool(self.source)
             and not self.mainunit.isobsolete()
         )
 
+    def set_target(self, target: Union[str, List[str]]):
+        """Set translation unit target."""
+        if self.get_unit_node(self.unit, "source") is None:
+            # Make sure source element is present, otherwise it breaks
+            # translate-toolkit expectations.
+            self.unit.set_source_dom(
+                self.unit.createlanguageNode(self.parent.source_language, "", "source")
+            )
+
+        self._invalidate_target()
+        if isinstance(target, list):
+            target = multistring(target)
+        if self.template is not None:
+            if self.parent.is_template:
+                # Use source for monolingual files if editing template
+                self.unit.source = target
+            elif self.unit.source:
+                # Update source to match current source
+                self.unit.source = self.template.source
+        # Always set target, even in monolingual template
+        self.unit.target = target
+
+    @cached_property
+    def source(self):
+        """Return source string from a Translate Toolkit unit."""
+        if self.template is not None:
+            # Use target if set, otherwise fall back to source
+            if self.template.target:
+                return get_string(self.template.target)
+            return get_string(self.template.source)
+        return get_string(self.unit.source)
+
+    @cached_property
+    def target(self):
+        """Return target string from a Translate Toolkit unit."""
+        if self.unit is None:
+            return ""
+
+        # Use source for monolingual base if target is not set
+        if self.unit.target is None:
+            if self.parent.is_template:
+                return get_string(self.unit.source)
+            return ""
+
+        return get_string(self.unit.target)
+
+
+class RichXliffUnit(XliffUnit):
+    """Wrapper unit for XLIFF with XML elements."""
+
+    @cached_property
+    def source(self):
+        """Return source string from a Translate Toolkit unit."""
+        if self.template is not None:
+            # Use target if set, otherwise fall back to source
+            if self.template.target:
+                return rich_to_xliff_string(self.template.rich_target)
+            return rich_to_xliff_string(self.template.rich_source)
+        return rich_to_xliff_string(self.unit.rich_source)
+
+    @cached_property
+    def target(self):
+        """Return target string from a Translate Toolkit unit."""
+        if self.unit is None:
+            return ""
+
+        # Use source for monolingual base if target is not set
+        if self.unit.target is None:
+            if self.parent.is_template:
+                return rich_to_xliff_string(self.unit.rich_source)
+            return ""
+
+        return rich_to_xliff_string(self.unit.rich_target)
+
+    @cached_property
+    def flags(self):
+        flags = Flags(super().flags)
+        flags.merge("xml-text")
+        return flags.format()
+
+    def set_target(self, target: Union[str, List[str]]):
+        """Set translation unit target."""
+        self._invalidate_target()
+        # Delete the empty target element
+        if not target:
+            xmlnode = self.get_xliff_node()
+            if xmlnode is not None:
+                xmlnode.getparent().remove(xmlnode)
+            return
+        try:
+            converted = xliff_string_to_rich(target)
+        except (XMLSyntaxError, TypeError, KeyError):
+            # KeyError happens on missing attribute
+            converted = [target]
+        if self.template is not None:
+            if self.parent.is_template:
+                # Use source for monolingual files if editing template
+                self.unit.rich_source = converted
+            elif self.unit.source:
+                # Update source to match current source
+                self.unit.rich_source = self.template.rich_source
+        # Always set target, even in monolingual template
+        self.unit.rich_target = converted
+
 
 class FlatXMLUnit(TTKitUnit):
     @cached_property
@@ -843,32 +909,19 @@ class JSONUnit(MonolingualSimpleUnit):
         return context
 
 
-class WebExtensionJSONUnit(JSONUnit):
+class PlaceholdersJSONUnit(JSONUnit):
     @cached_property
     def flags(self):
         placeholders = self.mainunit.placeholders
         if not placeholders:
             return ""
-        return "placeholders:{}".format(
+        return "placeholders:{},case-insensitive".format(
             ":".join(
                 Flags.format_value(f"${key.upper()}$") for key in placeholders.keys()
             )
         )
 
 
-class ARBJSONUnit(JSONUnit):
-    @cached_property
-    def flags(self):
-        placeholders = self.mainunit.placeholders
-        if not placeholders:
-            return ""
-        return "placeholders:{}".format(
-            ":".join(
-                Flags.format_value(f"{{{key.upper()}}}") for key in placeholders.keys()
-            )
-        )
-
-
 class CSVUnit(MonolingualSimpleUnit):
     @staticmethod
     def unescape_csv(string):
@@ -890,14 +943,19 @@ class CSVUnit(MonolingualSimpleUnit):
 
     @cached_property
     def context(self):
+        def get_context(unit):
+            if unit.id:
+                return unit.id
+            if unit.context:
+                return unit.context
+            return unit.getid()
+
         # Needed to avoid Translate Toolkit construct ID
         # as context\04source
         if self.template is not None:
-            if self.template.id:
-                return self.template.id
-            if self.template.context:
-                return self.template.context
-            return self.template.getid()
+            return get_context(self.template)
+        if self.parent.is_template:
+            return get_context(self.unit)
         return self.unescape_csv(self.mainunit.getcontext())
 
     @cached_property
@@ -983,8 +1041,6 @@ class BasePoFormat(TTKitFormat, BilingualUpdateMixin):
     @classmethod
     def get_plural(cls, language, store=None):
         """Return matching plural object."""
-        from weblate.lang.models import Plural
-
         if store:
             header = store.store.parseheader()
         else:
@@ -1141,9 +1197,9 @@ class TSFormat(TTKitFormat):
 
 class XliffFormat(TTKitFormat):
     name = _("XLIFF translation file")
-    format_id = "xliff"
+    format_id = "plainxliff"
     loader = xlifffile
-    autoload: Tuple[str, ...] = ("*.xlf", "*.xliff", "*.sdlxliff", "*.mxliff")
+    autoload = ()
     unit_class = XliffUnit
     language_format = "bcp"
     use_settarget = True
@@ -1151,12 +1207,9 @@ class XliffFormat(TTKitFormat):
     def untranslate_unit(self, unit, plural, fuzzy: bool):
         super().untranslate_unit(unit, plural, fuzzy)
         # Delete empty <target/> tag
-        try:
-            xmlnode = self.unit.getlanguageNode(lang=None, index=1)
-            if xmlnode is not None:
-                xmlnode.getparent().remove(xmlnode)
-        except AttributeError:
-            pass
+        xmlnode = self.unit_class.get_unit_node(unit)
+        if xmlnode is not None:
+            xmlnode.getparent().remove(xmlnode)
 
     def construct_unit(self, source: str):
         unit = super().construct_unit(source)
@@ -1179,6 +1232,13 @@ class XliffFormat(TTKitFormat):
         return unit
 
 
+class RichXliffFormat(XliffFormat):
+    name = _("XLIFF with placeables support")
+    format_id = "xliff"
+    autoload: Tuple[str, ...] = ("*.xlf", "*.xliff", "*.sdlxliff", "*.mxliff")
+    unit_class = RichXliffUnit
+
+
 class PoXliffFormat(XliffFormat):
     name = _("XLIFF with gettext extensions")
     format_id = "poxliff"
@@ -1230,7 +1290,7 @@ class PropertiesUtf8Format(PropertiesBaseFormat):
     format_id = "properties-utf8"
     loader = ("properties", "javautf8file")
     new_translation = "\n"
-    language_format = "java"
+    language_format = "linux"
     check_flags = ("auto-java-messageformat",)
 
 
@@ -1238,32 +1298,22 @@ class PropertiesUtf16Format(PropertiesBaseFormat):
     name = _("Java Properties (UTF-16)")
     format_id = "properties-utf16"
     loader = ("properties", "javafile")
-    language_format = "java"
+    language_format = "linux"
     new_translation = "\n"
-
-    def fixup(self, store):
-        """Force encoding.
-
-        Translate Toolkit autodetection might fail in some cases.
-        """
-        store.encoding = "utf-16"
+    # Translate Toolkit autodetection might fail in some cases.
+    force_encoding = "utf-16"
 
 
 class PropertiesFormat(PropertiesBaseFormat):
     name = _("Java Properties (ISO 8859-1)")
     format_id = "properties"
     loader = ("properties", "javafile")
-    language_format = "java"
+    language_format = "linux"
     new_translation = "\n"
     autoload = ("*.properties",)
-
-    def fixup(self, store):
-        """Force encoding.
-
-        Java properties need to be ISO 8859-1, but Translate Toolkit converts them to
-        UTF-8.
-        """
-        store.encoding = "iso-8859-1"
+    # Java properties need to be ISO 8859-1, but Translate Toolkit converts
+    # them to UTF-8.
+    force_encoding = "iso-8859-1"
 
 
 class JoomlaFormat(PropertiesBaseFormat):
@@ -1281,6 +1331,7 @@ class GWTFormat(StringsFormat):
     loader = ("properties", "gwtfile")
     new_translation = "\n"
     check_flags = ("auto-java-messageformat",)
+    language_format = "linux"
 
 
 class PhpFormat(TTKitFormat):
@@ -1328,8 +1379,13 @@ class AndroidFormat(TTKitFormat):
     new_translation = '<?xml version="1.0" encoding="utf-8"?>\n<resources></resources>'
     autoload = ("strings*.xml", "values*.xml")
     language_format = "android"
-    check_flags = ("java-format",)
+    check_flags = ("java-printf-format",)
     autoaddon = {"weblate.cleanup.blank": {}}
+    plural_preference = (
+        Plural.SOURCE_ANDROID,
+        Plural.SOURCE_CLDR,
+        Plural.SOURCE_DEFAULT,
+    )
 
 
 class DictStoreMixin:
@@ -1376,7 +1432,7 @@ class WebExtensionJSONFormat(JSONFormat):
     loader = ("jsonl10n", "WebExtensionJsonFile")
     monolingual = True
     autoload = ("messages*.json",)
-    unit_class = WebExtensionJSONUnit
+    unit_class = PlaceholdersJSONUnit
 
 
 class I18NextFormat(JSONFormat):
@@ -1399,7 +1455,7 @@ class ARBFormat(JSONFormat):
     format_id = "arb"
     loader = ("jsonl10n", "ARBJsonFile")
     autoload = ("*.arb",)
-    unit_class = ARBJSONUnit
+    unit_class = PlaceholdersJSONUnit
     check_flags = ("icu-message-format",)
 
 
@@ -1409,7 +1465,7 @@ class CSVFormat(TTKitFormat):
     loader = ("csvl10n", "csvfile")
     unit_class = CSVUnit
     autoload: Tuple[str, ...] = ("*.csv",)
-    encoding = "auto"
+    force_encoding = "auto"
 
     def __init__(
         self,
@@ -1458,12 +1514,10 @@ class CSVFormat(TTKitFormat):
 
     def parse_store(self, storefile):
         """Parse the store."""
-        storeclass = self.get_class()
-
         content, filename = self.get_content_and_filename(storefile)
 
         # Parse file
-        store = storeclass()
+        store = self.get_store_instance()
         store.parse(content, sample_length=40000)
         # Did detection of headers work?
         if store.fieldnames != ["location", "source", "target"]:
@@ -1485,18 +1539,24 @@ class CSVFormat(TTKitFormat):
         return self.parse_simple_csv(content, filename)
 
     def parse_simple_csv(self, content, filename):
-        storeclass = self.get_class()
-        result = storeclass(fieldnames=["source", "target"], encoding=self.encoding)
+        result = self.get_store_instance(fieldnames=["source", "target"])
         result.parse(content, sample_length=None)
         result.filename = filename
         return result
 
 
+class CSVUtf8Format(CSVFormat):
+    name = _("CSV file (UTF-8)")
+    format_id = "csv-utf-8"
+    autoload = ()
+    force_encoding = "utf-8"
+
+
 class CSVSimpleFormat(CSVFormat):
     name = _("Simple CSV file")
     format_id = "csv-simple"
     autoload: Tuple[str, ...] = ("*.txt",)
-    encoding = "auto"
+    force_encoding = "auto"
 
     @staticmethod
     def extension():
@@ -1513,7 +1573,14 @@ class CSVSimpleFormat(CSVFormat):
 class CSVSimpleFormatISO(CSVSimpleFormat):
     name = _("Simple CSV file (ISO-8859-1)")
     format_id = "csv-simple-iso"
-    encoding = "iso-8859-1"
+    force_encoding = "iso-8859-1"
+    autoload = ()
+
+
+class CSVUtf8SimpleFormat(CSVSimpleFormat):
+    name = _("Simple CSV file (UTF-8)")
+    format_id = "csv-simple-utf-8"
+    force_encoding = "utf-8"
     autoload = ()
 
 
@@ -1570,7 +1637,9 @@ class DTDFormat(TTKitFormat):
 class SubtitleUnit(MonolingualIDUnit):
     @cached_property
     def source(self):
-        return self.template.source
+        if self.template is not None:
+            return self.template.source
+        return self.unit.source
 
     @cached_property
     def target(self):
@@ -1591,6 +1660,7 @@ class SubRipFormat(TTKitFormat):
     unit_class = SubtitleUnit
     autoload = ("*.srt",)
     monolingual = True
+    autoaddon = {"weblate.flags.same_edit": {}}
 
     @staticmethod
     def mimetype():
@@ -1628,6 +1698,19 @@ class FlatXMLFormat(TTKitFormat):
     new_translation = '<?xml version="1.0" encoding="utf-8"?>\n<root></root>'
 
 
+class ResourceDictionaryFormat(FlatXMLFormat):
+    name = _("ResourceDictionary file")
+    format_id = "resourcedictionary"
+    loader = ("resourcedictionary", "ResourceDictionaryFile")
+    check_flags = ("c-sharp-format",)
+    language_format = "bcp_legacy"
+    new_translation = """<ResourceDictionary
+  xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
+  xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
+  xmlns:system="clr-namespace:System;assembly=mscorlib">
+</ResourceDictionary>"""
+
+
 class INIFormat(TTKitFormat):
     name = _("INI file")
     format_id = "ini"
@@ -1720,10 +1803,11 @@ class XWikiPropertiesFormat(PropertiesBaseFormat):
     name = "XWiki Java Properties"
     format_id = "xwiki-java-properties"
     loader = ("properties", "xwikifile")
-    language_format = "java"
+    language_format = "bcp_legacy"
     autoload = ("*.properties",)
     new_translation = "\n"
     can_add_unit: bool = False
+    can_delete_unit: bool = False
     set_context_bilingual: bool = True
 
     # Ensure that untranslated units are saved too as missing properties and
@@ -1774,15 +1858,7 @@ class XWikiPagePropertiesFormat(XWikiPropertiesFormat):
     name = "XWiki Page Properties"
     format_id = "xwiki-page-properties"
     loader = ("properties", "XWikiPageProperties")
-    language_format = "java"
-
-    def fixup(self, store):
-        """Fix encoding.
-
-        Force encoding to UTF-8 since we inherit from XWikiProperties which force
-        for ISO-8859-1.
-        """
-        store.encoding = "utf-8"
+    force_encoding = "utf-8"
 
     def save_content(self, handle):
         if self.store.root is None:
@@ -1800,7 +1876,6 @@ class XWikiFullPageFormat(XWikiPagePropertiesFormat):
     name = "XWiki Full Page"
     format_id = "xwiki-fullpage"
     loader = ("properties", "XWikiFullPage")
-    language_format = "java"
 
 
 class TBXUnit(TTKitUnit):
@@ -1853,7 +1928,7 @@ class PropertiesMi18nFormat(PropertiesUtf8Format):
     name = _("mi18n lang file")
     format_id = "mi18n-lang"
     new_translation = "\n"
-    language_format = "java"
+    language_format = "bcp_legacy"
     check_flags = ("es-format",)
     monolingual = True
 
@@ -1907,9 +1982,19 @@ class StringsdictFormat(DictStoreMixin, TTKitFormat):
 
 
 class FluentUnit(MonolingualSimpleUnit):
-    def set_target(self, target):
+    def set_target(self, target: Union[str, List[str]]):
         super().set_target(target)
         self.unit.source = target
+        # This triggers serialization discovering any syntax issues
+        self.unit.to_entry()
+
+    @cached_property
+    def flags(self):
+        placeables = self.mainunit.getplaceables()
+        if not placeables:
+            return ""
+        placeables.insert(0, "placeholders")
+        return Flags.format_flag(tuple(placeables))
 
 
 class FluentFormat(TTKitFormat):
@@ -1939,19 +2024,3 @@ class FluentFormat(TTKitFormat):
         unit = super().create_unit(key, source, target)
         unit.source = unit.target
         return unit
-
-    def is_valid(self):
-        """Check whether store seems to be valid."""
-        # Workaround for https://github.com/translate/translate/issues/4615
-        for unit in self.store.units:
-            errors = unit.geterrors()
-            if errors:
-                raise ValueError(
-                    "Syntax error: {}".format(
-                        ", ".join(
-                            f"{errorname}: {errortext}"
-                            for errorname, errortext in errors.items()
-                        )
-                    )
-                )
-        return super().is_valid()
