@@ -174,7 +174,7 @@ class TextUnit(TranslationUnit):
             return self.mainunit.flags
         return ""
 
-    def set_target(self, target):
+    def set_target(self, target: Union[str, List[str]]):
         """Set translation unit target."""
         self._invalidate_target()
         self.unit.text = target
@@ -188,10 +188,11 @@ class AppStoreFormat(TranslationFormat):
     name = _("App store metadata files")
     format_id = "appstore"
     can_add_unit = False
+    can_delete_unit = True
     monolingual = True
     unit_class = TextUnit
     simple_filename = False
-    language_format = "java"
+    language_format = "googleplay"
     create_style = "directory"
 
     def load(self, storefile, template_store):
