@@ -24,7 +24,15 @@ from weblate_language_data.ambiguous import AMBIGUOUS
 
 NO_CODE_LANGUAGES = {lang[0] for lang in languages.LANGUAGES}
 
-UNDERSCORE_EXCEPTIONS = {"nb_NO", "zh_Hant", "zh_Hans", "be_Latn", "ro_MD", "pt_BR"}
+UNDERSCORE_EXCEPTIONS = {
+    "nb_NO",
+    "zh_Hant",
+    "zh_Hans",
+    "be_Latn",
+    "ro_MD",
+    "pt_BR",
+    "pa_PK",
+}
 AT_EXCEPTIONS = {"ca@valencia"}
 
 
@@ -131,6 +139,7 @@ ONE_TWO_FEW_MANY_OTHER_PLURALS = (
     "n==1 ? 0 : n==2 ? 1 : n<7 ? 2 : n<11 ? 3 : 4",
     "n==1 ? 0 : n==2 ? 1 : (n>2 && n<7) ? 2 :(n>6 && n<11) ? 3 : 4",
     "(n % 10 == 1 && n % 100 != 11 && n % 100 != 71 && n % 100 != 91) ? 0 : ((n % 10 == 2 && n % 100 != 12 && n % 100 != 72 && n % 100 != 92) ? 1 : ((((n % 10 == 3 || n % 10 == 4) || n % 10 == 9) && (n % 100 < 10 || n % 100 > 19) && (n % 100 < 70 || n % 100 > 79) && (n % 100 < 90 || n % 100 > 99)) ? 2 : ((n != 0 && n % 1000000 == 0) ? 3 : 4)))",
+    "(n == 1) ? 0 : ((n == 2) ? 1 : ((n == 0 || n % 100 >= 3 && n % 100 <= 10) ? 2 : ((n % 100 >= 11 && n % 100 <= 19) ? 3 : 4)))",
 )
 
 ONE_FEW_MANY_OTHER_PLURALS = (
@@ -151,6 +160,10 @@ ZERO_ONE_TWO_FEW_MANY_OTHER_PLURALS = (
     "n==0 ? 0 : n==1 ? 1 : n==2 ? 2 : (n>2 && n<7) ? 3 :(n>6 && n<11) ? 4 : 5",
     "n==0 ? 0 : (n % 10 == 1 && n % 100 != 11 && n % 100 != 71 && n % 100 != 91) ? 1 : ((n % 10 == 2 && n % 100 != 12 && n % 100 != 72 && n % 100 != 92) ? 2 : ((((n % 10 == 3 || n % 10 == 4) || n % 10 == 9) && (n % 100 < 10 || n % 100 > 19) && (n % 100 < 70 || n % 100 > 79) && (n % 100 < 90 || n % 100 > 99)) ? 3 : ((n != 0 && n % 1000000 == 0) ? 4 : 5)))",
 )
+ONE_MANY_OTHER_PLURALS = (
+    "(n == 1) ? 0 : ((n != 0 && n % 1000000 == 0) ? 1 : 2)",
+    "(n == 0 || n == 1) ? 0 : ((n != 0 && n % 1000000 == 0) ? 1 : 2)",
+)
 
 ZERO_OTHER_PLURALS = ("n==0 ? 0 : 1",)
 
@@ -175,6 +188,7 @@ PLURAL_ZERO_ONE_FEW_OTHER = 16
 PLURAL_ZERO_ONE_TWO_FEW_OTHER = 17
 PLURAL_ZERO_ONE_TWO_OTHER = 18
 PLURAL_ZERO_ONE_FEW_MANY_OTHER = 19
+PLURAL_ONE_MANY_OTHER = 20
 PLURAL_UNKNOWN = 666
 
 # Extra zero plural handling for stringsdict
@@ -189,6 +203,7 @@ FORMULA_WITH_ZERO = {
     "n > 1": "n==0 ? 0 : n>1 ? 1 : 2",
     "n != 1": "n==0 ? 0 : n!=1 ? 1 : 2",
     "(n == 0 || n == 1) ? 0 : ((n >= 2 && n <= 10) ? 1 : 2)": "n == 0 ? 0 : n == 1 ? 1 : ((n >= 2 && n <= 10) ? 2 : 3)",
+    "n == 1 ? 0 : n == 2 ? 1 : 2": "n == 0 ? 0 : n == 1 ? 1 : n == 2 ? 2 : 3",
     "n==1 || n%10==1 ? 0 : 1": "n == 0 ? 0 : n==1 || n%10==1 ? 1 : 2",
     "n==1 ? 0 : n==2 ? 1 : 2": "n==0 ? 0 : n==1 ? 1 : n==2 ? 2 : 3",
     "(n==1 || n==11) ? 0 : (n==2 || n==12) ? 1 : (n > 2 && n < 20) ? 2 : 3": "n==0 ? 0 : (n==1 || n==11) ? 1 : (n==2 || n==12) ? 2 : (n > 2 && n < 20) ? 3 : 4",
@@ -240,6 +255,7 @@ PLURAL_MAPPINGS = (
         PLURAL_ZERO_ONE_TWO_FEW_MANY_OTHER,
     ),
     (nospace_set(ZERO_OTHER_PLURALS), PLURAL_ZERO_OTHER),
+    (nospace_set(ONE_MANY_OTHER_PLURALS), PLURAL_ONE_MANY_OTHER),
 )
 
 # Plural names mapping
@@ -355,4 +371,9 @@ PLURAL_NAMES = {
         pgettext_lazy("Plural form description", "Zero"),
         pgettext_lazy("Plural form description", "Other"),
     ),
+    PLURAL_ONE_MANY_OTHER: (
+        pgettext_lazy("Plural form description", "One"),
+        pgettext_lazy("Plural form description", "Many"),
+        pgettext_lazy("Plural form description", "Other"),
+    ),
 }
