@@ -29,14 +29,13 @@ from django.db.models import Q
 from django.db.utils import OperationalError
 from django.urls import reverse
 from django.utils.functional import cached_property
-from django.utils.safestring import mark_safe
+from django.utils.html import format_html
 from django.utils.translation import gettext as _
 from django.utils.translation import gettext_lazy, pgettext_lazy
 from django.utils.translation.trans_real import parse_accept_lang_header
 from weblate_language_data.aliases import ALIASES
 from weblate_language_data.countries import DEFAULT_LANGS
-from weblate_language_data.languages import LANGUAGES
-from weblate_language_data.plurals import EXTRAPLURALS
+from weblate_language_data.plurals import CLDRPLURALS, EXTRAPLURALS
 from weblate_language_data.rtl import RTL_LANGS
 
 from weblate.lang import data
@@ -374,13 +373,21 @@ class LanguageManager(models.Manager.from_queryset(LanguageQuerySet)):
     @cached_property
     def default_language(self):
         """Return English language object."""
-        return self.get(code=settings.DEFAULT_LANGUAGE, skip_cache=True)
+        # Intentionally skip population field here as it
+        # might not yet be created during migrations.
+        # TODO: Drop this in Weblate 5.1
+        return self.only("name", "code", "direction").get(
+            code=settings.DEFAULT_LANGUAGE, skip_cache=True
+        )
 
     def setup(self, update, logger=lambda x: x):
         """Create basic set of languages.
 
         It is based on languages defined in the languages-data repo.
         """
+        from weblate_language_data.languages import LANGUAGES
+        from weblate_language_data.population import POPULATION
+
         # Invalidate cache, we might change languages
         self.flush_object_cache()
         languages = {
@@ -389,17 +396,26 @@ class LanguageManager(models.Manager.from_queryset(LanguageQuerySet)):
         plurals = {}
         # Create Weblate languages
         for code, name, nplurals, plural_formula in LANGUAGES:
+            population = POPULATION[code]
+
             if code in languages:
                 lang = languages[code]
             else:
-                languages[code] = lang = self.create(code=code, name=name)
+                languages[code] = lang = self.create(
+                    code=code, name=name, population=population
+                )
                 logger(f"Created language {code}")
 
             direction = lang.guess_direction()
             # Should we update existing?
-            if update and (lang.name != name or lang.direction != direction):
+            if update and (
+                lang.name != name
+                or lang.direction != direction
+                or lang.population != population
+            ):
                 lang.name = name
                 lang.direction = direction
+                lang.population = population
                 logger(f"Updated language {code}")
                 lang.save()
 
@@ -439,32 +455,40 @@ class LanguageManager(models.Manager.from_queryset(LanguageQuerySet)):
                 )
 
         # Create addditiona plurals
-        for code, _unused, nplurals, plural_formula in EXTRAPLURALS:
-            lang = languages[code]
+        extra_plurals = (
+            (Plural.SOURCE_GETTEXT, EXTRAPLURALS),
+            (Plural.SOURCE_CLDR, CLDRPLURALS),
+        )
+        for source, definitions in extra_plurals:
+            for code, _unused, nplurals, plural_formula in definitions:
+                lang = languages[code]
 
-            for plural in plurals[code][Plural.SOURCE_GETTEXT]:
-                try:
-                    if plural.same_plural(nplurals, plural_formula):
-                        break
-                except ValueError:
-                    # Fall back to string compare if parsing failed
-                    if plural.number == nplurals and plural.formula == plural_formula:
-                        break
-            else:
-                plural = lang.plural_set.create(
-                    source=Plural.SOURCE_GETTEXT,
-                    number=nplurals,
-                    formula=plural_formula,
-                    type=get_plural_type(lang.base_code, plural_formula),
-                )
-                logger(f"Created plural {plural_formula} for language {code}")
+                for plural in plurals[code][source]:
+                    try:
+                        if plural.same_plural(nplurals, plural_formula):
+                            break
+                    except ValueError:
+                        # Fall back to string compare if parsing failed
+                        if (
+                            plural.number == nplurals
+                            and plural.formula == plural_formula
+                        ):
+                            break
+                else:
+                    plural = lang.plural_set.create(
+                        source=source,
+                        number=nplurals,
+                        formula=plural_formula,
+                        type=get_plural_type(lang.base_code, plural_formula),
+                    )
+                    logger(f"Created plural {plural_formula} for language {code}")
 
 
 def setup_lang(sender, **kwargs):
     """Hook for creating basic set of languages on database migration."""
     if settings.UPDATE_LANGUAGES:
         with transaction.atomic():
-            Language.objects.setup(False)
+            Language.objects.setup(True)
 
 
 class Language(models.Model, CacheKeyMixin):
@@ -486,6 +510,11 @@ class Language(models.Model, CacheKeyMixin):
             ("rtl", gettext_lazy("Right to left")),
         ),
     )
+    population = models.BigIntegerField(
+        gettext_lazy("Number of speakers"),
+        help_text=gettext_lazy("Number of people speaking this language."),
+        default=0,
+    )
 
     objects = LanguageManager()
 
@@ -537,7 +566,7 @@ class Language(models.Model, CacheKeyMixin):
 
         Includes language and direction HTML.
         """
-        return mark_safe(f'lang="{self.code}" dir="{self.direction}"')
+        return format_html('lang="{}" dir="{}"', self.code, self.direction)
 
     @cached_property
     def base_code(self):
@@ -641,6 +670,10 @@ class Plural(models.Model):
             data.PLURAL_ZERO_ONE_FEW_MANY_OTHER,
             pgettext_lazy("Plural type", "Zero/one/few/many/other"),
         ),
+        (
+            data.PLURAL_ONE_MANY_OTHER,
+            pgettext_lazy("Plural type", "One/many/other"),
+        ),
         (
             data.PLURAL_UNKNOWN,
             pgettext_lazy("Plural type", "Unknown"),
@@ -650,6 +683,8 @@ class Plural(models.Model):
     SOURCE_GETTEXT = 1
     SOURCE_MANUAL = 2
     SOURCE_STRINGSDICT = 3
+    SOURCE_CLDR = 4
+    SOURCE_ANDROID = 5
     source = models.SmallIntegerField(
         default=SOURCE_DEFAULT,
         verbose_name=gettext_lazy("Plural definition source"),
@@ -657,6 +692,8 @@ class Plural(models.Model):
             (SOURCE_DEFAULT, gettext_lazy("Default plural")),
             (SOURCE_GETTEXT, gettext_lazy("gettext plural formula")),
             (SOURCE_STRINGSDICT, gettext_lazy("stringsdict plural")),
+            (SOURCE_CLDR, gettext_lazy("CLDR v38+ plural")),
+            (SOURCE_ANDROID, gettext_lazy("Android plural")),
             (SOURCE_MANUAL, gettext_lazy("Manually entered formula")),
         ),
     )
@@ -710,7 +747,10 @@ class Plural(models.Model):
 
     @cached_property
     def plural_function(self):
-        return gettext.c2py(self.formula if self.formula else "0")
+        try:
+            return gettext.c2py(self.formula if self.formula else "0")
+        except ValueError as error:
+            raise ValueError(f"Failed to compile formula '{self.formula}': {error}")
 
     @cached_property
     def examples(self):
@@ -759,7 +799,8 @@ class Plural(models.Model):
 
     def get_plural_label(self, idx):
         """Return label for plural form."""
-        return PLURAL_TITLE.format(
+        return format_html(
+            PLURAL_TITLE,
             name=self.get_plural_name(idx),
             icon=icon("info.svg"),
             # Translators: Label for plurals with example counts
