@@ -29,7 +29,7 @@ from django.test import TestCase
 from django.urls import reverse
 from django.utils.translation import activate
 from weblate_language_data.languages import LANGUAGES
-from weblate_language_data.plurals import EXTRAPLURALS
+from weblate_language_data.plurals import CLDRPLURALS, EXTRAPLURALS
 
 from weblate.lang import data
 from weblate.lang.models import Language, Plural, get_plural_type
@@ -311,7 +311,7 @@ class VerifyPluralsTest(TestCase):
 
     @staticmethod
     def all_data():
-        return chain(LANGUAGES, EXTRAPLURALS)
+        return chain(LANGUAGES, EXTRAPLURALS, CLDRPLURALS)
 
     def test_valid(self):
         """Validate that we can name all plural formulas."""
@@ -319,7 +319,7 @@ class VerifyPluralsTest(TestCase):
             self.assertNotEqual(
                 get_plural_type(code.replace("_", "-").split("-")[0], plural_formula),
                 data.PLURAL_UNKNOWN,
-                f"Can not guess plural type for {code} ({plural_formula})",
+                f"Can not guess plural type for {code}: {plural_formula}",
             )
 
     def test_formula(self):
@@ -336,7 +336,7 @@ class VerifyPluralsTest(TestCase):
             self.assertEqual(
                 calculated,
                 nplurals,
-                "Invalid nplurals for {}: {} ({}, {})".format(
+                "Invalid nplurals for {}: calculated={} (number={}, formula={})".format(
                     code, calculated, nplurals, plural_formula
                 ),
             )
@@ -392,6 +392,7 @@ class LanguagesViewTest(FixtureTestCase):
                 "direction": "ltr",
                 "number": "2",
                 "formula": "n != 1",
+                "population": 10,
             },
         )
         self.assertRedirects(response, reverse("show_language", kwargs={"lang": "xx"}))
@@ -412,7 +413,7 @@ class LanguagesViewTest(FixtureTestCase):
         self.user.save()
         response = self.client.post(
             reverse("edit-language", kwargs={"pk": language.pk}),
-            {"code": "xx", "name": "XX", "direction": "ltr"},
+            {"code": "xx", "name": "XX", "direction": "ltr", "population": 10},
         )
         self.assertRedirects(response, reverse("show_language", kwargs={"lang": "xx"}))
 
