@@ -21,11 +21,14 @@
 import random
 import time
 from hashlib import md5
+from itertools import chain
 from typing import Dict, List
+from urllib.parse import quote
 
 from django.core.cache import cache
-from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import ValidationError
 from django.utils.functional import cached_property
+from django.utils.translation import gettext as _
 from requests.exceptions import HTTPError
 
 from weblate.checks.utils import highlight_string
@@ -48,10 +51,6 @@ class MachineTranslationError(Exception):
     """Generic Machine translation error."""
 
 
-class MissingConfiguration(ImproperlyConfigured):
-    """Exception raised when configuration is wrong."""
-
-
 class MachineryRateLimit(MachineTranslationError):
     """Raised when rate limiting is detected."""
 
@@ -70,36 +69,73 @@ class MachineTranslation:
     language_map: Dict[str, str] = {}
     same_languages = False
     do_cleanup = True
+    # Batch size is currently used in autotranslate
     batch_size = 20
     accounting_key = "external"
     force_uncleanup = False
     hightlight_syntax = False
+    settings_form = None
+    validate_payload = ("en", "de", "test", None, None, False, 75)
 
     @classmethod
     def get_rank(cls):
         return cls.max_score + cls.rank_boost
 
-    def __init__(self):
+    def __init__(self, settings: Dict[str, str]):
         """Create new machine translation object."""
-        self.mtid = self.name.lower().replace(" ", "-")
+        self.mtid = self.get_identifier()
         self.rate_limit_cache = f"{self.mtid}-rate-limit"
         self.languages_cache = f"{self.mtid}-languages"
         self.comparer = Comparer()
         self.supported_languages_error = None
         self.supported_languages_error_age = 0
+        self.settings = settings
 
     def delete_cache(self):
         cache.delete_many([self.rate_limit_cache, self.languages_cache])
 
-    def get_identifier(self):
-        return self.mtid
+    @staticmethod
+    def migrate_settings():
+        # TODO: Drop in Weblate 5.1
+        return {}
+
+    def validate_settings(self):
+        try:
+            self.download_languages()
+        except Exception as error:
+            raise ValidationError(_("Failed to fetch supported languages: %s") % error)
+        try:
+            self.download_translations(*self.validate_payload)
+        except Exception as error:
+            raise ValidationError(_("Failed to fetch translation: %s") % error)
+
+    @property
+    def api_base_url(self):
+        base = self.settings["url"]
+        if base.endswith("/"):
+            base = base.rstrip("/")
+        return base
+
+    def get_api_url(self, *parts):
+        """Generates service URL gracefully handle trailing slashes."""
+        return "/".join(
+            chain([self.api_base_url], (quote(part, b"") for part in parts))
+        )
+
+    @classmethod
+    def get_identifier(cls):
+        return cls.name.lower().replace(" ", "-")
+
+    @classmethod
+    def get_doc_anchor(cls):
+        return f"mt-{cls.get_identifier()}"
 
     def account_usage(self, project, delta: int = 1):
         key = f"machinery-accounting:{self.accounting_key}:{project.id}"
         try:
             cache.incr(key, delta=delta)
         except ValueError:
-            cache.set(key, delta)
+            cache.set(key, delta, 24 * 3600)
 
     def get_authentication(self):
         """Hook for backends to allow add authentication headers to request."""
