@@ -62,7 +62,7 @@ class MemoryModelTest(FixtureTestCase):
     def test_machine(self):
         add_document()
         unit = self.get_unit()
-        machine_translation = WeblateMemory()
+        machine_translation = WeblateMemory({})
         self.assertEqual(
             machine_translation.translate(unit, search="Hello"),
             [
@@ -72,6 +72,25 @@ class MemoryModelTest(FixtureTestCase):
                     "origin": "File: test",
                     "source": "Hello",
                     "text": "Ahoj",
+                    "show_quality": True,
+                    "delete_url": None,
+                }
+            ],
+        )
+
+        self.user.is_superuser = True
+        self.user.save()
+        self.assertEqual(
+            machine_translation.translate(unit, user=self.user, search="Hello"),
+            [
+                {
+                    "quality": 100,
+                    "service": "Weblate Translation Memory",
+                    "origin": "File: test",
+                    "source": "Hello",
+                    "text": "Ahoj",
+                    "show_quality": True,
+                    "delete_url": f"/api/memory/{Memory.objects.first().pk}/",
                 }
             ],
         )
@@ -79,7 +98,7 @@ class MemoryModelTest(FixtureTestCase):
     def test_machine_batch(self):
         add_document()
         unit = self.get_unit()
-        machine_translation = WeblateMemory()
+        machine_translation = WeblateMemory({})
         unit.source = "Hello"
         machine_translation.batch_translate([unit])
         self.assertEqual(unit.machinery, {"best": 100, "translation": "Ahoj"})
@@ -163,6 +182,8 @@ class MemoryViewTest(FixtureTestCase):
     def test_memory(
         self, match="Number of your entries", fail=False, prefix: str = "", **kwargs
     ):
+
+        is_project_scoped = "kwargs" in kwargs and "project" in kwargs["kwargs"]
         # Test wipe without confirmation
         response = self.client.get(reverse(f"{prefix}memory-delete", **kwargs))
         self.assertRedirects(response, reverse(f"{prefix}memory", **kwargs))
@@ -171,7 +192,7 @@ class MemoryViewTest(FixtureTestCase):
         self.assertRedirects(response, reverse(f"{prefix}memory", **kwargs))
 
         # Test rebuild without confirmation
-        if "project" in kwargs:
+        if is_project_scoped:
             response = self.client.get(reverse(f"{prefix}memory-rebuild", **kwargs))
             self.assertRedirects(response, reverse(f"{prefix}memory", **kwargs))
 
@@ -215,19 +236,18 @@ class MemoryViewTest(FixtureTestCase):
         if fail:
             self.assertContains(response, "Permission Denied", status_code=403)
         else:
-            self.assertContains(response, "Entries deleted")
+            self.assertContains(response, "Entries were deleted")
             self.assertEqual(count, Memory.objects.count())
             response = self.client.post(
                 reverse(f"{prefix}memory-delete", **kwargs),
                 {"confirm": "1"},
                 follow=True,
             )
-            self.assertContains(response, "Entries deleted")
+            self.assertContains(response, "Entries were deleted")
             self.assertGreater(count, Memory.objects.count())
 
         # Test rebuild
-        if "project" in kwargs:
-            count = Memory.objects.count()
+        if is_project_scoped:
             response = self.client.post(
                 reverse(f"{prefix}memory-rebuild", **kwargs),
                 {"confirm": "1", "origin": "invalid"},
@@ -242,15 +262,19 @@ class MemoryViewTest(FixtureTestCase):
             if fail:
                 self.assertContains(response, "Permission Denied", status_code=403)
             else:
-                self.assertContains(response, "Entries deleted and memory")
-                self.assertEqual(count, Memory.objects.count())
+                self.assertContains(
+                    response, "Entries were deleted and the translation memory"
+                )
+                self.assertEqual(4, Memory.objects.count())
                 response = self.client.post(
                     reverse(f"{prefix}memory-rebuild", **kwargs),
                     {"confirm": "1"},
                     follow=True,
                 )
-                self.assertContains(response, "Entries deleted and memory")
-                self.assertGreater(count, Memory.objects.count())
+                self.assertContains(
+                    response, "Entries were deleted and the translation memory"
+                )
+                self.assertEqual(4, Memory.objects.count())
 
         # Test invalid upload
         response = self.upload_file("cs.json", **kwargs)
