@@ -26,6 +26,7 @@ from django.http import Http404
 
 from weblate.utils.environment import (
     get_env_bool,
+    get_env_credentials,
     get_env_float,
     get_env_int,
     get_env_list,
@@ -48,7 +49,7 @@ SITE_URL = "{}://{}".format("https" if ENABLE_HTTPS else "http", SITE_DOMAIN)
 # Django settings for Weblate project.
 #
 
-DEBUG = get_env_bool("WEBLATE_DEBUG", True)
+DEBUG = get_env_bool("WEBLATE_DEBUG", False)
 
 ADMINS = (
     (
@@ -89,8 +90,6 @@ DATABASES = {
     }
 }
 
-BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-
 # Data directory
 DATA_DIR = os.environ.get("WEBLATE_DATA_DIR", "/app/data")
 CACHE_DIR = os.environ.get("WEBLATE_CACHE_DIR", "/app/cache")
@@ -203,7 +202,7 @@ STATICFILES_FINDERS = (
 )
 
 # Make this unique, and don't share it with anybody.
-# You can generate it using weblate/examples/generate-secret-key
+# You can generate it using weblate-generate-secret-key
 with open("/app/data/secret") as handle:
     SECRET_KEY = handle.read()
 
@@ -228,23 +227,19 @@ TEMPLATES = [
 
 # GitHub username and token for sending pull requests.
 # Please see the documentation for more details.
-GITHUB_USERNAME = os.environ.get("WEBLATE_GITHUB_USERNAME")
-GITHUB_TOKEN = os.environ.get("WEBLATE_GITHUB_TOKEN")
+GITHUB_USERNAME, GITHUB_TOKEN, GITHUB_CREDENTIALS = get_env_credentials("GITHUB")
 
 # GitLab username and token for sending merge requests.
 # Please see the documentation for more details.
-GITLAB_USERNAME = os.environ.get("WEBLATE_GITLAB_USERNAME")
-GITLAB_TOKEN = os.environ.get("WEBLATE_GITLAB_TOKEN")
+GITLAB_USERNAME, GITLAB_TOKEN, GITLAB_CREDENTIALS = get_env_credentials("GITLAB")
 
 # Gitea username and token for sending pull requests.
 # Please see the documentation for more details.
-GITEA_USERNAME = os.environ.get("WEBLATE_GITEA_USERNAME")
-GITEA_TOKEN = os.environ.get("WEBLATE_GITEA_TOKEN")
+GITEA_USERNAME, GITEA_TOKEN, GITEA_CREDENTIALS = get_env_credentials("GITEA")
 
 # Pagure username and token for sending merge requests.
 # Please see the documentation for more details.
-PAGURE_USERNAME = os.environ.get("WEBLATE_PAGURE_USERNAME")
-PAGURE_TOKEN = os.environ.get("WEBLATE_PAGURE_TOKEN")
+PAGURE_USERNAME, PAGURE_TOKEN, PAGURE_CREDENTIALS = get_env_credentials("PAGURE")
 
 # Default pull request message.
 # Please see the documentation for more details.
@@ -391,6 +386,8 @@ if "WEBLATE_SAML_IDP_URL" in os.environ:
             "url": SITE_URL,
         }
     }
+    SOCIAL_AUTH_SAML_IMAGE = os.environ.get("WEBLATE_SAML_IDP_IMAGE", "")
+    SOCIAL_AUTH_SAML_TITLE = os.environ.get("WEBLATE_SAML_IDP_TITLE", "")
 
 # Azure
 if "WEBLATE_SOCIAL_AUTH_AZUREAD_OAUTH2_KEY" in os.environ:
@@ -438,6 +435,12 @@ if "WEBLATE_SOCIAL_AUTH_KEYCLOAK_KEY" in os.environ:
     SOCIAL_AUTH_KEYCLOAK_ACCESS_TOKEN_URL = os.environ.get(
         "WEBLATE_SOCIAL_AUTH_KEYCLOAK_ACCESS_TOKEN_URL", ""
     )
+    SOCIAL_AUTH_KEYCLOAK_IMAGE = os.environ.get(
+        "WEBLATE_SOCIAL_AUTH_KEYCLOAK_IMAGE", ""
+    )
+    SOCIAL_AUTH_KEYCLOAK_TITLE = os.environ.get(
+        "WEBLATE_SOCIAL_AUTH_KEYCLOAK_TITLE", ""
+    )
     SOCIAL_AUTH_KEYCLOAK_ID_KEY = "email"
 
 # Linux distros
@@ -455,6 +458,33 @@ if "WEBLATE_SOCIAL_AUTH_SLACK_KEY" in os.environ:
     SOCIAL_AUTH_SLACK_KEY = os.environ.get("WEBLATE_SOCIAL_AUTH_SLACK_KEY", "")
     SOCIAL_AUTH_SLACK_SECRET = os.environ.get("WEBLATE_SOCIAL_AUTH_SLACK_SECRET", "")
 
+# Generic OpenID Connect
+if "WEBLATE_SOCIAL_AUTH_OIDC_OIDC_ENDPOINT" in os.environ:
+    AUTHENTICATION_BACKENDS += (
+        "social_core.backends.open_id_connect.OpenIdConnectAuth",
+    )
+    SOCIAL_AUTH_OIDC_OIDC_ENDPOINT = os.environ.get(
+        "WEBLATE_SOCIAL_AUTH_OIDC_OIDC_ENDPOINT", ""
+    )
+    SOCIAL_AUTH_OIDC_KEY = os.environ.get("WEBLATE_SOCIAL_AUTH_OIDC_KEY", "")
+    SOCIAL_AUTH_OIDC_SECRET = os.environ.get("WEBLATE_SOCIAL_AUTH_OIDC_SECRET", "")
+    if "WEBLATE_SOCIAL_AUTH_OIDC_USERNAME_KEY" in os.environ:
+        SOCIAL_AUTH_OIDC_USERNAME_KEY = os.environ[
+            "WEBLATE_SOCIAL_AUTH_OIDC_USERNAME_KEY"
+        ]
+
+# Gitea
+if "WEBLATE_SOCIAL_AUTH_GITEA_KEY" in os.environ:
+    AUTHENTICATION_BACKENDS += ("social_core.backends.gitea.GiteaOAuth2",)
+
+    if "WEBLATE_SOCIAL_AUTH_GITEA_API_URL" in os.environ:
+        SOCIAL_AUTH_GITEA_API_URL = os.environ.get(
+            "WEBLATE_SOCIAL_AUTH_GITEA_API_URL", ""
+        )
+
+    SOCIAL_AUTH_GITEA_KEY = os.environ.get("WEBLATE_SOCIAL_AUTH_GITEA_KEY", "")
+    SOCIAL_AUTH_GITEA_SECRET = os.environ.get("WEBLATE_SOCIAL_AUTH_GITEA_SECRET", "")
+
 # https://docs.weblate.org/en/latest/admin/auth.html#ldap-authentication
 if "WEBLATE_AUTH_LDAP_SERVER_URI" in os.environ:
     import ldap
@@ -597,6 +627,10 @@ REGISTRATION_ALLOW_BACKENDS = get_env_list("WEBLATE_REGISTRATION_ALLOW_BACKENDS"
 # Email registration filter
 REGISTRATION_EMAIL_MATCH = os.environ.get("WEBLATE_REGISTRATION_EMAIL_MATCH", ".*")
 
+if "WEBLATE_PRIVATE_COMMIT_EMAIL_TEMPLATE" in os.environ:
+    PRIVATE_COMMIT_EMAIL_TEMPLATE = os.environ["WEBLATE_PRIVATE_COMMIT_EMAIL_TEMPLATE"]
+PRIVATE_COMMIT_EMAIL_OPT_IN = get_env_bool("WEBLATE_PRIVATE_COMMIT_EMAIL_OPT_IN", True)
+
 # Shortcut for login required setting
 REQUIRE_LOGIN = get_env_bool("WEBLATE_REQUIRE_LOGIN")
 
@@ -868,24 +902,6 @@ MT_GOOGLE_LOCATION = os.environ.get("WEBLATE_MT_GOOGLE_LOCATION", "global")
 if MT_GOOGLE_CREDENTIALS and MT_GOOGLE_PROJECT:
     MT_SERVICES += ("weblate.machinery.googlev3.GoogleV3Translation",)
 
-# Baidu app key and secret
-MT_BAIDU_ID = None
-MT_BAIDU_SECRET = None
-
-# Youdao Zhiyun app key and secret
-MT_YOUDAO_ID = None
-MT_YOUDAO_SECRET = None
-
-# Netease Sight (Jianwai) app key and secret
-MT_NETEASE_KEY = None
-MT_NETEASE_SECRET = None
-
-# API key for Yandex Translate API
-MT_YANDEX_KEY = None
-
-# tmserver URL
-MT_TMSERVER = None
-
 # SAP Translation Hub
 MT_SAP_BASE_URL = os.environ.get("WEBLATE_MT_SAP_BASE_URL")
 MT_SAP_SANDBOX_APIKEY = os.environ.get("WEBLATE_MT_SAP_SANDBOX_APIKEY")
@@ -912,6 +928,7 @@ SESSION_COOKIE_SECURE = ENABLE_HTTPS
 SESSION_COOKIE_HTTPONLY = True
 # SSL redirect
 SECURE_SSL_REDIRECT = ENABLE_HTTPS
+SECURE_SSL_HOST = SITE_DOMAIN
 # Sent referrrer only for same origin links
 SECURE_REFERRER_POLICY = "same-origin"
 # SSL redirect URL exemption list
@@ -968,7 +985,7 @@ EMAIL_SEND_HTML = True
 EMAIL_SUBJECT_PREFIX = f"[{SITE_TITLE}] "
 
 # Enable remote hooks
-ENABLE_HOOKS = True
+ENABLE_HOOKS = get_env_bool("WEBLATE_ENABLE_HOOKS", True)
 
 # Version hiding
 HIDE_VERSION = get_env_bool("WEBLATE_HIDE_VERSION")
@@ -1098,6 +1115,7 @@ WEBLATE_ADDONS = [
     "weblate.addons.generate.PseudolocaleAddon",
     "weblate.addons.generate.PrefillAddon",
     "weblate.addons.json.JSONCustomizeAddon",
+    "weblate.addons.xml.XMLCustomizeAddon",
     "weblate.addons.properties.PropertiesSortAddon",
     "weblate.addons.git.GitSquashAddon",
     "weblate.addons.removal.RemoveComments",
@@ -1129,8 +1147,8 @@ CACHES = {
         "LOCATION": "{}://{}:{}/{}".format(
             REDIS_PROTO,
             os.environ.get("REDIS_HOST", "cache"),
-            os.environ.get("REDIS_PORT", "6379"),
-            os.environ.get("REDIS_DB", "1"),
+            get_env_int("REDIS_PORT", 6379),
+            get_env_int("REDIS_DB", 1),
         ),
         # If redis is running on same host as Weblate, you might
         # want to use unix sockets instead:
@@ -1143,6 +1161,7 @@ CACHES = {
             "CONNECTION_POOL_KWARGS": {},
         },
         "KEY_PREFIX": "weblate",
+        "TIMEOUT": 3600,
     },
     "avatar": {
         "BACKEND": "django.core.cache.backends.filebased.FileBasedCache",
@@ -1264,8 +1283,8 @@ CELERY_BROKER_URL = "{}://{}{}:{}/{}".format(
     REDIS_PROTO,
     f":{REDIS_PASSWORD}@" if REDIS_PASSWORD else "",
     os.environ.get("REDIS_HOST", "cache"),
-    os.environ.get("REDIS_PORT", "6379"),
-    os.environ.get("REDIS_DB", "1"),
+    get_env_int("REDIS_PORT", 6379),
+    get_env_int("REDIS_DB", 1),
 )
 if REDIS_PROTO == "rediss":
     CELERY_BROKER_URL = "{}?ssl_cert_reqs={}".format(
@@ -1299,6 +1318,9 @@ UPDATE_LANGUAGES = get_env_bool("WEBLATE_UPDATE_LANGUAGES", True)
 
 # Avatars
 ENABLE_AVATARS = get_env_bool("WEBLATE_ENABLE_AVATARS", True)
+AVATAR_URL_PREFIX = os.environ.get(
+    "WEBLATE_AVATAR_URL_PREFIX", "https://www.gravatar.com/"
+)
 
 # Default access control
 DEFAULT_ACCESS_CONTROL = get_env_int("WEBLATE_DEFAULT_ACCESS_CONTROL")
@@ -1326,6 +1348,10 @@ SSH_EXTRA_ARGS = os.environ.get("WEBLATE_SSH_EXTRA_ARGS", "")
 
 BORG_EXTRA_ARGS = get_env_list("WEBLATE_BORG_EXTRA_ARGS")
 
+ENABLE_SHARING = get_env_bool("WEBLATE_ENABLE_SHARING")
+
+EXTRA_HTML_HEAD = os.environ.get("WEBLATE_EXTRA_HTML_HEAD", "")
+
 # Wildcard loading
 for name in os.environ:
     if name.startswith("WEBLATE_RATELIMIT_") and name.endswith(
