@@ -88,7 +88,7 @@ WLT.Editor = (function () {
       } else {
         addAlert(
           gettext("Please select target plural by clicking."),
-          (kind = "info")
+          (kind = "info"),
         );
         editors.addClass("editor-click-select");
         editors.click(function () {
@@ -146,11 +146,7 @@ WLT.Editor = (function () {
     /* Copy from source text highlight check */
     this.$editor.on("click", hlSelector, function (e) {
       var $this = $(this);
-      var text = $this.clone();
-
-      text.find(hlNumberSelector).remove();
-      text = text.text();
-      insertEditor(text, $this);
+      insertEditor($this.data("value"), $this);
       e.preventDefault();
     });
 
@@ -176,7 +172,7 @@ WLT.Editor = (function () {
             title = interpolate(gettext("Ctrl+%s"), [key]);
           }
           $this.attr("title", title);
-          $this.find(hlNumberSelector).html("<kbd>" + key + "</kbd>");
+          $this.find(hlNumberSelector).html($("<kbd/>").text(key));
 
           Mousetrap.bindGlobal("mod+" + key, function (e) {
             $this.click();
@@ -194,14 +190,14 @@ WLT.Editor = (function () {
       function (e) {
         $(hlNumberSelector).show();
       },
-      "keydown"
+      "keydown",
     );
     Mousetrap.bindGlobal(
       "mod",
       function (e) {
         $(hlNumberSelector).hide();
       },
-      "keyup"
+      "keyup",
     );
   };
 
@@ -214,6 +210,9 @@ WLT.Editor = (function () {
       if (root.length === 0) {
         root = element.closest(".translation-form");
       }
+      if (root.length === 0) {
+        root = $(document);
+      }
     } else {
       root = $(document);
     }
@@ -226,7 +225,7 @@ WLT.Editor = (function () {
       }
     }
 
-    editor.insertAtCaret($.trim(text));
+    editor.insertAtCaret(text);
   }
 
   return {
