@@ -36,6 +36,41 @@
       submitForm({ target: this.$translationArea });
     });
 
+    /* Delete machinery results */
+    this.$editor.on("click", ".js-delete-machinery", (e) => {
+      var $el = $(e.target);
+
+      /* Delete Url dialog */
+      var $deleteEntriesDialog = null;
+      this.$editor.on("show.bs.modal", "#delete-url-modal", (e) => {
+        $deleteEntriesDialog = $(e.currentTarget);
+        $deleteEntriesDialog.find(".modal-body").html("");
+        var text = $el.parent().parent().data("raw").text;
+        var modalBody = this.machinery.renderDeleteUrls(text);
+        $deleteEntriesDialog.find(".modal-body").append(modalBody);
+      });
+
+      this.$editor.on("hide.bs.modal", "#delete-url-modal", (e) => {
+        $deleteEntriesDialog = null;
+      });
+
+      this.$editor.on("submit", ".delete-url-form", (e) => {
+        var $form = $(e.currentTarget);
+        var $deleteEntries = $form.find("input.form-check-input:checked");
+        if ($deleteEntriesDialog === null) {
+          return false;
+        }
+        $deleteEntriesDialog.modal("hide");
+
+        Object.entries($deleteEntries).forEach(([_, entry]) => {
+          if (typeof entry.id !== "undefined") {
+            this.removeTranslationEntry(entry.id);
+          }
+        });
+        return false;
+      });
+    });
+
     Mousetrap.bindGlobal("alt+end", function (e) {
       window.location = $("#button-end").attr("href");
       return false;
@@ -45,7 +80,7 @@
       function (e) {
         window.location = $("#button-next").attr("href");
         return false;
-      }
+      },
     );
     Mousetrap.bindGlobal(["alt+pageup", "mod+up", "alt+up"], function (e) {
       window.location = $("#button-prev").attr("href");
@@ -63,9 +98,12 @@
       $('input[name="fuzzy"]').click();
       return false;
     });
-    Mousetrap.bindGlobal("mod+shift+enter", function (e) {
+    Mousetrap.bindGlobal("mod+shift+enter", function (e, combo) {
       $('input[name="fuzzy"]').prop("checked", false);
-      return submitForm(e);
+      return submitForm(e, combo);
+    });
+    Mousetrap.bindGlobal("alt+enter", function (e, combo) {
+      return submitForm(e, combo, 'button[name="suggest"]');
     });
     Mousetrap.bindGlobal("mod+e", () => {
       this.$translationArea.get(0).focus();
@@ -106,11 +144,21 @@
         {
           scrollTop: $("#comment-form").offset().top,
         },
-        1000
+        1000,
       );
       $("#id_comment").focus();
     });
 
+    this.$translationForm.on("click", ".add-alternative-post", function () {
+      var elm = $("<input>")
+        .attr("type", "hidden")
+        .attr("name", "add_alternative")
+        .attr("value", "1");
+      self.$translationForm.append(elm);
+      self.$translationForm.submit();
+      return false;
+    });
+
     /* Form persistence. Restores translation form upon comment submission */
     var restoreKey = "translation_autosave";
     var restoreValue = window.localStorage.getItem(restoreKey);
@@ -200,6 +248,20 @@
     });
   };
 
+  FullEditor.prototype.removeTranslationEntry = function (delete_url) {
+    $.ajax({
+      type: "DELETE",
+      url: delete_url,
+      headers: { "X-CSRFToken": this.csrfToken },
+      success: () => {
+        addAlert(gettext("Translation memory entry removed."));
+      },
+      error: (jqXHR, textStatus, errorThrown) => {
+        addAlert(errorThrown);
+      },
+    });
+  };
+
   FullEditor.prototype.fetchMachinery = function (serviceName) {
     $.ajax({
       type: "POST",
@@ -220,7 +282,7 @@
   FullEditor.prototype.processMachineryError = function (
     jqXHR,
     textStatus,
-    errorThrown
+    errorThrown,
   ) {
     decreaseLoading("machinery");
     if (jqXHR.state() !== "rejected") {
@@ -229,7 +291,7 @@
           " " +
           textStatus +
           ": " +
-          errorThrown
+          errorThrown,
       );
     }
   };
@@ -239,7 +301,7 @@
     if (data.responseStatus !== 200) {
       var msg = interpolate(
         gettext("The request for machine translation using %s has failed:"),
-        [data.service]
+        [data.service],
       );
       addAlert(msg + " " + data.responseDetails);
 
@@ -251,6 +313,7 @@
         ...this.machinery.state.translations,
         ...data.translations,
       ],
+      weblateTranslationMemory: new Set(),
       lang: data.lang,
       dir: data.dir,
     });
@@ -278,7 +341,7 @@
         }
         $(this)
           .find(".machinery-number")
-          .html(' <kbd title="' + title + '">' + key + "</kbd>");
+          .html($("<kbd/>").attr("title", title).text(key));
         Mousetrap.bindGlobal(["mod+m " + key, "mod+m mod+" + key], function () {
           $translationRows.eq(idx).find(".js-copy-machinery").click();
           return false;
@@ -382,17 +445,17 @@
         } else {
           title = interpolate(
             gettext("Press Ctrl+I then %s to dismiss this."),
-            [key]
+            [key],
           );
         }
-        $number.html(' <kbd title="' + title + '">' + key + "</kbd>");
+        $number.html($("<kbd/>").attr("title", title).text(key));
 
         Mousetrap.bindGlobal(
           ["mod+i " + key, "mod+i mod+" + key],
           function (e) {
             $this.find(".check-dismiss-single").click();
             return false;
-          }
+          },
         );
       } else {
         $number.html("");
@@ -415,7 +478,7 @@
         text = target.find(".source").text();
       }
 
-      this.insertIntoTranslation(text);
+      this.insertIntoTranslation($.trim(text));
       e.preventDefault();
     });
 
@@ -430,7 +493,7 @@
       }
       /* Relies on clone source implementation */
       let cloneElement = document.querySelector(
-        ".source-language-group [data-clone-text]"
+        ".source-language-group [data-clone-text]",
       );
       if (cloneElement !== null) {
         let source = cloneElement.getAttribute("data-clone-text");
@@ -439,7 +502,7 @@
           let term_target = document.getElementById("id_add_term_target");
           term_source.value = source;
           term_target.value = document.querySelector(
-            ".translation-editor"
+            ".translation-editor",
           ).value;
         }
       }
@@ -487,6 +550,7 @@
     constructor(initialState = {}) {
       this.state = {
         translations: [],
+        weblateTranslationMemory: new Set(),
         lang: null,
         dir: null,
       };
@@ -503,15 +567,17 @@
           .attr("class", "target machinery-text")
           .attr("lang", this.state.lang)
           .attr("dir", this.state.dir)
-          .text(el.text)
+          .text(el.text),
       );
       row.append($("<td/>").attr("class", "machinery-text").text(el.source));
       row.append(service);
 
       /* Quality score as bar with the text */
-      row.append(
-        $("<td class='number'><strong>" + el.quality + "</strong> %</td>")
-      );
+      let quality_cell = $("<td class='number'></td>");
+      if (el.show_quality) {
+        quality_cell.html("<strong>" + el.quality + "</strong> %");
+      }
+      row.append(quality_cell);
       /* Translators: Verb for copy operation */
       row.append(
         $(
@@ -525,10 +591,24 @@
             '<a class="js-copy-save-machinery btn btn-primary">' +
             gettext("Copy and save") +
             "</a>" +
-            "</td>"
-        )
+            "</td>",
+        ),
       );
 
+      if (this.state.weblateTranslationMemory.has(el.text)) {
+        row.append(
+          $(
+            "<td>" +
+              '<a class="js-delete-machinery btn btn-danger" data-toggle="modal" data-target="#delete-url-modal">' +
+              gettext("Delete entry") +
+              "</a>" +
+              "</td>",
+          ),
+        );
+      } else {
+        row.append($("<td></td>"));
+      }
+
       return row;
     }
 
@@ -537,6 +617,7 @@
       if (typeof el.origin !== "undefined") {
         service.append(" (");
         var origin;
+        var deleteUrl = false;
         if (typeof el.origin_detail !== "undefined") {
           origin = $("<abbr/>").text(el.origin).attr("title", el.origin_detail);
         } else if (typeof el.origin_url !== "undefined") {
@@ -544,12 +625,43 @@
         } else {
           origin = el.origin;
         }
+        if (el.delete_url) {
+          this.state.weblateTranslationMemory.add(el.text);
+        }
         service.append(origin);
         service.append(")");
       }
       return service;
     }
 
+    renderDeleteUrls(text) {
+      var translations = this.state.translations;
+      var modalBody = $("<label>").text("");
+
+      translations.forEach((translation) => {
+        if (
+          text === translation.text &&
+          typeof translation.delete_url !== "undefined"
+        ) {
+          var inputElement = $("<input>")
+            .attr("class", "form-check-input")
+            .attr("type", "checkbox")
+            .attr("value", "")
+            .attr("id", translation.delete_url)
+            .attr("checked", true);
+          var labelElement = $("<label>")
+            .attr("class", "form-check-label")
+            .attr("for", translation.delete_url)
+            .text(translation.origin);
+          var divElement = $("<div>")
+            .attr("class", "form-check")
+            .append(inputElement, labelElement);
+          modalBody.append(divElement);
+        }
+      });
+      return modalBody;
+    }
+
     render(translations) {
       var $translations = $("#machinery-translations");
       translations.forEach((translation) => {
