@@ -30,10 +30,9 @@ function decreaseLoading(sel) {
 function addAlert(message, kind = "danger", delay = 3000) {
   var alerts = $("#popup-alerts");
   var e = $(
-    '<div class="alert alert-' +
-      kind +
-      ' alert-dismissible" role="alert"><button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button></div>'
+    '<div class="alert alert-dismissible" role="alert"><button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button></div>',
   );
+  e.addClass("alert-" + kind);
   e.append(new Text(message));
   e.hide();
   alerts.show().append(e);
@@ -78,21 +77,27 @@ jQuery.fn.extend({
         this.value += myValue;
         this.focus();
       }
-      this.dispatchEvent(new Event("input"));
-      /* Zen editor still relies on jQuery here */
-      $(this).change();
+      // Need `bubbles` because some event listeners (like this
+      // https://github.com/WeblateOrg/weblate/blob/86d4fb308c9941f32b48f007e16e8c153b0f3fd7/weblate/static/editor/base.js#L50
+      // ) are attached to the parent elements.
+      this.dispatchEvent(new Event("input", { bubbles: true }));
+      this.dispatchEvent(new Event("change", { bubbles: true }));
     });
   },
 
   replaceValue: function (myValue) {
     return this.each(function () {
       this.value = myValue;
-      this.dispatchEvent(new Event("input"));
+      // Need `bubbles` because some event listeners (like this
+      // https://github.com/WeblateOrg/weblate/blob/86d4fb308c9941f32b48f007e16e8c153b0f3fd7/weblate/static/editor/base.js#L50
+      // ) are attached to the parent elements.
+      this.dispatchEvent(new Event("input", { bubbles: true }));
+      this.dispatchEvent(new Event("change", { bubbles: true }));
     });
   },
 });
 
-function submitForm(evt) {
+function submitForm(evt, combo, selector) {
   var $target = $(evt.target);
   var $form = $target.closest("form");
 
@@ -100,18 +105,22 @@ function submitForm(evt) {
     $form = $(".translation-form");
   }
   if ($form.length > 0) {
-    let submits = $form.find('input[type="submit"]');
+    if (typeof selector !== "undefined") {
+      $form.find(selector).click();
+    } else {
+      let submits = $form.find('input[type="submit"]');
 
-    if (submits.length === 0) {
-      submits = $form.find('button[type="submit"]');
-    }
-    if (submits.length > 0) {
-      submits[0].click();
+      if (submits.length === 0) {
+        submits = $form.find('button[type="submit"]');
+      }
+      if (submits.length > 0) {
+        submits[0].click();
+      }
     }
   }
   return false;
 }
-Mousetrap.bindGlobal(["alt+enter", "mod+enter"], submitForm);
+Mousetrap.bindGlobal("mod+enter", submitForm);
 
 function screenshotStart() {
   $("#search-results tbody.unit-listing-body").empty();
@@ -146,7 +155,9 @@ function screenshotAddString() {
 
 function screnshotResultError(severity, message) {
   $("#search-results tbody.unit-listing-body").html(
-    '<tr class="' + severity + '"><td colspan="4">' + message + "</td></tr>"
+    $("<tr/>")
+      .addClass(severity)
+      .html($('<td colspan="4"></td>').text(message)),
   );
 }
 
@@ -157,7 +168,7 @@ function screenshotLoaded(data) {
   } else if (data.results.length === 0) {
     screnshotResultError(
       "warning",
-      gettext("No new matching source strings found.")
+      gettext("No new matching source strings found."),
     );
   } else {
     $("#search-results table").replaceWith(data.results);
@@ -251,7 +262,7 @@ function loadTableSorting() {
                   inverse *
                   compareCells(
                     extractText($a.find("td,th")[myIndex]),
-                    extractText($b.find("td,th")[myIndex])
+                    extractText($b.find("td,th")[myIndex]),
                   )
                 );
               })
@@ -391,7 +402,7 @@ function initHighlight(root) {
           "\u2002|\u2003|\u2004|\u2005|",
           "\u2006|\u2007|\u2008|\u2009|",
           "\u200A|\u202F|\u205F|\u3000",
-        ].join("")
+        ].join(""),
       );
       let extension = {
         hlspace: {
@@ -474,13 +485,13 @@ $(function () {
               " (" +
               xhr.status +
               "): " +
-              responseText
+              responseText,
           );
         }
         $target.data("loaded", 1);
         loadTableSorting();
       });
-    }
+    },
   );
 
   if ($("#form-activetab").length > 0) {
@@ -505,7 +516,7 @@ $(function () {
       activeTab = $(
         '.nav [data-toggle=tab][href="' +
           location.hash.substr(0, separator) +
-          '"]'
+          '"]',
       );
       if (activeTab.length) {
         activeTab.tab("show");
@@ -524,7 +535,7 @@ $(function () {
   ) {
     /* From cookie */
     activeTab = $(
-      '[data-toggle=tab][href="' + Cookies.get("translate-tab") + '"]'
+      '[data-toggle=tab][href="' + Cookies.get("translate-tab") + '"]',
     );
     if (activeTab.length) {
       activeTab.tab("show");
@@ -713,7 +724,13 @@ $(function () {
     var $this = $(this);
     $("#imagepreview").attr("src", $this.attr("href"));
     $("#screenshotModal").text($this.attr("title"));
-    $("#modalEditLink").attr("href", $this.data("edit"));
+
+    var detailsLink = $("#modalDetailsLink");
+    detailsLink.attr("href", $this.data("details-url"));
+    if ($this.data("can-edit")) {
+      detailsLink.text(detailsLink.data("edit-text"));
+    }
+
     $("#imagemodal").modal("show");
     return false;
   });
@@ -853,7 +870,7 @@ $(function () {
       .find('input[name="name"]')
       .on("change keypress keydown keyup paste", function () {
         $slug.val(
-          slugify($(this).val(), { remove: /[^\w\s-]+/g }).toLowerCase()
+          slugify($(this).val(), { remove: /[^\w\s-]+/g }).toLowerCase(),
         );
       });
   });
@@ -941,16 +958,13 @@ $(function () {
       })
       .data("sort");
     var sort_value = $("#id_sort_by").val();
+    var $label = $(this).find("span.search-icon");
     if (sort_dropdown_value) {
       if (
         sort_value.replace("-", "") === sort_dropdown_value.replace("-", "") &&
         sort_value !== sort_dropdown_value
       ) {
-        $("#query-sort-toggle .asc").hide();
-        $("#query-sort-toggle .desc").show();
-      } else {
-        $("#query-sort-toggle .desc").hide();
-        $("#query-sort-toggle .asc").show();
+        $label.toggle();
       }
     }
   }
@@ -1029,9 +1043,7 @@ $(function () {
   });
   $(".query-sort-toggle").click(function () {
     var $this = $(this);
-    var $label = $this.find("span.search-icon");
     var $input = $this.closest(".search-group").find("input[name=sort_by]");
-    $label.toggle();
     var sort_params = $input.val().split(",");
     sort_params.forEach(function (param, index) {
       if (param.indexOf("-") !== -1) {
@@ -1054,17 +1066,9 @@ $(function () {
         return false;
       }
     });
-  $("#id_position").on("keydown", function (event) {
-    if (event.key === "Enter") {
-      $(this).closest("form").submit();
-    }
-  });
-  $("#id_q").on("keydown", function (event) {
-    if (event.key === "Enter") {
-      var $form = $(this).closest("form");
-      $form.find("input[name=offset]").val("1");
-      $form.submit();
-    }
+  $("#id_q").on("change", function (event) {
+    var $form = $(this).closest("form");
+    $form.find("input[name=offset]").val("1");
   });
   $(".search-add").click(function () {
     var group = $(this).closest(".search-group");
@@ -1083,13 +1087,13 @@ $(function () {
           button.attr("data-field") +
           prefix +
           quoteSearch(input.val()) +
-          " "
+          " ",
       );
     }
   });
   $(".search-insert").click(function () {
     $("#id_q").insertAtCaret(
-      " " + $(this).closest("tr").find("code").text() + " "
+      " " + $(this).closest("tr").find("code").text() + " ",
     );
   });
 
@@ -1108,7 +1112,7 @@ $(function () {
         target.val(name.substring(0, name.lastIndexOf(".")));
         target.change();
       }
-    }
+    },
   );
 
   /* Alert when creating a component */
@@ -1117,9 +1121,9 @@ $(function () {
       addAlert(
         gettext("Weblate is now scanning the repository, please be patient."),
         (kind = "info"),
-        (delay = 0)
+        (delay = 0),
       );
-    }
+    },
   );
 
   /* Username autocompletion */
@@ -1167,6 +1171,38 @@ $(function () {
     });
   });
 
+  /* forset fields adding */
+  $(".add-multifield").on("click", function () {
+    const updateElementIndex = function (el, prefix, ndx) {
+      const id_regex = new RegExp("(" + prefix + "-(\\d+|__prefix__))");
+      const replacement = prefix + "-" + ndx;
+      if ($(el).prop("for")) {
+        $(el).prop("for", $(el).prop("for").replace(id_regex, replacement));
+      }
+      if (el.id) {
+        el.id = el.id.replace(id_regex, replacement);
+      }
+      if (el.name) {
+        el.name = el.name.replace(id_regex, replacement);
+      }
+    };
+    var $this = $(this);
+    var $form = $this.parents("form");
+    var prefix = $this.data("prefix");
+    var blank = $form.find(".multiFieldEmpty");
+    var row = blank.clone();
+    var totalForms = $("#id_" + prefix + "-TOTAL_FORMS");
+    row.removeClass(["multiFieldEmpty", "hidden"]).addClass("multiField");
+    row.find("*").each(function () {
+      updateElementIndex(this, prefix, totalForms.val());
+    });
+
+    row.insertBefore(blank);
+    totalForms.val(parseInt(totalForms.val(), 10) + 1);
+
+    return false;
+  });
+
   /* Textarea highlighting */
   Prism.languages.none = {};
   initHighlight(document);
@@ -1179,20 +1215,20 @@ $(function () {
   console.log(
     "%c" +
       pgettext("Alert to user when opening browser developer console", "Stop!"),
-    "color: red; font-weight: bold; font-size: 50px; font-family: sans-serif; -webkit-text-stroke: 1px black;"
+    "color: red; font-weight: bold; font-size: 50px; font-family: sans-serif; -webkit-text-stroke: 1px black;",
   );
   console.log(
     "%c" +
       gettext(
-        "This is a browser feature intended for developers. If someone told you to copy-paste something here, they are likely trying to compromise your Weblate account."
+        "This is a browser feature intended for developers. If someone told you to copy-paste something here, they are likely trying to compromise your Weblate account.",
       ),
-    "font-size: 20px; font-family: sans-serif"
+    "font-size: 20px; font-family: sans-serif",
   );
   console.log(
     "%c" +
       gettext(
-        "See https://en.wikipedia.org/wiki/Self-XSS for more information."
+        "See https://en.wikipedia.org/wiki/Self-XSS for more information.",
       ),
-    "font-size: 20px; font-family: sans-serif"
+    "font-size: 20px; font-family: sans-serif",
   );
 });
