@@ -47,7 +47,7 @@ class DoubleApostrophes(AutoFix):
     def fix_single_target(self, target, source, unit):
         flags = unit.all_flags
         if ("auto-java-messageformat" not in flags or "{0" not in source) and (
-            "java-messageformat" not in flags
+            "java-format" not in flags
         ):
             return target, False
         # Split on apostrophe
