@@ -23,9 +23,8 @@ from urllib.parse import urlparse
 
 from django.conf import settings
 from django.core.cache import cache
-from django.utils.html import escape
+from django.utils.html import format_html
 from django.utils.http import url_has_allowed_host_and_scheme
-from django.utils.safestring import mark_safe
 from django.utils.translation import gettext as _
 
 import weblate.screenshots.views
@@ -54,6 +53,7 @@ CONTEXT_SETTINGS = [
     "FONTS_CDN_URL",
     "AVATAR_URL_PREFIX",
     "HIDE_VERSION",
+    "EXTRA_HTML_HEAD",
     # Hosted Weblate integration
     "PAYMENT_ENABLED",
 ]
@@ -167,13 +167,12 @@ def weblate_context(request):
         "version": weblate.utils.version.VERSION,
         "bread_image": get_bread_image(request.path),
         "description": description,
-        "weblate_link": mark_safe(f'<a href="{escape(WEBLATE_URL)}">weblate.org</a>'),
-        "weblate_name_link": mark_safe(f'<a href="{escape(WEBLATE_URL)}">Weblate</a>'),
-        "weblate_version_link": mark_safe(
-            '<a href="{}">Weblate {}</a>'.format(
-                escape(WEBLATE_URL),
-                "" if settings.HIDE_VERSION else weblate.utils.version.VERSION,
-            )
+        "weblate_link": format_html('<a href="{}">weblate.org</a>', WEBLATE_URL),
+        "weblate_name_link": format_html('<a href="{}">Weblate</a>', WEBLATE_URL),
+        "weblate_version_link": format_html(
+            '<a href="{}">Weblate {}</a>',
+            WEBLATE_URL,
+            "" if settings.HIDE_VERSION else weblate.utils.version.VERSION,
         ),
         "donate_url": DONATE_URL,
         "site_url": get_site_url(),
