@@ -46,10 +46,10 @@ class Alert(models.Model):
     updated = models.DateTimeField(auto_now=True)
     name = models.CharField(max_length=150)
     dismissed = models.BooleanField(default=False, db_index=True)
-    details = JSONField(default={})
+    details = JSONField(default=dict)
 
     class Meta:
-        unique_together = ("component", "name")
+        unique_together = [("component", "name")]
         verbose_name = "component alert"
         verbose_name_plural = "component alerts"
 
@@ -81,6 +81,7 @@ class BaseAlert:
     verbose = ""
     on_import = False
     link_wide = False
+    project_wide = False
     dismissable = False
     doc_page = ""
     doc_anchor = ""
@@ -364,6 +365,7 @@ class BrokenProjectURL(BaseAlert):
     dismissable = True
     doc_page = "admin/projects"
     doc_anchor = "project-web"
+    project_wide = True
 
     def __init__(self, instance, error=None):
         super().__init__(instance)
