@@ -22,9 +22,9 @@ from datetime import datetime
 from django.conf import settings
 from django.db import models, transaction
 from django.db.models import Count, Q
+from django.db.models.base import post_save
 from django.utils import timezone
-from django.utils.html import escape
-from django.utils.safestring import mark_safe
+from django.utils.html import escape, format_html
 from django.utils.translation import gettext as _
 from django.utils.translation import (
     gettext_lazy,
@@ -33,7 +33,7 @@ from django.utils.translation import (
     pgettext,
     pgettext_lazy,
 )
-from jellyfish import damerau_levenshtein_distance
+from rapidfuzz.distance import DamerauLevenshtein
 
 from weblate.lang.models import Language
 from weblate.trans.mixins import UserDisplayMixin
@@ -120,6 +120,7 @@ class ChangeQuerySet(models.QuerySet):
             "translation",
             "component",
             "project",
+            "component__source_language",
             "unit",
             "translation__language",
             "translation__plural",
@@ -141,17 +142,33 @@ class ChangeQuerySet(models.QuerySet):
         """Companion for prefetch to fill in nested references."""
         return self.preload_list(self, *args)
 
-    def last_changes(self, user):
+    def last_changes(
+        self, user, unit=None, translation=None, component=None, project=None
+    ):
         """Return the most recent changes for an user.
 
         Filters Change objects by user permissions and fetches related fields for
         last changes display.
         """
-        if user.is_superuser:
-            return self.prefetch().order()
-        return (
-            self.prefetch()
-            .filter(
+        result = self
+        if unit is not None:
+            result = result.filter(unit=unit)
+            if not user.can_access_component(unit.translation.component):
+                result = result.none()
+        elif translation is not None:
+            result = result.filter(translation=translation)
+            if not user.can_access_component(translation.component):
+                result = result.none()
+        elif component is not None:
+            result = result.filter(component=component)
+            if not user.can_access_component(component):
+                result = result.none()
+        elif project is not None:
+            result = result.filter(project=project)
+            if not user.can_access_project(project):
+                result = result.none()
+        elif not user.is_superuser:
+            result = result.filter(
                 Q(project_id__in=user.allowed_project_ids)
                 & (
                     Q(component__isnull=True)
@@ -159,8 +176,7 @@ class ChangeQuerySet(models.QuerySet):
                     | Q(component_id__in=user.component_permissions)
                 )
             )
-            .order()
-        )
+        return result.prefetch().order()
 
     def authors_list(self, date_range=None):
         """Return list of authors."""
@@ -177,6 +193,13 @@ class ChangeQuerySet(models.QuerySet):
     def order(self):
         return self.order_by("-timestamp")
 
+    def bulk_create(self, *args, **kwargs):
+        """Executes post save to ensure messages are sent to fedora messaging."""
+        changes = super().bulk_create(*args, **kwargs)
+        for change in changes:
+            post_save.send(change.__class__, instance=change, created=True)
+        return changes
+
 
 class ChangeManager(models.Manager):
     def create(self, *, user=None, **kwargs):
@@ -244,6 +267,12 @@ class Change(models.Model, UserDisplayMixin):
     ACTION_SCREENSHOT_ADDED = 57
     ACTION_SCREENSHOT_UPLOADED = 58
     ACTION_STRING_REPO_UPDATE = 59
+    ACTION_ADDON_CREATE = 60
+    ACTION_ADDON_CHANGE = 61
+    ACTION_ADDON_REMOVE = 62
+    ACTION_STRING_REMOVE = 63
+    ACTION_COMMENT_DELETE = 64
+    ACTION_COMMENT_RESOLVE = 65
 
     ACTION_CHOICES = (
         # Translators: Name of event in the history
@@ -364,6 +393,18 @@ class Change(models.Model, UserDisplayMixin):
         (ACTION_SCREENSHOT_UPLOADED, gettext_lazy("Screnshot uploaded")),
         # Translators: Name of event in the history
         (ACTION_STRING_REPO_UPDATE, gettext_lazy("String updated in the repository")),
+        # Translators: Name of event in the history
+        (ACTION_ADDON_CREATE, gettext_lazy("Add-on installed")),
+        # Translators: Name of event in the history
+        (ACTION_ADDON_CHANGE, gettext_lazy("Add-on configuration changed")),
+        # Translators: Name of event in the history
+        (ACTION_ADDON_REMOVE, gettext_lazy("Add-on uninstalled")),
+        # Translators: Name of event in the history
+        (ACTION_STRING_REMOVE, gettext_lazy("Removed string")),
+        # Translators: Name of event in the history
+        (ACTION_COMMENT_DELETE, gettext_lazy("Removed comment")),
+        # Translators: Name of event in the history
+        (ACTION_COMMENT_RESOLVE, gettext_lazy("Resolved comment")),
     )
     ACTIONS_DICT = dict(ACTION_CHOICES)
     ACTION_STRINGS = {
@@ -397,6 +438,7 @@ class Change(models.Model, UserDisplayMixin):
         ACTION_BULK_EDIT,
         ACTION_APPROVE,
         ACTION_MARKED_EDIT,
+        ACTION_SOURCE_CHANGE,
     }
 
     # Actions shown on the repository management page
@@ -480,7 +522,7 @@ class Change(models.Model, UserDisplayMixin):
         related_name="author_set",
         on_delete=models.deletion.CASCADE,
     )
-    timestamp = models.DateTimeField(auto_now_add=True, db_index=True)
+    timestamp = models.DateTimeField(auto_now_add=True)
     action = models.IntegerField(
         choices=ACTION_CHOICES, default=ACTION_CHANGE, db_index=True
     )
@@ -493,7 +535,8 @@ class Change(models.Model, UserDisplayMixin):
     class Meta:
         app_label = "trans"
         index_together = [
-            ("translation", "action", "timestamp"),
+            ("timestamp", "project", "component", "language", "action"),
+            ("action", "translation", "timestamp"),
         ]
         verbose_name = "history event"
         verbose_name_plural = "history events"
@@ -594,6 +637,7 @@ class Change(models.Model, UserDisplayMixin):
         )
 
     def get_details_display(self):  # noqa: C901
+        from weblate.addons.models import ADDONS
         from weblate.utils.markdown import render_markdown
 
         details = self.details
@@ -613,25 +657,36 @@ class Change(models.Model, UserDisplayMixin):
         if self.action in (self.ACTION_ANNOUNCEMENT, self.ACTION_AGREEMENT_CHANGE):
             return render_markdown(self.target)
 
+        if self.action in (
+            self.ACTION_ADDON_CREATE,
+            self.ACTION_ADDON_CHANGE,
+            self.ACTION_ADDON_REMOVE,
+        ):
+            try:
+                return ADDONS[self.target].name
+            except KeyError:
+                return self.target
+
         if self.action in self.AUTO_ACTIONS and self.auto_status:
             return str(self.AUTO_ACTIONS[self.action])
 
         if self.action == self.ACTION_UPDATE:
             reason = details.get("reason", "content changed")
-            filename = "<code>{}</code>".format(
-                escape(
-                    details.get(
-                        "filename",
-                        self.translation.filename if self.translation else "",
-                    )
-                )
+            filename = format_html(
+                "<code>{}</code>",
+                details.get(
+                    "filename",
+                    self.translation.filename if self.translation else "",
+                ),
             )
             if reason == "content changed":
-                return mark_safe(_('The "%s" file was changed.') % filename)
+                return format_html(escape(_('The "{}" file was changed.')), filename)
             if reason == "check forced":
-                return mark_safe(_('Parsing of the "%s" file was enforced.') % filename)
+                return format_html(
+                    escape(_('Parsing of the "{}" file was enforced.')), filename
+                )
             if reason == "new file":
-                return mark_safe(_("File %s was added.") % filename)
+                return format_html(escape(_("File {} was added.")), filename)
             raise ValueError(f"Unknown reason: {reason}")
 
         if self.action == self.ACTION_LICENSE_CHANGE:
@@ -685,11 +740,21 @@ class Change(models.Model, UserDisplayMixin):
         return ""
 
     def get_distance(self):
-        try:
-            return damerau_levenshtein_distance(self.old, self.target)
-        except MemoryError:
-            # Too long strings
-            return abs(len(self.old) - len(self.target))
+        return DamerauLevenshtein.distance(self.old, self.target)
 
     def get_source(self):
         return self.details.get("source", self.unit.source)
+
+    def get_ip_address(self):
+        if self.suggestion and "address" in self.suggestion.userdetails:
+            return self.suggestion.userdetails["address"]
+        if self.comment and "address" in self.comment.userdetails:
+            return self.comment.userdetails["address"]
+        return None
+
+    def show_unit_state(self):
+        return "state" in self.details and self.action not in (
+            self.ACTION_SUGGESTION,
+            self.ACTION_SUGGESTION_DELETE,
+            self.ACTION_SUGGESTION_CLEANUP,
+        )
