@@ -19,6 +19,7 @@
 
 import os
 import os.path
+from datetime import datetime
 from typing import Optional
 
 from django.conf import settings
@@ -29,21 +30,27 @@ from django.db.models import Count, Value
 from django.db.models.functions import Replace
 from django.urls import reverse
 from django.utils.functional import cached_property
+from django.utils.timezone import make_aware
 from django.utils.translation import gettext_lazy
 
+from weblate.configuration.models import Setting
 from weblate.formats.models import FILE_FORMATS
 from weblate.lang.models import Language
 from weblate.memory.tasks import import_memory
 from weblate.trans.defines import PROJECT_NAME_LENGTH
 from weblate.trans.mixins import CacheKeyMixin, PathMixin, URLMixin
 from weblate.utils.data import data_dir
-from weblate.utils.db import FastDeleteModelMixin, FastDeleteQuerySetMixin
 from weblate.utils.site import get_site_url
 from weblate.utils.stats import ProjectStats
-from weblate.utils.validators import validate_language_aliases, validate_slug
+from weblate.utils.validators import (
+    validate_language_aliases,
+    validate_project_name,
+    validate_project_web,
+    validate_slug,
+)
 
 
-class ProjectQuerySet(FastDeleteQuerySetMixin, models.QuerySet):
+class ProjectQuerySet(models.QuerySet):
     def order(self):
         return self.order_by("name")
 
@@ -51,11 +58,10 @@ class ProjectQuerySet(FastDeleteQuerySetMixin, models.QuerySet):
 def prefetch_project_flags(projects):
     lookup = {project.id: project for project in projects}
     if lookup:
+        queryset = Project.objects.filter(id__in=lookup.keys()).values("id")
         # Indicate alerts
-        for alert in (
-            projects.values("id")
-            .filter(component__alert__dismissed=False)
-            .annotate(Count("component__alert"))
+        for alert in queryset.filter(component__alert__dismissed=False).annotate(
+            Count("component__alert")
         ):
             lookup[alert["id"]].__dict__["has_alerts"] = bool(
                 alert["component__alert__count"]
@@ -65,8 +71,7 @@ def prefetch_project_flags(projects):
             project.__dict__["locked"] = True
         # Filter unlocked projects
         for locks in (
-            projects.filter(component__locked=False)
-            .values("id")
+            queryset.filter(component__locked=False)
             .distinct()
             .annotate(Count("component__id"))
         ):
@@ -74,7 +79,7 @@ def prefetch_project_flags(projects):
     return projects
 
 
-class Project(FastDeleteModelMixin, models.Model, URLMixin, PathMixin, CacheKeyMixin):
+class Project(models.Model, URLMixin, PathMixin, CacheKeyMixin):
     ACCESS_PUBLIC = 0
     ACCESS_PROTECTED = 1
     ACCESS_PRIVATE = 100
@@ -92,6 +97,7 @@ class Project(FastDeleteModelMixin, models.Model, URLMixin, PathMixin, CacheKeyM
         max_length=PROJECT_NAME_LENGTH,
         unique=True,
         help_text=gettext_lazy("Display name"),
+        validators=[validate_project_name],
     )
     slug = models.SlugField(
         verbose_name=gettext_lazy("URL slug"),
@@ -104,6 +110,7 @@ class Project(FastDeleteModelMixin, models.Model, URLMixin, PathMixin, CacheKeyM
         verbose_name=gettext_lazy("Project website"),
         blank=not settings.WEBSITE_REQUIRED,
         help_text=gettext_lazy("Main website of translated project."),
+        validators=[validate_project_web],
     )
     instructions = models.TextField(
         verbose_name=gettext_lazy("Translation instructions"),
@@ -171,6 +178,8 @@ class Project(FastDeleteModelMixin, models.Model, URLMixin, PathMixin, CacheKeyM
         validators=[validate_language_aliases],
     )
 
+    machinery_settings = models.JSONField(default=dict, blank=True)
+
     is_lockable = True
     _reverse_url_name = "project"
 
@@ -323,16 +332,15 @@ class Project(FastDeleteModelMixin, models.Model, URLMixin, PathMixin, CacheKeyM
         """Check whether there are some not committed changes."""
         return self.count_pending_units > 0
 
-    def on_repo_components(self, default, call, *args, **kwargs):
+    def on_repo_components(self, use_all: bool, func: str, *args, **kwargs):
         """Wrapper for operations on repository."""
-        ret = default
-        for component in self.all_repo_components:
-            res = getattr(component, call)(*args, **kwargs)
-            if default:
-                ret = ret & res
-            else:
-                ret = ret | res
-        return ret
+        generator = (
+            getattr(component, func)(*args, **kwargs)
+            for component in self.all_repo_components
+        )
+        if use_all:
+            return all(generator)
+        return any(generator)
 
     def commit_pending(self, reason, user):
         """Commit any pending changes."""
@@ -364,6 +372,10 @@ class Project(FastDeleteModelMixin, models.Model, URLMixin, PathMixin, CacheKeyM
         """Push all Git repos."""
         return self.on_repo_components(True, "do_file_sync", request)
 
+    def has_push_configuration(self):
+        """Check whether any suprojects can push."""
+        return self.on_repo_components(False, "has_push_configuration")
+
     def can_push(self):
         """Check whether any suprojects can push."""
         return self.on_repo_components(False, "can_push")
@@ -439,9 +451,23 @@ class Project(FastDeleteModelMixin, models.Model, URLMixin, PathMixin, CacheKeyM
 
         return User.objects.all_admins(self).select_related("profile")
 
+    def get_child_components_access(self, user):
+        """
+        Lists child components.
+
+        This is slower than child_components, but allows additional
+        filtering on the result.
+        """
+        child_components = (
+            self.component_set.distinct() | self.shared_components.distinct()
+        )
+        return child_components.filter_access(user).order()
+
     @cached_property
     def child_components(self):
-        return self.component_set.distinct() | self.shared_components.distinct()
+        own = self.component_set.all()
+        shared = self.shared_components.all()
+        return own.union(shared)
 
     def scratch_create_component(
         self,
@@ -503,3 +529,38 @@ class Project(FastDeleteModelMixin, models.Model, URLMixin, PathMixin, CacheKeyM
             result = get_glossary_automaton(self)
             cache.set(self.glossary_automaton_key, result, 24 * 3600)
         return result
+
+    def get_machinery_settings(self):
+        settings = Setting.objects.get_settings_dict(Setting.CATEGORY_MT)
+        for item, value in self.machinery_settings.items():
+            if value is None:
+                if item in settings:
+                    del settings[item]
+            else:
+                settings[item] = value
+        return settings
+
+    def list_backups(self):
+        backup_dir = data_dir("projectbackups", f"{self.pk}")
+        result = []
+        if not os.path.exists(backup_dir):
+            return result
+        with os.scandir(backup_dir) as iterator:
+            for entry in iterator:
+                if not entry.name.endswith(".zip"):
+                    continue
+                result.append(
+                    {
+                        "name": entry.name,
+                        "path": os.path.join(backup_dir, entry.name),
+                        "timestamp": make_aware(
+                            datetime.fromtimestamp(int(entry.name.split(".")[0]))
+                        ),
+                        "size": entry.stat().st_size // 1024,
+                    }
+                )
+        return sorted(result, key=lambda item: item["timestamp"], reverse=True)
+
+    @cached_property
+    def enable_review(self):
+        return self.translation_review or self.source_review
