@@ -21,6 +21,7 @@ import codecs
 import os
 import tempfile
 from datetime import datetime
+from itertools import chain
 from typing import BinaryIO, Dict, List, Optional, Union
 
 from django.core.cache import cache
@@ -36,11 +37,15 @@ from weblate.checks.flags import Flags
 from weblate.checks.models import CHECKS
 from weblate.formats.auto import try_load
 from weblate.formats.base import UnitNotFound
-from weblate.formats.helpers import BytesIOMode
+from weblate.formats.helpers import CONTROLCHARS, BytesIOMode
 from weblate.lang.models import Language, Plural
 from weblate.trans.checklists import TranslationChecklist
 from weblate.trans.defines import FILENAME_LENGTH
-from weblate.trans.exceptions import FileParseError, PluralFormsMismatch
+from weblate.trans.exceptions import (
+    FailedCommitError,
+    FileParseError,
+    PluralFormsMismatch,
+)
 from weblate.trans.mixins import CacheKeyMixin, LoggerMixin, URLMixin
 from weblate.trans.models.change import Change
 from weblate.trans.models.suggestion import Suggestion
@@ -55,16 +60,16 @@ from weblate.trans.models.variant import Variant
 from weblate.trans.signals import component_post_update, store_post_load, vcs_pre_commit
 from weblate.trans.util import join_plural, split_plural
 from weblate.trans.validators import validate_check_flags
-from weblate.utils.db import FastDeleteModelMixin, FastDeleteQuerySetMixin
 from weblate.utils.errors import report_error
-from weblate.utils.hash import calculate_hash
 from weblate.utils.render import render_template
 from weblate.utils.site import get_site_url
 from weblate.utils.stats import GhostStats, TranslationStats
 
 
 class TranslationManager(models.Manager):
-    def check_sync(self, component, lang, code, path, force=False, request=None):
+    def check_sync(
+        self, component, lang, code, path, force=False, request=None, change=None
+    ):
         """Parse translation meta info and updates translation object."""
         translation = component.translation_set.get_or_create(
             language=lang,
@@ -84,12 +89,12 @@ class TranslationManager(models.Manager):
             force = True
             translation.check_flags = flags
             translation.save(update_fields=["check_flags"])
-        translation.check_sync(force, request=request)
+        translation.check_sync(force, request=request, change=change)
 
         return translation
 
 
-class TranslationQuerySet(FastDeleteQuerySetMixin, models.QuerySet):
+class TranslationQuerySet(models.QuerySet):
     def prefetch(self):
         from weblate.trans.models import Alert
 
@@ -123,9 +128,7 @@ class TranslationQuerySet(FastDeleteQuerySetMixin, models.QuerySet):
         )
 
 
-class Translation(
-    FastDeleteModelMixin, models.Model, URLMixin, LoggerMixin, CacheKeyMixin
-):
+class Translation(models.Model, URLMixin, LoggerMixin, CacheKeyMixin):
     component = models.ForeignKey("Component", on_delete=models.deletion.CASCADE)
     language = models.ForeignKey(Language, on_delete=models.deletion.CASCADE)
     plural = models.ForeignKey(Plural, on_delete=models.deletion.CASCADE)
@@ -148,7 +151,7 @@ class Translation(
 
     class Meta:
         app_label = "trans"
-        unique_together = ("component", "language")
+        unique_together = [("component", "language")]
         verbose_name = "translation"
         verbose_name_plural = "translations"
 
@@ -280,7 +283,9 @@ class Translation(
         if fileobj is None:
             fileobj = self.get_filename()
         elif self.is_template:
-            template = self.component.load_template_store(fileobj)
+            template = self.component.load_template_store(
+                BytesIOMode(fileobj.name, fileobj.read())
+            )
             fileobj.seek(0)
         store = self.component.file_format_cls.parse(
             fileobj,
@@ -385,7 +390,6 @@ class Translation(
             return
         details["reason"] = self.reason
 
-        self.log_info("processing %s, %s", self.filename, self.reason)
         self.component.check_template_valid()
 
         # List of updated units (used for cleanup and duplicates detection)
@@ -395,6 +399,18 @@ class Translation(
             store = self.store
             translation_store = None
 
+            try:
+                store_units = store.content_units
+            except ValueError as error:
+                raise FileParseError(str(error))
+
+            self.log_info(
+                "processing %s, %s, %d strings",
+                self.filename,
+                self.reason,
+                len(store_units),
+            )
+
             # Store plural
             plural = store.get_plural(self.language, store)
             if plural != self.plural:
@@ -414,13 +430,17 @@ class Translation(
             if self.component.intermediate:
                 translation_store = store
                 store = self.load_store(force_intermediate=True)
+                try:
+                    store_units = store.content_units
+                except ValueError as error:
+                    raise FileParseError(str(error))
 
-            for pos, unit in enumerate(store.content_units):
+            for pos, unit in enumerate(store_units):
                 # Use translation store if exists and if it contains the string
                 if translation_store is not None:
                     try:
                         translated_unit, created = translation_store.find_unit(
-                            unit.context
+                            unit.context, unit.source
                         )
                         if translated_unit and not created:
                             unit = translated_unit
@@ -430,7 +450,10 @@ class Translation(
                     except UnitNotFound:
                         pass
 
-                id_hash = unit.id_hash
+                try:
+                    id_hash = unit.id_hash
+                except Exception as error:
+                    self.component.handle_parse_error(error, self)
 
                 # Check for possible duplicate units
                 if id_hash in updated:
@@ -467,7 +490,7 @@ class Translation(
         # Delete stale units
         stale = set(dbunits) - set(updated)
         if stale:
-            self.log_debug("deleting %d stale strings", len(stale))
+            self.log_info("deleting %d stale strings", len(stale))
             self.unit_set.filter(id_hash__in=stale).delete()
             self.component.needs_cleanup = True
 
@@ -516,6 +539,9 @@ class Translation(
     def can_push(self):
         return self.component.can_push()
 
+    def has_push_configuration(self):
+        return self.component.has_push_configuration()
+
     def get_hash_filenames(self):
         """Return filenames to include in the hash."""
         component = self.component
@@ -545,65 +571,70 @@ class Translation(
         self.revision = self.get_git_blob_hash()
         self.save(update_fields=["revision"])
 
-    def get_last_author(self, email=False):
+    def get_last_author(self):
         """Return last author of change done in Weblate."""
         if not self.stats.last_author:
             return None
         from weblate.auth.models import User
 
-        return User.objects.get(pk=self.stats.last_author).get_author_name(email)
+        return User.objects.get(pk=self.stats.last_author).get_visible_name()
 
     @transaction.atomic
-    def commit_pending(
-        self, reason: str, user, skip_push: bool = False, signals: bool = True
-    ):
+    def commit_pending(self, reason: str, user, skip_push: bool = False):
         """Commit any pending changes."""
-        with self.component.repository.lock:
-            # Commit template first
-            if (
-                not self.is_source
-                and self.component.has_template()
-                and self.component.source_translation.needs_commit()
-            ):
-                self.component.source_translation.commit_pending(
-                    reason, user, skip_push=skip_push, signals=signals
-                )
+        if not self.needs_commit():
+            return False
+        return self.component.commit_pending(reason, user, skip_push=skip_push)
 
-            if not self.needs_commit():
-                return False
+    @transaction.atomic
+    def _commit_pending(self, reason: str, user):
+        """
+        Translation commit implementation.
 
-            self.log_info("committing pending changes (%s)", reason)
+        Assumptions:
 
-            try:
-                store = self.store
-            except FileParseError as error:
-                report_error(cause="Failed to parse file on commit")
-                self.log_error("skipping commit due to error: %s", error)
-                return False
+        - repository lock is held
+        - the source translation needs to be commited first
+        - signals and alerts are updated by the caller
+        - repository push is handled by the caller
+        """
+        self.log_info("committing pending changes (%s)", reason)
 
-            units = (
-                self.unit_set.filter(pending=True)
-                .prefetch_recent_content_changes()
-                .select_for_update()
-            )
+        try:
+            store = self.store
+        except FileParseError as error:
+            report_error(cause="Failed to parse file on commit")
+            self.log_error("skipping commit due to error: %s", error)
+            return False
 
-            for unit in units:
-                # We reuse the queryset, so pending units might reappear here
-                if not unit.pending:
-                    continue
+        try:
+            store.ensure_index()
+        except ValueError as error:
+            report_error(cause="Failed to parse file on commit")
+            self.log_error("skipping commit due to error: %s", error)
+            return False
 
-                # Get last change metadata
-                author, timestamp = unit.get_last_content_change()
+        units = (
+            self.unit_set.filter(pending=True)
+            .prefetch_recent_content_changes()
+            .select_for_update()
+        )
 
-                author_name = author.get_author_name()
+        for unit in units:
+            # We reuse the queryset, so pending units might reappear here
+            if not unit.pending:
+                continue
 
-                # Flush pending units for this author
-                self.update_units(units, store, author_name, author.id)
+            # Get last change metadata
+            author, timestamp = unit.get_last_content_change()
 
-                # Commit changes
-                self.git_commit(
-                    user, author_name, timestamp, skip_push=skip_push, signals=signals
-                )
+            author_name = author.get_author_name()
+
+            # Flush pending units for this author
+            self.update_units(units, store, author_name, author.id)
+
+            # Commit changes
+            self.git_commit(user, author_name, timestamp, skip_push=True, signals=False)
 
         # Update stats (the translated flag might have changed)
         self.invalidate_cache()
@@ -714,8 +745,6 @@ class Translation(
                     unit.delete()
                     continue
 
-                updated = True
-
                 # Optionally add unit to translation file.
                 # This has be done prior setting target as some formats
                 # generate content based on target language.
@@ -723,10 +752,17 @@ class Translation(
                     store.add_unit(pounit.unit)
 
                 # Store translations
-                if unit.is_plural:
-                    pounit.set_target(unit.get_target_plurals())
-                else:
-                    pounit.set_target(unit.target)
+                try:
+                    if unit.is_plural:
+                        pounit.set_target(unit.get_target_plurals())
+                    else:
+                        pounit.set_target(unit.target)
+                except Exception as error:
+                    self.component.handle_parse_error(error, self, reraise=False)
+                    report_error(cause="Failed to update unit")
+                    continue
+
+                updated = True
 
             # Update fuzzy/approved flag
             pounit.set_state(unit.state)
@@ -783,7 +819,9 @@ class Translation(
         # All strings
         result.add(self.stats, "all", "")
 
-        result.add_if(self.stats, "readonly", "success")
+        result.add_if(
+            self.stats, "readonly", "info" if self.enable_review else "success"
+        )
 
         if not self.is_readonly:
             if self.enable_review:
@@ -959,7 +997,13 @@ class Translation(
         filenames = []
         with component.repository.lock:
             # Commit pending changes
-            component.commit_pending("source update", request.user)
+            try:
+                component.commit_pending("source update", request.user)
+            except Exception as error:
+                raise FailedCommitError(
+                    _("Failed to commit pending changes: %s")
+                    % str(error).replace(self.component.full_path, "")
+                )
 
             # Create actual file with the uploaded content
             temp = tempfile.NamedTemporaryFile(
@@ -1032,10 +1076,16 @@ class Translation(
         fileobj.close()
         fileobj = BytesIOMode(fileobj.name, filecopy)
         with self.component.repository.lock:
-            if self.is_source:
-                self.component.commit_pending("replace file", request.user)
-            else:
-                self.commit_pending("replace file", request.user)
+            try:
+                if self.is_source:
+                    self.component.commit_pending("replace file", request.user)
+                else:
+                    self.commit_pending("replace file", request.user)
+            except Exception as error:
+                raise FailedCommitError(
+                    _("Failed to commit pending changes: %s")
+                    % str(error).replace(self.component.full_path, "")
+                )
             # This will throw an exception in case of error
             store2 = self.load_store(fileobj)
             store2.check_valid()
@@ -1072,9 +1122,11 @@ class Translation(
         else:
             existing = set(self.unit_set.values_list("context", "source"))
         for _set_fuzzy, unit in store.iterate_merge(fuzzy, only_translated=False):
-            if (has_template and unit.context in existing) or (
-                not has_template and (unit.context, unit.source) in existing
-            ):
+            if has_template:
+                idkey = unit.context
+            else:
+                idkey = (unit.context, unit.source)
+            if idkey in existing:
                 skipped += 1
                 continue
             self.add_unit(
@@ -1084,7 +1136,10 @@ class Translation(
                 split_plural(unit.target) if not self.is_source else [],
                 is_batch_update=True,
             )
+            existing.add(idkey)
             accepted += 1
+        self.was_new = accepted
+        self.notify_new(request)
         component.invalidate_cache()
         if component.needs_variants_update:
             component.update_variants()
@@ -1095,7 +1150,7 @@ class Translation(
         return (0, skipped, accepted, len(store.content_units))
 
     @transaction.atomic
-    def handle_upload(
+    def handle_upload(  # noqa: C901
         self,
         request,
         fileobj: BinaryIO,
@@ -1146,7 +1201,13 @@ class Translation(
 
             # Commit pending changes in template
             if component.has_template():
-                component.source_translation.commit_pending("upload", request.user)
+                try:
+                    component.commit_pending("upload", request.user)
+                except Exception as error:
+                    raise FailedCommitError(
+                        _("Failed to commit pending changes: %s")
+                        % str(error).replace(self.component.full_path, "")
+                    )
 
             # Load backend file
             if method == "add" and self.is_template:
@@ -1201,7 +1262,7 @@ class Translation(
     def invalidate_cache(self):
         """Invalidate any cached stats."""
         # Invalidate summary stats
-        if self._invalidate_scheduled and 0:
+        if self._invalidate_scheduled:
             return
         self._invalidate_scheduled = True
         transaction.on_commit(self._invalidate_triger)
@@ -1254,10 +1315,11 @@ class Translation(
         # delete_unit might do changes in the database only and not touch the files
         # for pending new units
         if self.is_source:
-            self.component.create_translations(request=request)
+            self.component.create_translations(request=request, change=change)
             self.component.invalidate_cache()
         else:
             self.check_sync(request=request, change=change)
+            self.notify_new(request)
             self.invalidate_cache()
         # Trigger post-update signal
         self.component.trigger_post_update(previous_revision, False)
@@ -1333,10 +1395,9 @@ class Translation(
                 current_target = ""
             if isinstance(current_target, list):
                 current_target = join_plural(current_target)
-            if has_template:
-                id_hash = calculate_hash(context)
-            else:
-                id_hash = calculate_hash(source, context)
+            id_hash = component.file_format_cls.unit_class.calculate_id_hash(
+                has_template, source, context
+            )
             # When adding to a target the source string can already exist
             unit = None
             if (skip_existing or not self.is_source) and is_source:
@@ -1410,8 +1471,21 @@ class Translation(
                 component.schedule_sync_terminology()
             component.invalidate_cache()
             component_post_update.send(sender=self.__class__, component=component)
+            self.was_new = 1
+            self.notify_new(request)
         return result
 
+    def notify_deletion(self, unit, user):
+        self.change_set.create(
+            action=Change.ACTION_STRING_REMOVE,
+            user=user,
+            target=unit.target,
+            details={
+                "source": unit.source,
+                "target": unit.target,
+            },
+        )
+
     @transaction.atomic
     def delete_unit(self, request, unit):
         from weblate.auth.models import get_anonymous
@@ -1421,6 +1495,7 @@ class Translation(
         with component.repository.lock:
             component.commit_pending("delete unit", user)
             previous_revision = self.component.repository.last_revision
+            cleanup_variants = False
             for translation in self.get_store_change_translations():
                 # Does unit exist here?
                 try:
@@ -1428,7 +1503,9 @@ class Translation(
                 except ObjectDoesNotExist:
                     continue
                 # Delete the removed unit from the database
+                cleanup_variants |= translation_unit.variant_id is not None
                 translation_unit.delete()
+                self.notify_deletion(translation_unit, user)
                 # Skip file processing on source language without a storage
                 if not self.filename:
                     continue
@@ -1454,6 +1531,7 @@ class Translation(
                     source_unit = translation_unit.source_unit
                     if source_unit.source_unit.unit_set.count() == 1:
                         source_unit.delete()
+                        source_unit.translation.notify_deletion(source_unit, user)
 
             if self.is_source and unit.position and not component.has_template():
                 # Adjust position is source language
@@ -1461,6 +1539,9 @@ class Translation(
                     position=F("position") - 1
                 )
 
+            if cleanup_variants:
+                self.component.update_variants()
+
             self.handle_store_change(request, user, previous_revision)
 
     @transaction.atomic
@@ -1468,6 +1549,7 @@ class Translation(
         if not self.is_source or not self.component.manage_units:
             return
         expected_count = self.component.translation_set.count()
+        self.was_new = 0
         for source in self.component.get_all_sources():
             # Is the string a terminology
             if "terminology" not in source.all_flags:
@@ -1484,6 +1566,8 @@ class Translation(
                 skip_existing=True,
                 sync_terminology=False,
             )
+            self.was_new += 1
+        self.notify_new(None)
 
     def validate_new_unit_data(  # noqa: C901
         self,
@@ -1497,6 +1581,11 @@ class Translation(
         extra = {}
         if isinstance(source, str):
             source = [source]
+        for text in chain(source, [context]):
+            if any(char in text for char in CONTROLCHARS):
+                raise ValidationError(
+                    _("String contains control character: %s") % repr(text)
+                )
         if context:
             self.component.file_format_cls.validate_context(context)
         if not self.component.has_template():
