@@ -49,11 +49,7 @@ from weblate.trans.util import (
 )
 from weblate.trans.validators import validate_check_flags
 from weblate.utils import messages
-from weblate.utils.db import (
-    FastDeleteModelMixin,
-    FastDeleteQuerySetMixin,
-    using_postgresql,
-)
+from weblate.utils.db import using_postgresql
 from weblate.utils.errors import report_error
 from weblate.utils.hash import calculate_hash, hash_to_checksum
 from weblate.utils.search import parse_query
@@ -83,7 +79,7 @@ SIMPLE_FILTERS = {
 NEWLINES = re.compile(r"\r\n|\r|\n")
 
 
-class UnitQuerySet(FastDeleteQuerySetMixin, models.QuerySet):
+class UnitQuerySet(models.QuerySet):
     def filter_type(self, rqtype):
         """Basic filtering based on unit state or failed checks."""
         if rqtype in SIMPLE_FILTERS:
@@ -296,7 +292,7 @@ class LabelsField(models.ManyToManyField):
                 through.filter(unit__source_unit=instance, label__name=label).delete()
 
 
-class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
+class Unit(models.Model, LoggerMixin):
 
     translation = models.ForeignKey("Translation", on_delete=models.deletion.CASCADE)
     id_hash = models.BigIntegerField()
@@ -354,7 +350,7 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
 
     class Meta:
         app_label = "trans"
-        unique_together = ("translation", "id_hash")
+        unique_together = [("translation", "id_hash")]
         verbose_name = "string"
         verbose_name_plural = "strings"
 
@@ -380,7 +376,11 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
         """Wrapper around save to run checks or update fulltext."""
         # Store number of words
         if not same_content or not self.num_words:
-            self.num_words = len(self.source_string.split())
+            self.num_words = sum(
+                len(s.split())
+                for s in self.get_source_plurals()
+                if not s.startswith("<unused singular")
+            )
             if update_fields and "num_words" not in update_fields:
                 update_fields.append("num_words")
 
@@ -602,7 +602,9 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
     def check_valid(texts):
         for text in texts:
             if any(char in text for char in CONTROLCHARS):
-                raise ValueError(f"String contains control char: {text!r}")
+                raise ValueError(
+                    gettext("String contains control character: %s") % repr(text)
+                )
 
     def update_source_unit(
         self, component, source, context, pos, note, location, flags
@@ -665,7 +667,7 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
             context = unit.context
             self.check_valid([context])
             note = unit.notes
-            previous_source = unit.previous_source
+            source_change = previous_source = unit.previous_source
         except DjangoDatabaseError:
             raise
         except Exception as error:
@@ -699,15 +701,17 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
             and same_target
         ):
             if not same_source and state in (STATE_TRANSLATED, STATE_APPROVED):
-                if self.previous_source == self.source and self.fuzzy:
+                if self.previous_source == source and self.fuzzy:
                     # Source change was reverted
+                    source_change = self.source
                     previous_source = ""
                     state = STATE_TRANSLATED
                 else:
                     # Store previous source and fuzzy flag for monolingual
                     if previous_source == "":
-                        previous_source = self.source
+                        source_change = previous_source = self.source
                     state = STATE_FUZZY
+                self.pending = True
             elif (
                 self.state == STATE_FUZZY
                 and state == STATE_FUZZY
@@ -765,7 +769,7 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
             return
 
         # Sanitize number of plurals
-        if self.is_plural:
+        if self.is_plural and not component.file_format_cls.has_multiple_strings:
             self.target = join_plural(self.get_target_plurals())
 
         if created:
@@ -781,12 +785,12 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
         if translation.is_template:
             component.updated_sources[self.id] = self
         # Indicate source string change
-        if not same_source and previous_source:
+        if not same_source and source_change:
             translation.update_changes.append(
                 Change(
                     unit=self,
                     action=Change.ACTION_SOURCE_CHANGE,
-                    old=previous_source,
+                    old=source_change,
                     target=self.source,
                 )
             )
@@ -804,6 +808,7 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
         # Update translation memory if needed
         if (
             self.state >= STATE_TRANSLATED
+            and self.target
             and (not translation.is_source or component.intermediate)
             and (created or not same_source or not same_target)
         ):
@@ -863,7 +868,7 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
         """
         plurals = self.get_source_plurals()
         singular = plurals[0]
-        if len(plurals) == 1 or "<unused singular" not in singular:
+        if len(plurals) == 1 or not singular.startswith("<unused singular"):
             return singular
         return plurals[1]
 
@@ -876,20 +881,21 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
         # Split plurals
         ret = split_plural(self.target)
 
-        if plurals is None:
-            plurals = self.translation.plural.number
+        if not self.translation.component.is_multivalue:
+            if plurals is None:
+                plurals = self.translation.plural.number
 
-        # Check if we have expected number of them
-        if len(ret) == plurals:
-            return ret
+            # Check if we have expected number of them
+            if len(ret) == plurals:
+                return ret
 
-        # Pad with empty translations
-        while len(ret) < plurals:
-            ret.append("")
+            # Pad with empty translations
+            while len(ret) < plurals:
+                ret.append("")
 
-        # Delete extra plurals
-        while len(ret) > plurals:
-            del ret[-1]
+            # Delete extra plurals
+            while len(ret) > plurals:
+                del ret[-1]
 
         return ret
 
@@ -1306,6 +1312,7 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
         propagate: bool = True,
         author=None,
         request=None,
+        add_alternative: bool = False,
     ):
         """
         Store new translation of a unit.
@@ -1326,6 +1333,13 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
         if not self.translation.is_template:
             new_target, self.fixups = fix_target(new_target, self)
 
+        # Handle managing alternative translations
+        if add_alternative:
+            new_target.append("")
+        elif component.is_multivalue:
+            new_target = [target for target in new_target if target]
+            if not new_target:
+                new_target = [""]
         # Update unit and save it
         self.target = join_plural(new_target)
         not_empty = bool(max(new_target))
@@ -1359,6 +1373,7 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
         if (
             user
             and self.target != self.old_unit["target"]
+            and self.target
             and self.state >= STATE_TRANSLATED
             and not component.is_glossary
         ):
@@ -1399,6 +1414,9 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
     def all_flags(self):
         return self.get_all_flags()
 
+    def get_unit_flags(self):
+        return Flags(self.extra_flags)
+
     @cached_property
     def edit_mode(self):
         """Returns syntax highlighting mode for Prismjs."""
@@ -1484,9 +1502,7 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
 
     @cached_property
     def content_hash(self):
-        if self.translation.component.template:
-            return calculate_hash(self.source, self.context)
-        return self.id_hash
+        return calculate_hash(self.source, self.context)
 
     @cached_property
     def recent_content_changes(self):
@@ -1536,7 +1552,7 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
         translation = self.translation
         component = translation.component
         result = []
-        if self.is_source or component.is_glossary:
+        if self.is_source:
             if "read-only" in flags:
                 if (
                     "read-only" not in translation.all_flags
@@ -1548,6 +1564,14 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
             else:
                 result.append(("addflag", "read-only", gettext("Mark as read-only")))
         if component.is_glossary:
+            if "read-only" in self.source_unit.get_unit_flags():
+                result.append(
+                    ("removeflag", "read-only", gettext("Unmark as untranslatable"))
+                )
+            else:
+                result.append(
+                    ("addflag", "read-only", gettext("Mark as untranslatable"))
+                )
             if "forbidden" in flags:
                 result.append(
                     (
@@ -1581,3 +1605,11 @@ class Unit(FastDeleteModelMixin, models.Model, LoggerMixin):
                     )
                 )
         return result
+
+    def invalidate_related_cache(self):
+        # Invalidate stats counts
+        self.translation.invalidate_cache()
+        # Invalidate unit cached properties
+        for key in ["all_comments", "suggestions"]:
+            if key in self.__dict__:
+                del self.__dict__[key]
