@@ -18,7 +18,7 @@
 #
 import os
 import time
-from datetime import date, timedelta
+from datetime import date, datetime, timedelta
 from glob import glob
 from typing import List, Optional
 
@@ -186,9 +186,13 @@ def cleanup_project(pk):
             continue
         with transaction.atomic():
             # Remove all units where there is just one referenced unit (self)
-            translation.unit_set.annotate(Count("unit")).filter(
-                unit__count__lte=1
-            ).delete()
+            deleted, details = (
+                translation.unit_set.annotate(Count("unit"))
+                .filter(unit__count__lte=1)
+                .delete()
+            )
+            if deleted:
+                translation.log_info("removed leaf units: %s", details)
 
 
 @app.task(trail=False)
@@ -329,20 +333,20 @@ def component_after_save(
 def component_removal(pk, uid):
     user = User.objects.get(pk=uid)
     try:
-        obj = Component.objects.get(pk=pk)
-        obj.acting_user = user
+        component = Component.objects.get(pk=pk)
+        component.acting_user = user
         Change.objects.create(
-            project=obj.project,
+            project=component.project,
             action=Change.ACTION_REMOVE_COMPONENT,
-            target=obj.slug,
+            target=component.slug,
             user=user,
             author=user,
         )
-        obj.delete()
-        if obj.allow_translation_propagation:
-            components = obj.project.component_set.filter(
+        component.delete()
+        if component.allow_translation_propagation:
+            components = component.project.component_set.filter(
                 allow_translation_propagation=True
-            ).exclude(pk=obj.pk)
+            ).exclude(pk=component.pk)
             for component_id in components.values_list("id", flat=True):
                 update_checks.delay(component_id)
     except Component.DoesNotExist:
@@ -353,11 +357,14 @@ def component_removal(pk, uid):
 def project_removal(pk, uid):
     user = User.objects.get(pk=uid)
     try:
-        obj = Project.objects.get(pk=pk)
+        project = Project.objects.get(pk=pk)
         Change.objects.create(
-            action=Change.ACTION_REMOVE_PROJECT, target=obj.slug, user=user, author=user
+            action=Change.ACTION_REMOVE_PROJECT,
+            target=project.slug,
+            user=user,
+            author=user,
         )
-        obj.delete()
+        project.delete()
     except Project.DoesNotExist:
         return
 
@@ -511,6 +518,51 @@ def daily_update_checks():
         update_checks.delay(component_id)
 
 
+@app.task(trail=False)
+def cleanup_project_backups():
+    # This intentionally does not use Project objects to remove stale backups
+    # for removed projects as well.
+    rootdir = data_dir("projectbackups")
+    backup_cutoff = datetime.now() - timedelta(days=settings.PROJECT_BACKUP_KEEP_DAYS)
+    for projectdir in glob(os.path.join(rootdir, "*")):
+        if not os.path.isdir(projectdir):
+            continue
+        if projectdir.endswith("import"):
+            # Keep imports for shorter time, but more of them
+            cutoff = datetime.now() - timedelta(days=1)
+            max_count = 30
+        else:
+            cutoff = backup_cutoff
+            max_count = settings.PROJECT_BACKUP_KEEP_COUNT
+        backups = sorted(
+            (
+                (
+                    path,
+                    datetime.fromtimestamp(int(path.split(".")[0])),
+                )
+                for path in os.listdir(projectdir)
+                if path.endswith(".zip") or path.endswith(".zip.part")
+            ),
+            key=lambda item: item[1],
+            reverse=True,
+        )
+        while len(backups) > max_count:
+            remove = backups.pop()
+            os.unlink(os.path.join(projectdir, remove[0]))
+
+        for backup in backups:
+            if backup[1] < cutoff:
+                os.unlink(os.path.join(projectdir, backup[0]))
+
+
+@app.task(trail=False)
+def create_project_backup(pk):
+    from weblate.trans.backups import ProjectBackup
+
+    project = Project.objects.get(pk=pk)
+    ProjectBackup().backup_project(project)
+
+
 @app.on_after_finalize.connect
 def setup_periodic_tasks(sender, **kwargs):
     sender.add_periodic_task(3600, commit_pending.s(), name="commit-pending")
@@ -534,3 +586,8 @@ def setup_periodic_tasks(sender, **kwargs):
     sender.add_periodic_task(
         3600 * 24, cleanup_old_comments.s(), name="cleanup-old-comments"
     )
+    sender.add_periodic_task(
+        crontab(hour=2, minute=30),
+        cleanup_project_backups.s(),
+        name="cleanup-project-backups",
+    )
