@@ -29,7 +29,7 @@ from django.template.loader import render_to_string
 from django.urls import reverse
 from django.utils import timezone
 from django.utils.formats import number_format as django_number_format
-from django.utils.html import escape, urlize
+from django.utils.html import escape, format_html, format_html_join, urlize
 from django.utils.safestring import mark_safe
 from django.utils.translation import gettext, gettext_lazy, ngettext, pgettext
 from siphashc import siphash
@@ -60,12 +60,13 @@ from weblate.utils.views import SORT_CHOICES
 register = template.Library()
 
 HIGHLIGTH_SPACE = '<span class="hlspace">{}</span>{}'
-SPACE_TEMPLATE = '<span class="{}"><span class="sr-only">{}</span></span>'
+SPACE_TEMPLATE = '<span class="{}">{}</span>'
 SPACE_SPACE = SPACE_TEMPLATE.format("space-space", " ")
 SPACE_NL = HIGHLIGTH_SPACE.format(SPACE_TEMPLATE.format("space-nl", ""), "<br />")
-SPACE_START = '<span class="hlspace"><span class="space-space"><span class="sr-only">'
-SPACE_MIDDLE = '</span></span><span class="space-space"><span class="sr-only">'
-SPACE_END = "</span></span></span>"
+SPACE_START = '<span class="hlspace"><span class="space-space">'
+SPACE_MIDDLE_1 = "</span>"
+SPACE_MIDDLE_2 = '<span class="space-space">'
+SPACE_END = "</span></span>"
 
 GLOSSARY_TEMPLATE = """<span class="glossary-term" title="{}">"""
 
@@ -87,10 +88,11 @@ NAME_MAPPING = {
 
 FLAG_TEMPLATE = '<span title="{0}" class="{1}">{2}</span>'
 
-SOURCE_LINK = """
-<a href="{0}" target="_blank" rel="noopener noreferrer"
-    class="wrap-text" dir="ltr">{1}</a>
-"""
+SOURCE_LINK = (
+    '<a href="{0}" target="_blank" rel="noopener noreferrer"'
+    ' class="{2}" dir="ltr">{1}</a>'
+)
+HLCHECK = '<span class="hlcheck" data-value="{}"><span class="highlight-number"></span>'
 
 
 class Formatter:
@@ -108,8 +110,6 @@ class Formatter:
         self.dmp = diff_match_patch()
 
     def parse(self):
-        if self.diff:
-            self.parse_diff()
         if self.unit:
             self.parse_highlight()
         if self.terms:
@@ -117,8 +117,10 @@ class Formatter:
         if self.search_match:
             self.parse_search()
         self.parse_whitespace()
+        if self.diff:
+            self.parse_diff()
 
-    def parse_diff(self):
+    def parse_diff(self):  # noqa: C901
         """Highlights diff, including extra whitespace."""
         dmp = self.dmp
         diff = dmp.diff_main(self.diff[self.idx], self.value)
@@ -132,9 +134,44 @@ class Formatter:
                 formatter.parse()
                 self.tags[offset].append(f"<del>{formatter.format()}</del>")
             elif op == dmp.DIFF_INSERT:
-                self.tags[offset].append("<ins>")
+
+                # Rearrange space highlighting
+                move_space = False
+                start_space = -1
+                for pos, tag in enumerate(self.tags[offset]):
+                    if tag == SPACE_MIDDLE_2:
+                        self.tags[offset][pos] = SPACE_MIDDLE_1
+                        move_space = True
+                        break
+                    if tag == SPACE_START:
+                        start_space = pos
+                        break
+
+                if start_space != -1:
+                    self.tags[offset].insert(start_space, "<ins>")
+                    last_middle = None
+                    for i in range(len(data)):
+                        tagoffset = offset + i + 1
+                        for pos, tag in enumerate(self.tags[tagoffset]):
+                            if tag == SPACE_END:
+                                # Whitespace ends within <ins>
+                                start_space = -1
+                                break
+                            if tag == SPACE_MIDDLE_2:
+                                last_middle = (tagoffset, pos)
+                        if start_space == -1:
+                            break
+                    if start_space != -1 and last_middle is not None:
+                        self.tags[tagoffset][pos] = SPACE_MIDDLE_1
+
+                else:
+                    self.tags[offset].append("<ins>")
+                if move_space:
+                    self.tags[offset].append(SPACE_START)
                 offset += len(data)
-                self.tags[offset].insert(0, "</ins>")
+                self.tags[offset].append("</ins>")
+                if start_space != -1:
+                    self.tags[offset].append(SPACE_START)
             elif op == dmp.DIFF_EQUAL:
                 offset += len(data)
 
@@ -142,10 +179,8 @@ class Formatter:
         """Highlights unit placeables."""
         highlights = highlight_string(self.value, self.unit)
         cleaned_value = list(self.value)
-        for start, end, _content in highlights:
-            self.tags[start].append(
-                '<span class="hlcheck"><span class="highlight-number"></span>'
-            )
+        for start, end, content in highlights:
+            self.tags[start].append(format_html(HLCHECK, content))
             self.tags[end].insert(0, "</span>")
             cleaned_value[start:end] = [" "] * (end - start)
 
@@ -208,7 +243,8 @@ class Formatter:
         for match in MULTISPACE_RE.finditer(self.value):
             self.tags[match.start()].append(SPACE_START)
             for i in range(match.start() + 1, match.end()):
-                self.tags[i].insert(0, SPACE_MIDDLE)
+                self.tags[i].insert(0, SPACE_MIDDLE_1)
+                self.tags[i].append(SPACE_MIDDLE_2)
             self.tags[match.end()].insert(0, SPACE_END)
 
         for match in WHITESPACE_RE.finditer(self.value):
@@ -219,22 +255,25 @@ class Formatter:
                 cls = "space-space"
             title = get_display_char(whitespace)[0]
             self.tags[match.start()].append(
-                '<span class="hlspace">'
-                f'<span class="{cls}" title="{title}">'
-                '<span class="sr-only">'
+                f'<span class="hlspace"><span class="{cls}" title="{title}">'
             )
-            self.tags[match.end()].insert(0, "</span></span></span>")
+            self.tags[match.end()].insert(0, "</span></span>")
 
     def format(self):
         tags = self.tags
         value = self.value
-        newline = SPACE_NL.format(gettext("New line"))
+        newline = format_html(SPACE_NL, gettext("New line"))
         output = []
         was_cr = False
         newlines = {"\r", "\n"}
         for pos, char in enumerate(value):
             # Special case for single whitespace char in diff
-            if char == " " and "<ins>" in tags[pos] and "</ins>" in tags[pos + 1]:
+            if (
+                char == " "
+                and "<ins>" in tags[pos]
+                and SPACE_START not in tags[pos]
+                and "</ins>" in tags[pos + 1]
+            ):
                 tags[pos].append(SPACE_START)
                 tags[pos + 1].insert(0, SPACE_END)
 
@@ -271,12 +310,13 @@ def format_translation(
     """Nicely formats translation text possibly handling plurals or diff."""
     # Split plurals to separate strings
     plurals = split_plural(value)
+    is_multivalue = unit is not None and unit.translation.component.is_multivalue
 
     if plural is None:
         plural = language.plural
 
     # Show plurals?
-    if int(num_plurals) <= 1:
+    if int(num_plurals) <= 1 and not is_multivalue:
         plurals = plurals[-1:]
 
     # Split diff plurals
@@ -300,7 +340,7 @@ def format_translation(
 
         # Show label for plural (if there are any)
         title = ""
-        if len(plurals) > 1:
+        if len(plurals) > 1 and not is_multivalue:
             title = plural.get_plural_name(idx)
 
         # Join paragraphs
@@ -514,10 +554,8 @@ def naturaltime(value, now=None):
         text = naturaltime_past(value, now)
     else:
         text = naturaltime_future(value, now)
-    return mark_safe(
-        '<span title="{}">{}</span>'.format(
-            escape(value.replace(microsecond=0).isoformat()), escape(text)
-        )
+    return format_html(
+        '<span title="{}">{}</span>', value.replace(microsecond=0).isoformat(), text
     )
 
 
@@ -528,12 +566,13 @@ def get_stats(obj):
 
 
 def translation_progress_data(
-    total: int, readonly: int, approved: int, translated: int
+    total: int, readonly: int, approved: int, translated: int, has_review: bool
 ):
-    translated -= approved
-    if approved:
+    if has_review:
+        translated -= approved
         approved += readonly
         translated -= readonly
+
     bad = total - approved - translated
     return {
         "approved": f"{translation_percent(approved, total, False):.1f}",
@@ -550,6 +589,7 @@ def translation_progress(obj):
         stats.readonly,
         stats.approved,
         stats.translated - stats.translated_checks,
+        stats.has_review,
     )
 
 
@@ -561,6 +601,7 @@ def words_progress(obj):
         stats.readonly_words,
         stats.approved_words,
         stats.translated_words - stats.translated_checks_words,
+        stats.has_review,
     )
 
 
@@ -602,6 +643,27 @@ def unit_state_title(unit) -> str:
     return "; ".join(state)
 
 
+def try_linkify_filename(
+    text, filename: str, line: str, unit, profile, link_class: str = ""
+):
+    """
+    Attempt to convert `text` to a repo link to `filename:line`.
+
+    If the `text` is prefixed with http:// or https://, the
+    link will be an absolute link to the specified resource.
+    """
+    link = None
+    if re.search(r"^https?://", text):
+        link = text
+    elif profile:
+        link = unit.translation.component.get_repoweb_link(
+            filename, line, profile.editor_link
+        )
+    if link:
+        return format_html(SOURCE_LINK, link, text, link_class)
+    return text
+
+
 @register.simple_tag
 def get_location_links(profile, unit):
     """Generate links to source files where translation was used."""
@@ -621,16 +683,12 @@ def get_location_links(profile, unit):
 
     # Go through all locations separated by comma
     for location, filename, line in unit.get_locations():
-        link = None
-        if profile:
-            link = unit.translation.component.get_repoweb_link(
-                filename, line, profile.editor_link
-            )
-        if link is None:
-            ret.append(escape(location))
-        else:
-            ret.append(SOURCE_LINK.format(escape(link), escape(location)))
-    return mark_safe('\n<span class="divisor">•</span>\n'.join(ret))
+        ret.append(
+            try_linkify_filename(location, filename, line, unit, profile, "wrap-text")
+        )
+    return format_html_join(
+        format_html('\n<span class="divisor">•</span>\n'), "{}", ((v,) for v in ret)
+    )
 
 
 @register.simple_tag(takes_context=True)
@@ -655,19 +713,19 @@ def announcements(context, project=None, component=None, language=None):
             )
         )
 
-    return mark_safe("\n".join(ret))
+    return format_html_join("\n", "{}", ((v,) for v in ret))
 
 
 @register.simple_tag(takes_context=True)
 def active_tab(context, slug):
     active = "active" if slug == context["active_tab_slug"] else ""
-    return mark_safe(f'class="tab-pane {active}" id="{slug}"')
+    return format_html('class="tab-pane {}" id="{}"', active, slug)
 
 
 @register.simple_tag(takes_context=True)
 def active_link(context, slug):
     if slug == context["active_tab_slug"]:
-        return mark_safe('class="active"')
+        return format_html('class="active"')
     return ""
 
 
@@ -869,7 +927,7 @@ def indicate_alerts(context, obj):
 
 @register.filter(is_safe=True)
 def markdown(text):
-    return mark_safe(f'<div class="markdown">{render_markdown(text)}</div>')
+    return format_html('<div class="markdown">{}</div>', render_markdown(text))
 
 
 @register.filter
@@ -923,10 +981,12 @@ def number_format(number):
     format_string = "%s"
     if number > 99999999:
         number = number // 1000000
-        format_string = "%s M"
+        # Translators: Number format, in millions (mega)
+        format_string = gettext("%s M")
     elif number > 99999:
         number = number // 1000
-        format_string = "%s k"
+        # Translators: Number format, in thousands (kilo)
+        format_string = gettext("%s k")
     return format_string % django_number_format(number, force_grouping=True)
 
 
@@ -941,10 +1001,11 @@ def trend_format(number):
     number = abs(number)
     if number < 0.1:
         return "—"
-    return mark_safe(
-        '{}{} <span class="{}"></span>'.format(
-            prefix, escape(percent_format(number)), trend
-        )
+    return format_html(
+        '{}{} <span class="{}"></span>',
+        prefix,
+        percent_format(number),
+        trend,
     )
 
 
