@@ -19,11 +19,13 @@
 
 """Test for translation views."""
 
+
 import time
 from unittest import SkipTest
 
 from django.urls import reverse
 
+from weblate.addons.resx import ResxUpdateAddon
 from weblate.checks.models import Check
 from weblate.trans.models import Change, Component, Unit
 from weblate.trans.tests.test_views import ViewTestCase
@@ -209,12 +211,12 @@ class EditValidationTest(ViewTestCase):
     def test_edit_invalid(self):
         """Editing with invalid params."""
         response = self.edit()
-        self.assertContains(response, "Missing translated string!")
+        self.assertContains(response, "This field is required.")
 
     def test_suggest_invalid(self):
         """Suggesting with invalid params."""
         response = self.edit(suggest="1")
-        self.assertContains(response, "Missing translated string!")
+        self.assertContains(response, "This field is required.")
 
     def test_merge(self):
         """Merging with invalid parameter."""
@@ -286,11 +288,27 @@ class EditResourceTest(EditTest):
             self.component.commit_pending("test", None)
         self.assertEqual(Unit.objects.filter(pending=True).count(), 0)
         self.assertEqual(Unit.objects.filter(context="key").count(), 2)
+        self.assertEqual(
+            Unit.objects.filter(context="key", state=STATE_TRANSLATED).count(), 2
+        )
+        self.component.create_translations(force=True)
+        self.assertEqual(
+            Unit.objects.filter(context="key", state=STATE_TRANSLATED).count(), 2
+        )
 
     def test_new_unit_translate_commit_translation(self, commit_translation=False):
         self.test_new_unit_translate(commit_translation=True)
 
 
+class EditResxTest(EditTest):
+    has_plurals = False
+
+    def create_component(self):
+        component = self.create_resx()
+        ResxUpdateAddon.create(component)
+        return component
+
+
 class EditLanguageTest(EditTest):
     """Language wide editing tests."""
 
@@ -922,7 +940,7 @@ class EditComplexTest(ViewTestCase):
             "Hello, world!\n", "Nazdar svete!\n", contentsum="aaa"
         )
         # We should get an error message
-        self.assertContains(response, "Source string has been changed meanwhile")
+        self.assertContains(response, "The source string has changed meanwhile.")
         self.assert_backend(0)
 
     def test_edit_changed_translation(self):
@@ -932,7 +950,7 @@ class EditComplexTest(ViewTestCase):
         )
         # We should get an error message
         self.assertContains(
-            response, "Translation of the string has been changed meanwhile"
+            response, "The translation of the string has changed meanwhile."
         )
         self.assert_backend(0)
 
