@@ -85,6 +85,7 @@ class ImportTest(ImportBaseTest):
     """Testing of file imports."""
 
     test_file = TEST_PO
+    has_plurals = True
 
     def test_import_normal(self):
         """Test importing normally."""
@@ -172,11 +173,15 @@ class ImportTest(ImportBaseTest):
         self.assertEqual(translation.stats.translated, 0)
         self.assertEqual(translation.stats.fuzzy, 0)
         self.assertEqual(translation.stats.all, 4)
-        self.assertEqual(translation.stats.suggestions, 1)
+        self.assertEqual(
+            translation.stats.suggestions, 2 if self.test_file == TEST_XLIFF else 1
+        )
 
     def test_import_xliff(self):
         response = self.do_import(test_file=TEST_XLIFF, follow=True)
-        self.assertContains(response, "updated: 1")
+        self.assertContains(
+            response, "updated: 2" if self.has_plurals else "updated: 1"
+        )
         # Verify stats
         translation = self.get_translation()
         self.assertEqual(translation.stats.translated, 1)
@@ -277,26 +282,62 @@ class ImportMoPoTest(ImportTest):
 
 
 class ImportJoomlaTest(ImportTest):
+    has_plurals = False
+
     def create_component(self):
         return self.create_joomla()
 
 
+class ImportCSVTest(ImportTest):
+    has_plurals = False
+
+    def create_component(self):
+        return self.create_csv_mono()
+
+    def test_import_source(self):
+        kwargs = self.kw_translation.copy()
+        kwargs["lang"] = "en"
+
+        with open(TEST_CSV, "rb") as handle:
+            response = self.client.post(
+                reverse("upload_translation", kwargs=kwargs),
+                {
+                    "file": handle,
+                    "method": "replace",
+                    "author_name": self.user.full_name,
+                    "author_email": self.user.email,
+                },
+                follow=True,
+            )
+        self.assertRedirects(response, reverse("translation", kwargs=kwargs))
+        messages = list(response.context["messages"])
+        self.assertIn("Processed 1 string from the uploaded files", messages[0].message)
+
+
 class ImportJSONTest(ImportTest):
+    has_plurals = False
+
     def create_component(self):
         return self.create_json()
 
 
 class ImportJSONMonoTest(ImportTest):
+    has_plurals = False
+
     def create_component(self):
         return self.create_json_mono()
 
 
 class ImportPHPMonoTest(ImportTest):
+    has_plurals = False
+
     def create_component(self):
         return self.create_php_mono()
 
 
 class StringsImportTest(ImportTest):
+    has_plurals = False
+
     def create_component(self):
         return self.create_iphone()
 
