@@ -22,6 +22,7 @@ import datetime
 
 from django.test import SimpleTestCase, TestCase
 from django.utils import timezone
+from django.utils.html import format_html
 
 from weblate.accounts.models import Profile
 from weblate.checks.tests.test_checks import MockLanguage, MockUnit
@@ -75,8 +76,10 @@ class NaturalTimeTest(SimpleTestCase):
         for diff, expected in TEST_DATA:
             testdate = now + datetime.timedelta(seconds=diff)
             result = naturaltime(testdate, now)
-            expected = '<span title="{}">{}</span>'.format(
-                testdate.replace(microsecond=0).isoformat(), expected
+            expected = format_html(
+                '<span title="{}">{}</span>',
+                testdate.replace(microsecond=0).isoformat(),
+                expected,
             )
             self.assertEqual(
                 expected,
@@ -94,9 +97,11 @@ class LocationLinksTest(TestCase):
                     source_language=Language(),
                     slug="c",
                     name="c",
+                    pk=-1,
                 ),
                 language=Language(),
-            )
+            ),
+            pk=-1,
         )
         self.unit.source_unit = self.unit
         self.profile = Profile()
@@ -192,6 +197,42 @@ class LocationLinksTest(TestCase):
             """,
         )
 
+    def test_absolute_url(self):
+        self.unit.translation.component.repoweb = (
+            "http://example.net/{{filename}}#L{{line}}"
+        )
+        self.unit.location = (
+            "foo.bar:123,bar.foo:321,https://example.com/foo,http://example.org/bar"
+        )
+        self.assertHTMLEqual(
+            get_location_links(self.profile, self.unit),
+            """
+            <a class="wrap-text"
+                href="http://example.net/foo.bar#L123" target="_blank"
+                dir="ltr" rel="noopener noreferrer">
+            foo.bar:123
+            </a>
+            <span class="divisor">•</span>
+            <a class="wrap-text"
+                href="http://example.net/bar.foo#L321" target="_blank"
+                dir="ltr" rel="noopener noreferrer">
+            bar.foo:321
+            </a>
+            <span class="divisor">•</span>
+            <a class="wrap-text"
+                href="https://example.com/foo" target="_blank"
+                dir="ltr" rel="noopener noreferrer">
+            https://example.com/foo
+            </a>
+            <span class="divisor">•</span>
+            <a class="wrap-text"
+                href="http://example.org/bar" target="_blank"
+                dir="ltr" rel="noopener noreferrer">
+            http://example.org/bar
+            </a>
+            """,
+        )
+
 
 class TranslationFormatTestCase(FixtureTestCase):
     def setUp(self):
@@ -225,7 +266,7 @@ class TranslationFormatTestCase(FixtureTestCase):
             Hello
             <del>
                 <span class="hlspace">
-                    <span class="space-space"><span class="sr-only"> </span></span>
+                    <span class="space-space"> </span>
                 </span>
             </del>
             world
@@ -242,7 +283,7 @@ class TranslationFormatTestCase(FixtureTestCase):
             """Hello
             <del>
                 <span class="hlspace">
-                    <span class="space-space"><span class="sr-only"> </span></span>
+                    <span class="space-space"> </span>
                 </span>
             </del>
             world
@@ -257,7 +298,29 @@ class TranslationFormatTestCase(FixtureTestCase):
             """Hello
             <ins>
                 <span class="hlspace">
-                    <span class="space-space"><span class="sr-only"> </span></span>
+                    <span class="space-space"> </span>
+                </span>
+            </ins>
+            world
+            """,
+        )
+
+    def test_diff_whitespace_changed(self):
+        self.assertHTMLEqual(
+            format_translation(
+                "Hello  world",
+                self.component.source_language,
+                diff="Hello world",
+            )["items"][0]["content"],
+            """Hello
+            <span class="hlspace">
+                <span class="space-space">
+                </span>
+            </span>
+            <ins>
+                <span class="hlspace">
+                    <span class="space-space">
+                    </span>
                 </span>
             </ins>
             world
@@ -274,18 +337,43 @@ class TranslationFormatTestCase(FixtureTestCase):
             """Hello
             <del>
                 <span class="hlspace">
-                    <span class="space-nl"><span class="sr-only"></span></span>
+                    <span class="space-nl"></span>
                 </span><br />
             </del>
             <ins>
                 <span class="hlspace">
-                    <span class="space-space"><span class="sr-only"> </span></span>
+                    <span class="space-space"> </span>
                 </span>
             </ins>
             world
             """,
         )
 
+    def test_diff_changed_whitespace(self):
+        self.assertHTMLEqual(
+            format_translation(
+                "     ${APP_NAME} is great",
+                self.component.source_language,
+                diff="    App is great",
+            )["items"][0]["content"],
+            """
+            <span class="hlspace">
+                <span class="space-space"> </span>
+                <span class="space-space"> </span>
+                <span class="space-space"> </span>
+                <span class="space-space"> </span>
+            </span>
+            <del>App</del>
+            <ins>
+                <span class="hlspace">
+                    <span class="space-space"> </span>
+                </span>
+                ${APP_NAME}
+            </ins>
+            is great
+            """,
+        )
+
     def test_glossary(self):
         self.assertHTMLEqual(
             format_translation(
@@ -331,10 +419,8 @@ class TranslationFormatTestCase(FixtureTestCase):
             text
             <span class="hlspace">
                 <span class="space-space">
-                    <span class="sr-only"> </span>
                 </span>
                 <span class="space-space">
-                    <span class="sr-only"> </span>
                 </span>
             </span>
             <span class="glossary-term"
@@ -382,7 +468,7 @@ class TranslationFormatTestCase(FixtureTestCase):
         )
 
     def test_glossary_format(self):
-        unit = Unit()
+        unit = Unit(translation=self.translation)
         unit.all_flags = {"php-format"}
         self.assertHTMLEqual(
             format_translation(
@@ -396,7 +482,7 @@ class TranslationFormatTestCase(FixtureTestCase):
                 unit=unit,
             )["items"][0]["content"],
             """
-            <span class="hlcheck">
+            <span class="hlcheck" data-value="%3$s">
             <span class="highlight-number"></span>
             %3$s
             </span>
@@ -412,9 +498,10 @@ class TranslationFormatTestCase(FixtureTestCase):
             ][0]["content"],
             """
             Orangutan has
-            <span class="hlcheck"><span class="highlight-number"></span>%d</span>
-            banana.<span class="hlspace"><span class="space-nl"><span class="sr-only">
+            <span class="hlcheck" data-value="%d">
+                <span class="highlight-number"></span>%d
             </span>
+            banana.<span class="hlspace"><span class="space-nl">
             </span>
             </span>
             <br/>
@@ -439,7 +526,6 @@ class TranslationFormatTestCase(FixtureTestCase):
             """
             <span class="hlspace">
                 <span class="space-space">
-                    <span class="sr-only"> </span>
                 </span>
             </span>
             Hello
@@ -453,10 +539,8 @@ class TranslationFormatTestCase(FixtureTestCase):
             """
             <span class="hlspace">
                 <span class="space-space">
-                    <span class="sr-only"> </span>
                 </span>
                 <span class="space-space">
-                    <span class="sr-only"> </span>
                 </span>
             </span>
             Hello
@@ -471,13 +555,10 @@ class TranslationFormatTestCase(FixtureTestCase):
             Hello
             <span class="hlspace">
                 <span class="space-space">
-                    <span class="sr-only"> </span>
                 </span>
                 <span class="space-space">
-                    <span class="sr-only"> </span>
                 </span>
                 <span class="space-space">
-                    <span class="sr-only"> </span>
                 </span>
             </span>
             world
@@ -490,8 +571,7 @@ class TranslationFormatTestCase(FixtureTestCase):
             """
             Hello
             world
-            <span class="hlspace"><span class="space-space"><span class="sr-only">
-            </span>
+            <span class="hlspace"><span class="space-space">
             </span>
             </span>
             """,
@@ -506,7 +586,7 @@ class TranslationFormatTestCase(FixtureTestCase):
             Hello
             <span class="hlspace">
                 <span class="space-space" title="NO-BREAK SPACE">
-                    <span class="sr-only">\u00A0</span>
+                    \u00A0
                 </span>
             </span>
             world
@@ -522,12 +602,10 @@ class TranslationFormatTestCase(FixtureTestCase):
             Hello
             <span class="hlspace">
                 <span class="space-nl">
-                    <span class="sr-only"> </span>
                 </span>
             </span><br>
             <span class="hlspace">
                 <span class="space-space">
-                    <span class="sr-only"> </span>
                 </span>
             </span>
             world
@@ -568,8 +646,6 @@ class DiffTestCase(SimpleTestCase):
             <del>old
              <span class="hlspace">
              <span class="space-space">
-             <span class="sr-only">
-             </span>
              </span>
             </del>
             text""",
@@ -601,7 +677,29 @@ class DiffTestCase(SimpleTestCase):
                 diff="Hello world! ",
             )["items"][0]["content"],
             'Hello world!<del><span class="hlspace"><span class="space-space">'
-            '<span class="sr-only"> </span></span></span></del>',
+            " </span></span></del>",
+        )
+
+    def test_format_diff_add_space(self):
+        unit = MockUnit(source="Hello.  World.")
+        self.assertHTMLEqual(
+            format_translation(
+                unit.source,
+                unit.translation.component.source_language,
+                diff="Hello. World.",
+            )["items"][0]["content"],
+            """
+            Hello.
+            <ins>
+                <span class="hlspace">
+                    <span class="space-space"></span>
+                </span>
+            </ins>
+            <span class="hlspace">
+                <span class="space-space"></span>
+            </span>
+            World.
+            """,
         )
 
     def test_format_entities(self):
