@@ -299,7 +299,7 @@ class TranslationManipulationTest(ViewTestCase):
     def test_model_add_duplicate(self):
         request = self.get_request()
         self.assertFalse(get_messages(request))
-        self.assertTrue(
+        self.assertIsNone(
             self.component.add_new_language(Language.objects.get(code="de"), request)
         )
         self.assertTrue(get_messages(request))
@@ -428,6 +428,16 @@ class BasicViewTest(ViewTestCase):
             status_code=301,
         )
 
+        # Non existing translated language should redirect with an info message
+        self.kw_component["lang"] = "Hindi"
+        response = self.client.get(reverse("translation", kwargs=self.kw_component))
+        self.kw_component.pop("lang")
+        self.assertRedirects(
+            response, reverse("component", kwargs=self.kw_component), status_code=302
+        )
+        messages = [m.message for m in get_messages(response.wsgi_request)]
+        self.assertIn("Hindi translation is currently not available", messages[0])
+
     def test_view_unit(self):
         unit = self.get_unit()
         response = self.client.get(unit.get_absolute_url())
