@@ -22,8 +22,8 @@ from itertools import chain
 
 from django.contrib.auth.decorators import login_required
 from django.core.exceptions import PermissionDenied
-from django.db.models import Prefetch
-from django.shortcuts import get_object_or_404, redirect
+from django.db.models import Count, Prefetch
+from django.shortcuts import redirect
 from django.template.loader import render_to_string
 from django.utils import timezone
 from django.utils.translation import gettext as _
@@ -32,10 +32,9 @@ from django.views.decorators.http import require_POST
 from weblate.accounts.models import AuditLog
 from weblate.accounts.utils import remove_user
 from weblate.auth.data import SELECTION_ALL
-from weblate.auth.forms import InviteUserForm, SimpleGroupForm, send_invitation
+from weblate.auth.forms import InviteUserForm, ProjectTeamForm, send_invitation
 from weblate.auth.models import Group, User
 from weblate.trans.forms import (
-    ProjectGroupDeleteForm,
     ProjectTokenCreateForm,
     ProjectUserGroupForm,
     UserBlockForm,
@@ -265,9 +264,11 @@ def manage_access(request, project):
     if not request.user.has_perm("project.permissions", obj):
         raise PermissionDenied()
 
-    groups = obj.defined_groups.order()
+    groups = (
+        obj.defined_groups.order().annotate(Count("user")).prefetch_related("languages")
+    )
     for group in groups:
-        group.edit_form = SimpleGroupForm(
+        group.edit_form = ProjectTeamForm(
             instance=group, auto_id=f"id_group_{group.id}_%s"
         )
     users = (
@@ -314,7 +315,7 @@ def manage_access(request, project):
             "blocked_users": obj.userblock_set.select_related("user"),
             "add_user_form": UserManageForm(),
             "create_project_token_form": ProjectTokenCreateForm(obj),
-            "create_team_form": SimpleGroupForm(
+            "create_team_form": ProjectTeamForm(
                 initial={"language_selection": SELECTION_ALL}
             ),
             "block_user_form": UserBlockForm(
@@ -351,46 +352,6 @@ def create_token(request, project):
     return redirect_param("manage-access", "#api", project=obj.slug)
 
 
-@require_POST
-@login_required
-def delete_group(request, project):
-    """Delete project group."""
-    obj = get_project(request, project)
-
-    if not request.user.has_perm("project.permissions", obj):
-        raise PermissionDenied()
-
-    form = ProjectGroupDeleteForm(obj, request.POST)
-
-    if form.is_valid():
-        form.cleaned_data["group"].delete()
-    else:
-        show_form_errors(request, form)
-
-    return redirect_param("manage-access", "#teams", project=obj.slug)
-
-
-@require_POST
-@login_required
-def edit_group(request, project, pk: int):
-    """Delete project group."""
-    obj = get_project(request, project)
-
-    if not request.user.has_perm("project.permissions", obj):
-        raise PermissionDenied()
-
-    group = get_object_or_404(obj.defined_groups.all(), pk=pk)
-
-    form = SimpleGroupForm(instance=group, data=request.POST)
-
-    if form.is_valid():
-        form.save()
-    else:
-        show_form_errors(request, form)
-
-    return redirect_param("manage-access", "#teams", project=obj.slug)
-
-
 @require_POST
 @login_required
 def create_group(request, project):
@@ -400,7 +361,7 @@ def create_group(request, project):
     if not request.user.has_perm("project.permissions", obj):
         raise PermissionDenied()
 
-    form = SimpleGroupForm(request.POST)
+    form = ProjectTeamForm(request.POST)
 
     if form.is_valid():
         form.save(project=obj)
