@@ -43,7 +43,6 @@ from weblate.checks.models import CHECKS, get_display_checks
 from weblate.glossary.forms import TermForm
 from weblate.glossary.models import get_glossary_terms
 from weblate.lang.models import Language
-from weblate.machinery import MACHINE_TRANSLATION_SERVICES
 from weblate.screenshots.forms import ScreenshotForm
 from weblate.trans.exceptions import FileParseError
 from weblate.trans.forms import (
@@ -60,7 +59,11 @@ from weblate.trans.forms import (
 )
 from weblate.trans.models import Change, Comment, Suggestion, Unit, Vote
 from weblate.trans.tasks import auto_translate
-from weblate.trans.templatetags.translations import unit_state_class, unit_state_title
+from weblate.trans.templatetags.translations import (
+    try_linkify_filename,
+    unit_state_class,
+    unit_state_title,
+)
 from weblate.trans.util import join_plural, redirect_next, render, split_plural
 from weblate.utils import messages
 from weblate.utils.antispam import is_spam
@@ -104,6 +107,7 @@ def get_other_units(unit):
         "matching": [],
         "context": [],
         "source": [],
+        "other": [],
     }
 
     allow_merge = False
@@ -116,15 +120,15 @@ def get_other_units(unit):
     if unit.source and unit.context:
         match = Q(source=unit.source) & Q(context=unit.context)
         if component.has_template():
-            query = Q(source=unit.source) | Q(context=unit.context)
+            query = Q(source__ilike=unit.source) | Q(context__ilike=unit.context)
         else:
-            query = Q(source=unit.source)
+            query = Q(source__ilike=unit.source)
     elif unit.source:
         match = Q(source=unit.source) & Q(context="")
-        query = Q(source=unit.source)
+        query = Q(source__ilike=unit.source)
     elif unit.context:
         match = Q(context=unit.context)
-        query = Q(context=unit.context)
+        query = Q(context__ilike=unit.context)
     else:
         return result
 
@@ -139,19 +143,33 @@ def get_other_units(unit):
                 When(condition=match, then=1), default=0, output_field=IntegerField()
             )
         )
+        .select_related(
+            "translation",
+            "translation__language",
+            "translation__plural",
+            "translation__component",
+            "translation__component__project",
+            "translation__component__source_language",
+        )
         .order_by("-matches_current")
     )
 
-    units_count = units.count()
+    max_units = 20
+    units_limited = units[:max_units]
+    units_count = len(units_limited)
 
     # Is it only this unit?
     if units_count == 1:
         return result
 
-    result["total"] = units_count
-    result["skipped"] = units_count > 20
+    if units_count == max_units:
+        # Get the real units count from the databse
+        units_count = units.count()
 
-    for item in units[:20]:
+    result["total"] = units_count
+    result["skipped"] = units_count > max_units
+
+    for item in units_limited:
         item.allow_merge = item.differently_translated = (
             item.translated and item.target != unit.target
         )
@@ -173,6 +191,8 @@ def get_other_units(unit):
             result["source"].append(item)
         elif item.context == unit.context:
             result["context"].append(item)
+        else:
+            result["other"].append(item)
 
     # Slightly different logic to allow applying current translation to
     # the propagated strings
@@ -188,7 +208,7 @@ def get_other_units(unit):
     return result
 
 
-def cleanup_session(session):
+def cleanup_session(session, delete_all: bool = False):
     """Delete old search results from session storage."""
     now = int(time.monotonic())
     keys = list(session.keys())
@@ -196,11 +216,18 @@ def cleanup_session(session):
         if not key.startswith("search_"):
             continue
         value = session[key]
-        if not isinstance(value, dict) or value["ttl"] < now or "items" not in value:
+        if (
+            delete_all
+            or not isinstance(value, dict)
+            or value["ttl"] < now
+            or "items" not in value
+        ):
             del session[key]
 
 
-def search(base, project, unit_set, request, blank: bool = False):
+def search(
+    base, project, unit_set, request, blank: bool = False, use_cache: bool = True
+):
     """Perform search or returns cached search results."""
     # Possible new search
     form = PositionSearchForm(user=request.user, data=request.GET, show_builder=False)
@@ -228,7 +255,8 @@ def search(base, project, unit_set, request, blank: bool = False):
     session_key = f"search_{base.cache_key}_{search_url}"
 
     if (
-        session_key in request.session
+        use_cache
+        and session_key in request.session
         and "offset" in request.GET
         and "items" in request.session[session_key]
     ):
@@ -259,7 +287,8 @@ def search(base, project, unit_set, request, blank: bool = False):
         "ids": unit_ids,
         "ttl": int(time.monotonic()) + 86400,
     }
-    request.session[session_key] = store_result
+    if use_cache:
+        request.session[session_key] = store_result
 
     search_result.update(store_result)
     return search_result
@@ -306,6 +335,8 @@ def perform_translation(unit, form, request):
     project = unit.translation.component.project
     # Remember old checks
     oldchecks = unit.all_checks_names
+    # Alernative translations handling
+    add_alternative = "add_alternative" in request.POST
 
     # Update explanation for glossary
     change_explanation = (
@@ -316,7 +347,11 @@ def perform_translation(unit, form, request):
         unit.explanation = form.cleaned_data["explanation"]
     # Save
     saved = unit.translate(
-        user, form.cleaned_data["target"], form.cleaned_data["state"], request=request
+        user,
+        form.cleaned_data["target"],
+        form.cleaned_data["state"],
+        request=request,
+        add_alternative=add_alternative,
     )
     # Make sure explanation is saved
     if not saved and change_explanation:
@@ -378,7 +413,7 @@ def perform_translation(unit, form, request):
         # Stay on same entry
         return False
 
-    return True
+    return True and not add_alternative
 
 
 @session_ratelimit_post("translate")
@@ -661,11 +696,24 @@ def translate(request, project, component, lang):  # noqa: C901
             ).order,
             "last_changes_url": urlencode(unit.translation.get_reverse_url_kwargs()),
             "display_checks": list(get_display_checks(unit)),
-            "machinery_services": json.dumps(list(MACHINE_TRANSLATION_SERVICES.keys())),
+            "comments_to_check": unit.unresolved_comments,
+            "machinery_services": json.dumps(
+                list(project.get_machinery_settings().keys())
+            ),
             "new_unit_form": get_new_unit_form(
                 unit.translation, user, initial={"variant": unit.pk}
             ),
             "screenshot_form": screenshot_form,
+            "translation_file_link": lambda: try_linkify_filename(
+                unit.translation.filename,
+                unit.translation.filename,
+                # '1' as a placeholder, because `get_repoweb_link` can't currently
+                # generate links without line specified. Although it's ok to use
+                # '' or '0' on GitHub or GitLab, let's play it safe for now.
+                "1",
+                unit,
+                user.profile,
+            ),
         },
     )
 
@@ -762,7 +810,7 @@ def delete_comment(request, pk):
 
     if "spam" in request.POST:
         comment_obj.report_spam()
-    comment_obj.delete()
+    comment_obj.delete(user=request.user)
     messages.info(request, _("Comment has been deleted."))
 
     return redirect_next(request.POST.get("next"), fallback_url)
@@ -774,13 +822,12 @@ def resolve_comment(request, pk):
     """Resolve comment."""
     comment_obj = get_object_or_404(Comment, pk=pk)
 
-    if not request.user.has_perm("comment.delete", comment_obj):
+    if not request.user.has_perm("comment.resolve", comment_obj):
         raise PermissionDenied()
 
     fallback_url = comment_obj.unit.get_absolute_url()
 
-    comment_obj.resolved = True
-    comment_obj.save(update_fields=["resolved"])
+    comment_obj.resolve(user=request.user)
     messages.info(request, _("Comment has been resolved."))
 
     return redirect_next(request.POST.get("next"), fallback_url)
@@ -966,13 +1013,15 @@ def delete_unit(request, unit_id):
         unit.translation.component.update_import_alerts(delete=False)
         messages.error(request, _("Failed to remove the string: %s") % error)
         return redirect(unit)
+    # Remove cached search results as we've just removed one of the unit there
+    cleanup_session(request.session, delete_all=True)
     return redirect(unit.translation)
 
 
 def browse(request, project, component, lang):
     """Strings browsing."""
     obj, project, unit_set = parse_params(request, project, component, lang)
-    search_result = search(obj, project, unit_set, request, blank=True)
+    search_result = search(obj, project, unit_set, request, blank=True, use_cache=False)
     offset = search_result["offset"]
     page = 20
     units = unit_set.prefetch_full().get_ordered(
