@@ -26,13 +26,12 @@ from django.utils.translation import ngettext
 from django.views.decorators.http import require_POST
 
 from weblate.lang.models import Language
-from weblate.trans.exceptions import PluralFormsMismatch
+from weblate.trans.exceptions import FailedCommitError, PluralFormsMismatch
 from weblate.trans.forms import DownloadForm, get_upload_form
 from weblate.trans.models import ComponentList, Translation
 from weblate.utils import messages
 from weblate.utils.data import data_dir
 from weblate.utils.errors import report_error
-from weblate.utils.lock import WeblateLockTimeout
 from weblate.utils.views import (
     download_translation_file,
     get_component,
@@ -50,7 +49,7 @@ def download_multi(translations, commit_objs, fmt=None, name="translations"):
     for obj in commit_objs:
         try:
             obj.commit_pending("download", None)
-        except WeblateLockTimeout:
+        except Exception:
             report_error(cause="Download commit")
 
     for translation in translations:
@@ -134,7 +133,7 @@ def download_translation(request, project, component, lang):
     kwargs = {}
 
     if "format" in request.GET or "q" in request.GET:
-        form = DownloadForm(request.GET)
+        form = DownloadForm(obj, request.GET)
         if not form.is_valid():
             show_form_errors(request, form)
             return redirect(obj)
@@ -209,6 +208,9 @@ def upload_translation(request, project, component, lang):
             request,
             _("Plural forms in the uploaded file do not match current translation."),
         )
+    except FailedCommitError as error:
+        messages.error(request, str(error))  # noqa: G200
+        report_error(cause="Upload error")
     except Exception as error:
         messages.error(
             request,
