@@ -30,10 +30,12 @@ import weblate.accounts.urls
 import weblate.accounts.views
 import weblate.addons.views
 import weblate.api.urls
+import weblate.auth.views
 import weblate.checks.views
 import weblate.fonts.views
 import weblate.glossary.views
 import weblate.lang.views
+import weblate.machinery.views
 import weblate.memory.views
 import weblate.screenshots.views
 import weblate.trans.views.about
@@ -82,6 +84,10 @@ handler500 = weblate.trans.views.error.server_error
 
 widget_pattern = "<word:widget>-<word:color>.<extension:extension>"
 
+URL_PREFIX = settings.URL_PREFIX
+if URL_PREFIX:
+    URL_PREFIX = URL_PREFIX.strip("/") + "/"
+
 real_patterns = [
     path("", weblate.trans.views.dashboard.home, name="home"),
     path("projects/", weblate.trans.views.basic.list_projects, name="projects"),
@@ -295,6 +301,16 @@ real_patterns = [
         weblate.trans.views.settings.change_component,
         name="settings",
     ),
+    path(
+        "backups/<name:project>/",
+        weblate.trans.views.settings.BackupsView.as_view(),
+        name="backups",
+    ),
+    path(
+        "backups/<name:project>/<name:backup>",
+        weblate.trans.views.settings.BackupsDownloadView.as_view(),
+        name="backups-download",
+    ),
     path(
         "labels/<name:project>/",
         weblate.trans.views.labels.project_labels,
@@ -330,6 +346,11 @@ real_patterns = [
         weblate.trans.views.create.CreateProject.as_view(),
         name="create-project",
     ),
+    path(
+        "create/project/import/",
+        weblate.trans.views.create.ImportProject.as_view(),
+        name="create-project-import",
+    ),
     path(
         "create/component/",
         weblate.trans.views.create.CreateComponentSelection.as_view(),
@@ -390,27 +411,17 @@ real_patterns = [
         weblate.trans.views.acl.set_groups,
         name="set-groups",
     ),
-    path(
-        "access/<name:project>/team/delete/",
-        weblate.trans.views.acl.delete_group,
-        name="delete-project-group",
-    ),
     path(
         "access/<name:project>/team/create/",
         weblate.trans.views.acl.create_group,
         name="create-project-group",
     ),
-    path(
-        "access/<name:project>/team/<int:pk>/",
-        weblate.trans.views.acl.edit_group,
-        name="edit-project-group",
-    ),
     path(
         "token/<name:project>/create/",
         weblate.trans.views.acl.create_token,
         name="create-project-token",
     ),
-    # Used by weblate.org to reder own activity chart on homepage
+    # Used by weblate.org to render own activity chart on homepage
     path(
         "activity/month.json",
         weblate.trans.views.charts.monthly_activity_json,
@@ -571,7 +582,7 @@ real_patterns = [
         weblate.trans.views.settings.remove_translation,
         name="remove_translation",
     ),
-    # Project renameing and moving
+    # Project renaming and moving
     path(
         "rename/<name:project>/",
         weblate.trans.views.settings.rename_project,
@@ -732,6 +743,27 @@ real_patterns = [
         weblate.memory.views.DownloadView.as_view(),
         name="memory-download",
     ),
+    # Machinery
+    path(
+        "manage/machinery/",
+        management_access(weblate.machinery.views.ListMachineryGlobalView.as_view()),
+        name="manage-machinery",
+    ),
+    path(
+        "manage/machinery/<name:machinery>/",
+        management_access(weblate.machinery.views.EditMachineryGlobalView.as_view()),
+        name="machinery-edit",
+    ),
+    path(
+        "machinery/<name:project>/",
+        weblate.machinery.views.ListMachineryProjectView.as_view(),
+        name="machinery-list",
+    ),
+    path(
+        "machinery/<name:project>/<name:machinery>/",
+        weblate.machinery.views.EditMachineryProjectView.as_view(),
+        name="machinery-edit",
+    ),
     # Languages browsing
     path("languages/", weblate.lang.views.show_languages, name="languages"),
     path(
@@ -848,9 +880,21 @@ real_patterns = [
         weblate.trans.views.widgets.widgets,
         name="widgets",
     ),
-    path("widgets/", RedirectView.as_view(url="/projects/", permanent=True)),
+    path(
+        "widgets/",
+        RedirectView.as_view(
+            url=f"/{URL_PREFIX}projects/",
+            permanent=True,
+        ),
+    ),
     # Data exports pages
-    path("data/", RedirectView.as_view(url="/projects/", permanent=True)),
+    path(
+        "data/",
+        RedirectView.as_view(
+            url=f"/{URL_PREFIX}projects/",
+            permanent=True,
+        ),
+    ),
     path(
         "data/<name:project>/",
         weblate.trans.views.basic.data_project,
@@ -884,12 +928,12 @@ real_patterns = [
     path("js/matomo/", weblate.trans.views.js.matomo, name="js-matomo"),
     path(
         "js/translate/<name:service>/<int:unit_id>/",
-        weblate.trans.views.js.translate,
+        weblate.machinery.views.translate,
         name="js-translate",
     ),
     path(
         "js/memory/<int:unit_id>/",
-        weblate.trans.views.js.memory,
+        weblate.machinery.views.memory,
         name="js-memory",
     ),
     path(
@@ -948,6 +992,16 @@ real_patterns = [
         weblate.wladmin.views.AdminUserList.as_view(),
         name="manage-users",
     ),
+    path(
+        "manage/teams/",
+        weblate.wladmin.views.TeamListView.as_view(),
+        name="manage-teams",
+    ),
+    path(
+        "teams/<int:pk>/",
+        weblate.auth.views.TeamUpdateView.as_view(),
+        name="team",
+    ),
     path(
         "manage/users/check/",
         weblate.wladmin.views.users_check,
@@ -1032,18 +1086,23 @@ real_patterns = [
     re_path(
         r"^(android-chrome|favicon)-(?P<size>192|512)x(?P=size)\.png$",
         RedirectView.as_view(
-            url=settings.STATIC_URL + "weblate-%(size)s.png", permanent=True
+            url=settings.STATIC_URL + "weblate-%(size)s.png",
+            permanent=True,
         ),
     ),
     path(
         "apple-touch-icon.png",
         RedirectView.as_view(
-            url=settings.STATIC_URL + "weblate-180.png", permanent=True
+            url=settings.STATIC_URL + "weblate-180.png",
+            permanent=True,
         ),
     ),
     path(
         "favicon.ico",
-        RedirectView.as_view(url=settings.STATIC_URL + "favicon.ico", permanent=True),
+        RedirectView.as_view(
+            url=settings.STATIC_URL + "favicon.ico",
+            permanent=True,
+        ),
     ),
     path(
         "robots.txt",
@@ -1067,6 +1126,11 @@ real_patterns = [
             )
         ),
     ),
+    # Redirects for .well-known
+    path(
+        ".well-known/change-password",
+        RedirectView.as_view(url=f"/{URL_PREFIX}accounts/password/", permanent=True),
+    ),
 ]
 
 # Billing integration
@@ -1095,7 +1159,7 @@ if "weblate.gitexport" in settings.INSTALLED_APPS:
         path(
             "projects/<name:project>/<name:component>/<gitpath:path>",
             RedirectView.as_view(
-                url="/git/%(project)s/%(component)s/%(path)s",
+                url=f"/{URL_PREFIX}git/%(project)s/%(component)s/%(path)s",
                 permanent=True,
                 query_string=True,
             ),
@@ -1103,7 +1167,7 @@ if "weblate.gitexport" in settings.INSTALLED_APPS:
         path(
             "projects/<name:project>/<name:component>.git/<gitpath:path>",
             RedirectView.as_view(
-                url="/git/%(project)s/%(component)s/%(path)s",
+                url=f"/{URL_PREFIX}git/%(project)s/%(component)s/%(path)s",
                 permanent=True,
                 query_string=True,
             ),
@@ -1112,7 +1176,7 @@ if "weblate.gitexport" in settings.INSTALLED_APPS:
         path(
             "git/<name:project>/<name:component>.git/<optionalpath:path>",
             RedirectView.as_view(
-                url="/git/%(project)s/%(component)s/%(path)s",
+                url=f"/{URL_PREFIX}git/%(project)s/%(component)s/%(path)s",
                 permanent=True,
                 query_string=True,
             ),
@@ -1121,7 +1185,7 @@ if "weblate.gitexport" in settings.INSTALLED_APPS:
         path(
             "projects/<name:project>/<name:component>/info/refs",
             RedirectView.as_view(
-                url="/git/%(project)s/%(component)s/%(path)s",
+                url=f"/{URL_PREFIX}git/%(project)s/%(component)s/%(path)s",
                 permanent=True,
                 query_string=True,
             ),
@@ -1184,7 +1248,7 @@ if "djangosaml2idp" in settings.INSTALLED_APPS:
     ]
 
 # Handle URL prefix configuration
-if not settings.URL_PREFIX:
+if not URL_PREFIX:
     urlpatterns = real_patterns
 else:
-    urlpatterns = [path(settings.URL_PREFIX.strip("/") + "/", include(real_patterns))]
+    urlpatterns = [path(URL_PREFIX, include(real_patterns))]
