@@ -81,7 +81,6 @@ DOC_LINKS = {
     "weblate.E013": ("admin/install", "production-email"),
     "weblate.E014": ("admin/install", "production-secret"),
     "weblate.E015": ("admin/install", "production-hosts"),
-    "weblate.E016": ("admin/install", "production-templates"),
     "weblate.E017": ("admin/install", "production-site"),
     "weblate.E018": ("admin/optionals", "avatars"),
     "weblate.E019": ("admin/install", "celery"),
@@ -105,6 +104,8 @@ DOC_LINKS = {
     "weblate.C036": ("admin/optionals", "gpg-sign"),
     "weblate.C037": ("admin/install", "production-database"),
     "weblate.C038": ("admin/install", "production-database"),
+    "weblate.W039": ("admin/machine",),
+    "weblate.W040": ("vcs",),
 }
 
 
@@ -212,9 +213,11 @@ def check_celery(app_configs, **kwargs):
                 )
             )
 
+        start = time.monotonic()
         result = ping.delay()
         try:
             pong = result.get(timeout=10, disable_sync_subtasks=False)
+            cache.set("celery_latency", round(1000 * (time.monotonic() - start)))
             current = ping()
             # Check for outdated Celery running different version of configuration
             if current != pong:
@@ -380,40 +383,16 @@ def check_settings(app_configs, **kwargs):
     return errors
 
 
-def check_templates(app_configs, **kwargs):
-    """Check for cached DjangoTemplates Loader."""
-    if settings.DEBUG:
-        return []
-
-    from django.template import engines
-    from django.template.backends.django import DjangoTemplates
-    from django.template.loaders import cached
-
-    is_cached = True
-
-    for engine in engines.all():
-        if not isinstance(engine, DjangoTemplates):
-            continue
-
-        for loader in engine.engine.template_loaders:
-            if not isinstance(loader, cached.Loader):
-                is_cached = False
-
-    if is_cached:
-        return []
-
-    return [
-        weblate_check(
-            "weblate.E016",
-            "Set up a cached template loader for better performance",
-            Error,
-        )
-    ]
-
-
 def check_data_writable(app_configs=None, **kwargs):
     """Check we can write to data dir."""
     errors = []
+    if not settings.DATA_DIR:
+        return [
+            weblate_check(
+                "weblate.E002",
+                "DATA_DIR is not configured.",
+            )
+        ]
     dirs = [
         settings.DATA_DIR,
         data_dir("home"),
@@ -443,6 +422,8 @@ def check_site(app_configs, **kwargs):
 
 def check_perms(app_configs=None, **kwargs):
     """Check that the data dir can be written to."""
+    if not settings.DATA_DIR:
+        return []
     start = time.monotonic()
     errors = []
     uid = os.getuid()
@@ -503,9 +484,10 @@ def check_encoding(app_configs=None, **kwargs):
 
 def check_diskspace(app_configs=None, **kwargs):
     """Check free disk space."""
-    stat = os.statvfs(settings.DATA_DIR)
-    if stat.f_bavail * stat.f_bsize < 10000000:
-        return [weblate_check("weblate.C032", "The disk is nearly full")]
+    if settings.DATA_DIR:
+        stat = os.statvfs(settings.DATA_DIR)
+        if stat.f_bavail * stat.f_bsize < 10000000:
+            return [weblate_check("weblate.C032", "The disk is nearly full")]
     return []
 
 
