@@ -81,10 +81,8 @@ def aggregate(stats, item, stats_obj):
         if stats_obj.last_changed and (not last or last < stats_obj.last_changed):
             stats["last_changed"] = stats_obj.last_changed
             stats["last_author"] = stats_obj.last_author
-    elif item == "last_author":
-        # Already handled above
-        return
-    else:
+    elif item != "last_author":
+        # The last_author is calculated with last_changed
         stats[item] += getattr(stats_obj, item)
 
 
@@ -670,10 +668,7 @@ class ComponentStats(LanguageStats):
 
     @cached_property
     def has_review(self):
-        return (
-            self._object.project.source_review
-            or self._object.project.translation_review
-        )
+        return self._object.enable_review
 
     @cached_property
     def lazy_translated_percent_key(self):
@@ -749,6 +744,14 @@ class ProjectLanguageComponent:
     def translation_set(self):
         return self.parent.translation_set
 
+    @property
+    def context_label(self):
+        return self.translation_set[0].component.context_label
+
+    @property
+    def source_language(self):
+        return self.translation_set[0].component.source_language
+
 
 class ProjectLanguage:
     """Wrapper class used in project-language listings and stats."""
@@ -804,13 +807,9 @@ class ProjectLanguage:
 
     @cached_property
     def translation_set(self):
-        result = (
-            self.language.translation_set.prefetch()
-            .filter(
-                Q(component__project=self.project) | Q(component__links=self.project)
-            )
-            .distinct()
-            .order_by("component__priority", "component__name")
+        all_langs = self.language.translation_set.prefetch()
+        result = all_langs.filter(component__project=self.project).union(
+            all_langs.filter(component__links=self.project)
         )
         for item in result:
             item.is_shared = (
@@ -818,7 +817,10 @@ class ProjectLanguage:
                 if item.component.project == self.project
                 else item.component.project
             )
-        return result
+        return sorted(
+            result,
+            key=lambda trans: (trans.component.priority, trans.component.name.lower()),
+        )
 
     @cached_property
     def is_source(self):
@@ -886,7 +888,7 @@ class ProjectStats(BaseStats):
 
     @cached_property
     def has_review(self):
-        return self._object.source_review or self._object.translation_review
+        return self._object.enable_review
 
     def get_invalidate_keys(
         self,
@@ -1041,6 +1043,9 @@ class GhostStats(BaseStats):
     def save(self):
         return
 
+    def load(self):
+        return {}
+
     def get_absolute_url(self):
         return None
 
