@@ -24,12 +24,12 @@ import os
 import os.path
 import subprocess
 from datetime import datetime
-from distutils.version import LooseVersion
 from typing import Iterator, List, Optional
 
 from dateutil import parser
 from django.core.cache import cache
 from django.utils.functional import cached_property
+from packaging.version import Version
 
 from weblate.trans.util import get_clean_env, path_separator
 from weblate.utils.errors import add_breadcrumb
@@ -379,9 +379,7 @@ class Repository:
             version = cls.get_version()
         except Exception:
             return False
-        return cls.req_version is None or LooseVersion(version) >= LooseVersion(
-            cls.req_version
-        )
+        return cls.req_version is None or Version(version) >= Version(cls.req_version)
 
     @classmethod
     def get_version(cls):
@@ -532,3 +530,7 @@ class Repository:
 
     def list_remote_branches(self):
         return []
+
+    @classmethod
+    def uses_deprecated_setting(cls) -> bool:
+        return False
