@@ -326,7 +326,7 @@ class GitRepository(Repository):
         if files:
             for name in files:
                 try:
-                    self.execute(["add", "--force", "--", name])
+                    self.execute(["add", "--force", "--", self.resolve_symlinks(name)])
                 except RepositoryException:
                     continue
         else:
@@ -787,6 +787,13 @@ class GitMergeRequestBase(GitForcePushRepository):
             "token": token,
         }
 
+    @classmethod
+    def uses_deprecated_setting(cls) -> bool:
+        return not getattr(settings, f"{cls.identifier.upper()}_CREDENTIALS") and (
+            getattr(settings, f"{cls.identifier.upper()}_USERNAME")
+            or getattr(settings, f"{cls.identifier.upper()}_TOKEN")
+        )
+
     @classmethod
     def is_configured(cls) -> bool:
         return getattr(settings, f"{cls.identifier.upper()}_USERNAME") or getattr(
