@@ -27,18 +27,20 @@ from django.core.mail import send_mail
 from django.db.models import Count, Q
 from django.http import HttpResponse
 from django.shortcuts import redirect, render
+from django.urls import reverse
 from django.utils import timezone
 from django.utils.decorators import method_decorator
-from django.utils.html import escape
-from django.utils.safestring import mark_safe
+from django.utils.html import escape, format_html
 from django.utils.translation import gettext as _
 from django.utils.translation import gettext_lazy
 from django.views.decorators.http import require_POST
+from django.views.generic import ListView
+from django.views.generic.edit import FormMixin
 
 from weblate.accounts.views import UserList
 from weblate.auth.decorators import management_access
-from weblate.auth.forms import AdminInviteUserForm
-from weblate.auth.models import User
+from weblate.auth.forms import AdminInviteUserForm, SitewideTeamForm
+from weblate.auth.models import Group, User
 from weblate.configuration.models import Setting
 from weblate.configuration.views import CustomCSSView
 from weblate.trans.forms import AnnouncementForm
@@ -80,8 +82,10 @@ MENU = (
     ("alerts", "manage-alerts", gettext_lazy("Alerts")),
     ("repos", "manage-repos", gettext_lazy("Repositories")),
     ("users", "manage-users", gettext_lazy("Users")),
+    ("teams", "manage-teams", gettext_lazy("Teams")),
     ("appearance", "manage-appearance", gettext_lazy("Appearance")),
     ("tools", "manage-tools", gettext_lazy("Tools")),
+    ("machinery", "manage-machinery", gettext_lazy("Automatic suggestions")),
 )
 if "weblate.billing" in settings.INSTALLED_APPS:
     MENU += (("billing", "manage-billing", gettext_lazy("Billing")),)
@@ -218,7 +222,7 @@ def backups(request):
             if form.is_valid():
                 form.save()
                 return redirect("manage-backups")
-        elif "remove" in request.POST:
+        elif "remove" in request.POST:  # noqa: R505
             service = BackupService.objects.get(pk=request.POST["service"])
             service.delete()
             return redirect("manage-backups")
@@ -272,6 +276,7 @@ def performance(request):
         "menu_page": "performance",
         "web_encoding": [sys.getfilesystemencoding(), sys.getdefaultencoding()],
         "celery_encoding": cache.get("celery_encoding"),
+        "celery_latency": cache.get("celery_latency"),
         "database_latency": measure_database_latency(),
         "cache_latency": measure_cache_latency(),
     }
@@ -352,12 +357,13 @@ class AdminUserList(UserList):
                 user = invite_form.save(request)
                 messages.success(
                     request,
-                    mark_safe(
-                        escape(_("Created user account %s."))
-                        % '<a href="{}">{}</a>'.format(
-                            escape(user.get_absolute_url()),
-                            escape(user.username),
-                        )
+                    format_html(
+                        escape(_("Created user account {}.")),
+                        format_html(
+                            '<a href="{}">{}</a>',
+                            user.get_absolute_url(),
+                            user.username,
+                        ),
                     ),
                 )
                 return redirect("manage-users")
@@ -492,3 +498,40 @@ def billing(request):
             "pending": pending,
         },
     )
+
+
+@method_decorator(management_access, name="dispatch")
+class TeamListView(FormMixin, ListView):
+    template_name = "manage/teams.html"
+    paginate_by = 50
+    model = Group
+    form_class = SitewideTeamForm
+
+    def get_queryset(self):
+        return (
+            super()
+            .get_queryset()
+            .prefetch_related("languages", "projects")
+            .filter(defining_project=None)
+            .annotate(Count("user"))
+            .order()
+        )
+
+    def get_context_data(self, **kwargs):
+        result = super().get_context_data(**kwargs)
+        result["menu_items"] = MENU
+        result["menu_page"] = "teams"
+        return result
+
+    def get_success_url(self):
+        return reverse("manage-teams")
+
+    def post(self, request, *args, **kwargs):
+        form = self.get_form()
+        if form.is_valid():
+            return self.form_valid(form)
+        return self.form_invalid(form)
+
+    def form_valid(self, form):
+        form.save()
+        return super().form_valid(form)
